/*************************************************************************/
/*                                                                       */
/*   Program Name :  data_double                                         */
/*                                                                       */
/*      Version:  2.00  2006-01-31                                       */
/*                                                                       */
/*      Copyright (c) 2004-2006 T.Kondo/NICT(CRL) All Right Reserved     */
/*                                                                       */
/*************************************************************************/
/*
 ho|ukah{[h擾f[^t@C̎Ԕ͈͂
 f[^𒊏oƂƂɃTvOg{ɂ

               Ver 0.00 2004.11.16
               Ver 1.01 2005.01.30
               Ver 1.02 2005.06.12  update due to update in common header name
               Ver 2.00 2006.01.30  support K5/VSSP32 format data

    点 data_double filename [soffset [period [outfile]]]
          filename -- f[^t@C
                            0 Ƃƃt@C
                           tds.datag
               soffset -- X^[gItZbgibj
                           ftHgl0
               period  -- of[^̎Ԓibj
                           ftHglׂ͂
               outfile -- f[^o̓t@C
                           ftHgfilename+'.dbl'

D:\IPVLBI\data\k5vssp32\NK5_32M1b4ch-Noise.dat.half
G:\mark5\k5-2bit.dat.half
*/
#define DATA_DOUBLE_VERSION "Ver. 2006-01-31"

#include "libk5.h"      // common header for IP-VLBI utilities
#include "libvssp32b.h"      // functions and type to handle K5/VSSP32 (#2)


int main(int argc, char *argv[])
{
       int sfreq, adbit, numch;
       long int smpl_count;    // sample number counts to extract
       long int counts;        //sample counter
       long int counts2;        //extract sample counter
       int nloop;
       int i,imax, m,k,n;
       int nerr;
       int smode,usampl;
       unsigned char *bbuf;
       //int nbyte=5000;
       unsigned short int *sbuf;
       unsigned int *ibuf;
       int bytesread;
       int obuf[1024];
       int nwrite=1024;
       int iwrite,opos;
       int bits, bytes, numb;
       unsigned int idat, idatw;
       int nch;
       double fsampl,tsampl,fact;
       int *idata;
       int bit_per_sampl,a,b;
       int ibcount;
       int hbytes; // header bytes
       //double soffset, fsec, period;
       int soffset,period;
       int soffset2;
       int extstart,hh,mm,ss;
       char fname[80],oname[80];
       char lab2[60], ltime1st[60];
       char labfsampl[10];
       char* dflt_fname="tds.data";
       char* dflt_oname="datacut.dat";
       ip_vlbi_file_t X;
       FILE *fo;
       IPBOARD_T ipb;

       if(argc == 1){
          printf("data_double %s\n",DATA_DOUBLE_VERSION);
          printf("Usage: data_double filename [soffset [period [outfile]]]\n");
          printf("\n");
          printf("         where  filename ---- Data file name\n");
          printf("                             if 0 is given, tds.data will be used\n");
          printf("                soffset --- Start time offset in sec (integer seconds)\n");
          printf("                             default is 0\n");         
          printf("                period  --- Extracting period in sec (integer seconds)\n");
          printf("                             default is all\n");         
          printf("                outfile ---- Extract data outfile name\n");
          printf("                             default is filename+\'.dbl\'\n");         
          exit(0);
       }

       /* data file name get */
       strcpy(fname,*(argv+1));
       if (fname[0]=='0'){
          strcpy(fname,dflt_fname);
       }
       X.fname=fname;

       /* default set */
       soffset=0;     
       period=0;
       sprintf(oname,"%s.dbl\0",fname);
       //strcpy(oname,dflt_oname);

       /* Start time offset get */
       if (argc>=3) soffset = atoi(*(argv + 2 ));

       /* Period to extract get */
       if (argc>=4) period = atoi(*(argv + 3 ));

       /* Out file name get */
       if (argc>=5) strcpy(oname,*(argv+4));

       /* Run condition monitor for debug*/
       //printf("Data File is %s\n",fname);
       //printf("nch, soffset, period  %d %f %f\n",nch,soffset,period);

       /* Program Name Display */
       printf("*****************************************************\n");
       printf("*  IP-VLBI DATA RE-SAMPLING (DOUBLE SPEED) PROGRAM  *\n");
       printf("*               (data_half)                         *\n");
       printf("*         %s    by  T.KONDO/NICT       *\n",DATA_DOUBLE_VERSION);
       printf("*****************************************************\n");
       printf("\n");

       /* Data file open */
       if( (X.stream  = fopen( X.fname, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",fname);
           goto exitend2;
       }

       /* output file create */
       if( (fo  = fopen( oname, "wb" )) == NULL )
       {
          printf("File  %s  Open Error! \n",oname);
          goto exitend3;
       } 

       /* Data header check to get sampling parameters */
       X.hd.format=0;  // for 1st call  2006-01-03
       if(checkheader(&X) < 0){
            goto exitend3;
       }

       /* parameter set */
        sfreq=X.hd.sfreq;
        adbit=X.hd.adbit;
        numch=X.hd.numch;
        fsampl=1000.0*(float)sfreq;   // sampling frequency (Hz)
        tsampl=1.0/fsampl;            // sampling period (sec)
        bits=((long)fsampl)*adbit*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X.fsample=(float)fsampl;

        bit_per_sampl=adbit*numch;
        /* Set extract Ch# to 1 in case of 1 ch board*/
        if(numch == 1) nch=1;   

        /* header parameters setup fot output */
        ipb.adbit=adbit;
        ipb.numch=numch;
        if(sfreq != 200){
            ipb.sfreq=sfreq*2;  //sampling frequency half
        } else {
            printf("Data sampling rate is 200kHz and unable to do double!\n");
            goto exitend3;
        }
        ipb.seconds=X.seconds;
        ip_hd_make_ini(&ipb,X.hd.psync2);



        /* File read buffer size set */
        smode=1;                         // low resolution mode

        //if(smode ==1){ usampl=20000; }
        if(smode ==1){ usampl=25600; }
        else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples

        //check numb is multiplier of 32 or not
        if((numb % 32) !=0){
            printf("Check numb!! %d %d\n",usampl,numb);
            goto exitend3;
        }

        //printf("imax numb  %d %d\n",imax,numb);    //debug


        bbuf = (unsigned char *)malloc(numb);
        idata = (int *)malloc(4*usampl*sizeof(int));


        sbuf=(unsigned short int *)bbuf;
        ibuf=(unsigned int *)bbuf;

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq < 1000){
            sprintf(labfsampl,"%dkHz\0",sfreq);
        } else {
            sprintf(labfsampl,"%dMHz\0",sfreq/1000);
        }

       /* Data extract start point set */ 
       //startoffset=(long int)soffset;
       //fsec=soffset-startoffset;
       //sofst_pos=(long int)(fsec/tsampl + 0.01);

       /* Data extraction period set */
       smpl_count=(long int)(period/tsampl + 0.01);

       /* Run condition display */

       //printf("startoffset, sofst_pos, smpl_count %d %d %d\n",startoffset,sofst_pos,smpl_count);
       printf("\n");
       printf("  Data File : %s",fname);
       if(X.hd.format == 1){
          printf(" (K5/VSSP format)\n");
       } else if (X.hd.format == 2){
          printf(" (K5/VSSP32 format)\n");
       } else {
          printf("\n");
       }
       printf("  Sampling Freq : %s\n",labfsampl);
       printf("  A/D bits      : %d\n",adbit);
       printf("  Number of CHs : %d\n",numch);
       printf("  Start Time(UT)        : %02d:%02d:%02d   Sec in Day = %d\n",
           X.hh,X.mm,X.ss,X.seconds);

       extstart=X.seconds+soffset;
       if(extstart >=86400) extstart-=86400;
       sec2hms(extstart, &hh,&mm,&ss);
       printf("  EXTRACT Start Time(UT): %02d:%02d:%02d   Sec in Day = %d\n",
           hh,mm,ss,extstart);

       /* foward startoffset seconds */
       printf("\n  %d seconds forwarding .....\n",soffset);
          /* actual forward is 3 sec shorter for safety */
       soffset2=soffset-3;
       //if(soffset2 > 0){
       //    if(fwdNsecHere(bbuf, soffset2, numb, imax, &X) <0){
       //           goto exitend;
       //    }
       //    nerr=checkheader(&X);
       //}
       if((nerr=fwdNsecHere(bbuf, soffset, numb, imax, &X)) <0){
               if(nerr == -99) goto exitend;
       }
       if(X.seconds != extstart){ 
           while(X.seconds < extstart) nerr=checkheader(&X);
       }

       printf("\n  Positioning finished\n");
       printf("  Extraction starting....\n");

       //printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",X.hh,X.mm,X.ss);
       strcpy(lab2,ltime1st);

       /* continue run info display */
       printf("\n");
       printf("  Start Offset (sec)       : %d\n",soffset);
       printf("  Start Time of Extraction : %02d:%02d:%02d\n",X.hh,X.mm,X.ss);
       printf("  Period (sec)             : %d\n",period);
       printf("  Out File : %s\n",oname);

       /* Data extraction start */
       counts=0;
       counts2=0;
       m=nch-1;
       nloop=0;
       ibcount=0;

       /* header data update */
       hbytes=8;
       if(X.hd.format == 2){  // for K5/VSSP32
           for(n=0; n<4; n++) ipb.hd[n+8]=X.hd.bhead[n+8];
           for(n=0; n<X.hd.aux_size; n++) ipb.hd[n+12]=X.hd.bhead[n+12];
           hbytes += (4+X.hd.aux_size);
       }
       ipb.seconds=X.seconds;
       ip_hd_make(&ipb);

       /* 1st header copy */
       //nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);
       opos=0;
       for(n=0; n < hbytes/4; n++){
          obuf[opos]=conv4byte2int(ipb.hd+n*4);
          opos++;
       }


       /* Data read loop start here */

loop1:
        if(nloop >0) {
           if(period > 0 && nloop >= period) goto exithere;
           if(checkheader(&X) < 0) goto exitend; 
            //printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
           //nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);
           ipb.seconds=X.seconds;
           ip_hd_make(&ipb);
           /* put K5 header data into obuf */
           for(n=0; n < hbytes/4; n++){
              obuf[opos]=conv4byte2int(ipb.hd+n*4);
              opos++;
              if(opos == nwrite){
                 //printf("Debug %d\n",nloop);
                 iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                 opos=0;
              }
           }
        }

        ibcount=0;
        idat=0;

        for(i=0; i< imax; i++)
        {
             bytesread = fread( bbuf, sizeof( char ), numb, X.stream );
             //printf("bytes read 2 %d\n",bytesread); //debug
             if( ferror( X.stream ) || bytesread==0 ) goto exithere; 
             if(bit_per_sampl == 1){
                 for(k=0; k<8*numb; k++){ // 
                     a=k/8;  // byte position
                     b= k % 8;
                     for(m=0; m<2; m++){
                         idat >>=1;
                         if(((bbuf[a] >> b) & 0x1) == 1){
                             idat |= 0x8000;
                         }
                         ibcount++;
                         if(ibcount==32){
                            obuf[opos]=idat;
                            opos++;
                            if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                                opos=0;
                            }
                            ibcount=0;
                            idat=0;
                         }
                     }
                 }
             } else if(bit_per_sampl == 2){
                 for(k=0; k<8*numb; k+=2){ // 
                     a=k/8;  // byte position
                     b= k % 8;
                     for(m=0; m<2; m++){
                         idat >>=2;
                         idatw=((unsigned int)((bbuf[a] >> b) & 0x3))<<30;
                         idat |= idatw;
                         ibcount+=2;
                         if(ibcount==32){
                            obuf[opos]=idat;
                            opos++;
                            if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                                opos=0;
                            }
                            ibcount=0;
                            idat=0;
                         }
                     }
                 }
             } else if(bit_per_sampl == 4){
                 for(k=0; k<8*numb; k+=4){ // 
                     a=k/8;  // byte position
                     b= k % 8;
                     for(m=0; m<2; m++){
                         idat >>=4;
                         idatw=((unsigned int)((bbuf[a] >> b) & 0xF))<<28;
                         idat |= idatw;
                         ibcount+=4;
                         if(ibcount==32){
                            obuf[opos]=idat;
                            opos++;
                            if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                                opos=0;
                            }
                            ibcount=0;
                            idat=0;
                         }
                     }
                 }
             } else if(bit_per_sampl == 8){
                 for(k=0; k<numb; k++){ // 
                     for(m=0; m<2; m++){
                         idat >>=8;
                         idatw=((unsigned int)bbuf[k] )<<24;
                         idat |= idatw;
                         ibcount+=8;
                         if(ibcount==32){
                            obuf[opos]=idat;
                            opos++;
                            if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                                opos=0;
                            }
                            ibcount=0;
                            idat=0;
                         }
                     }
                 }
             } else if(bit_per_sampl == 16){
                 for(k=0; k<numb/2; k++){ // 
                     for(m=0; m<2; m++){
                         idat >>=16;
                         idatw=((unsigned int)sbuf[k] )<<16;
                         idat |= idatw;
                         ibcount+=16;
                         if(ibcount==32){
                            obuf[opos]=idat;
                            opos++;
                            if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                                opos=0;
                            }
                            ibcount=0;
                            idat=0;
                         }
                     }
                 }
             } else if(bit_per_sampl == 32){
                 for(k=0; k<numb/4; k++){ // 
                     for(m=0; m<2; m++){
                         obuf[opos]=ibuf[k];
                         opos++;
                         if(opos == nwrite){
                                //printf("Debug %d\n",nloop);
                            iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                             opos=0;
                         }
                     }
                 }

             }

        }
        nloop++;
        goto loop1;
exithere:
       printf("\n");
       printf("Data Re-sampling (double speed) finished.\n");
       printf("Created file : %s\n",oname);
       i=fclose( fo ) ;

exitend: 
       
       free(bbuf);
       free(idata);

exitend3: 
       i=fclose( X.stream ) ;

 exitend2: 

       return 0;
}
