/*********************************************************************/
/*                                                                   */
/*   Program Name :  datachk32                                       */
/*                                                                   */
/*      Version:  0.0   2001-08-17                                   */
/*                2.10  2002-06-04                                   */
/*                2.11  2002-10-21  to allow extended file           */
/*                2.20  2004-08-12  add sampling statistics          */
/*                2.21  2005-06-12  minor update                     */
/*                2.30  2005-10-05  achive speed up by factor upto 3 */
/*                2.31  2005-10-07  more faster 100th times          */
/*                2.32  2005-10-11  a bit faster in statistics mode  */
/*                2.40  2006-01-02  support K5/VSSP32                */
/*                2.41  2006-02-01  minor debug                     */
/*                                                                   */
/*                                                                   */
/*      Data Check for PC-VSSP(IP-VLBI) board                        */
/*                                                                   */
/*                                                                   */
/*      Copyright (c) 2002-2006 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*
      usage
             datachk file_name [mode]
                  where   file_name -- K5/VSSP (IP-VLBI) data file name
                          mode     --  1: Display sampling statistics
                                       0: no statistics (default)

        ver 2.11 2002-10-21  support data file created under file extension mode
                             sampling
        ver 2.20 2004-08-12  add display mode that do statistics about sampling biases
        Ver 2.21 2005.06.12  update due to update in common header name
        Ver 2.30 2005.10.05  achieve speed up by factor 1.5 to 3
        Ver 2.31 2005.10.07  more faster by factor 100 or more for bit count
        Ver 2.32 2005.10.11  a bit faster in statistics mode
        Ver 2.40 2006.01.02  support K5/VSSP32
        Ver 2.41 2006.02.01  minor debug (host name disappeared in case of filter through)

  NOTE: Use of compiler option "-O3" will result in error at inline-assembler
        function. Use "-O2" option.

*/
#define PROGRAM_NAME "datachk"
#define PROGRAM_VERSION "Ver. 2006-02-01"

#include "libk5.h"      // common header for IP-VLBI utilities

//#ifndef WIN32
//#include <stdint.h>
//#endif
//#define REWIND_BYTE  0  // define how many bytes rewind if failure in 
                          // header detection  on 1st trial (shoud be multiples of 4)
                          // 0 means rewind till previous header
#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#include "libvssp32.h"

//#define REWIND_BYTE 200 
//#define USE_ASM    // use inline assemble function


#ifdef USE_ASM
    #ifdef WIN32     // in-line assembler for VC++

    static void hist_count4(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data
           npos  --- # of samples in idata
           xhalf --  threshold
           xhist --  count results

         count every 4 data step (suppose 4ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        __asm{
            mov edx, npos
            cmp edx, 0
            jz end_func
            mov eax, idata  // start address of decoded data
            mov esi, xhalf
            xor ecx,ecx   // counter for 32bit-word number
            xor edi,edi   // counter for plus sign
    loop_head:
            mov ebx, [eax+ecx*4]
            //mov esi, xhalf
            cmp ebx, esi
            jl pass1
            //inc edi
            add edi, 1    // faster than inc
    pass1:
            add ecx,4  // next  data 
            cmp ecx, edx
            jnz loop_head
    end_func:
            mov eax, xhist
            mov [eax], edi
        }
    } 
    
    static void hist_count1(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data
           npos  --- # of samples in idata
           xhalf --  threshold
           xhist --  count results

         count every 1 data step (suppose 1ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        __asm{
            mov edx, npos
            cmp edx, 0
            jz end_func
            mov eax, idata  // start address of decoded data
            mov esi, xhalf
            xor ecx,ecx   // counter for 32bit-word number
            xor edi,edi   // counter for plus sign
    loop_head:
            mov ebx, [eax+ecx*4]
            //mov esi, xhalf
            cmp ebx, esi
            jl pass1
            //inc edi
            add edi, 1    // faster than inc
    pass1:
            add ecx,1  // next  data 
            cmp ecx, edx
            jnz loop_head
    end_func:
            mov eax, xhist
            mov [eax], edi
        }
    }
    #else     // gcc inline assemble

    static void hist_count4(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           idata --- array to contain decoded data   8(%ebp)
           npos  --- # of 32 bits word data in idata 12(%ebp)
           xhist --  array to contain results

           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data 8(%ebp)
           npos  --- # of samples in idata        12(%ebp)
           xhalf --  threshold                    16(%ebp)
           xhist --  count results                20(%ebp)

         count every 4 data step (suppose 4ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        asm volatile (
            "pushl %eax \n\t"     // save register used in this function
            "pushl %ebx \n\t"
            "pushl %ecx \n\t"
            "pushl %edx \n\t"
            "pushl %edi \n\t"
            "pushl %esi \n\t"
            "movl 12(%ebp), %edx \n\t"
            "cmp $0, %edx  \n\t"
            "jz end_func4 \n\t"
            "movl 8(%ebp), %eax  \n\t" // start address of decoded data
            "movl 16(%ebp), %esi \n\t"
            "xor %ecx, %ecx \n\t"   // counter for 32bit-word number
            "xor %edi, %edi \n\t"   // counter for plus sign
    "loop_head4: \n\t"
            "movl (%eax,%ecx,4), %ebx \n\t"
            "cmp %esi, %ebx  \n\t"
            "jl pass14 \n\t"
            "add $1, %edi \n\t"    // faster than inc
    "pass14: \n\t"
            "add $4, %ecx  \n\t"  // next  data 
            "cmp %edx, %ecx \n\t"
            "jnz loop_head4 \n\t"
    "end_func4: \n\t"
            "movl 20(%ebp), %eax  \n\t"
            "movl %edi, (%eax) \n\t"
            "popl %esi \n\t"    // recover saved register 
            "popl %edi \n\t"
            "popl %edx \n\t"
            "popl %ecx \n\t"
            "popl %ebx \n\t"
            "popl %eax \n\t"
        );
    } 


    static void hist_count1(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           idata --- array to contain decoded data   8(%ebp)
           npos  --- # of 32 bits word data in idata 12(%ebp)
           xhist --  array to contain results

           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data 8(%ebp)
           npos  --- # of samples in idata        12(%ebp)
           xhalf --  threshold                    16(%ebp)
           xhist --  count results                20(%ebp)

         count every 1 data step (suppose 1ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        asm volatile (
            "pushl %eax \n\t"     // save register used in this function
            "pushl %ebx \n\t"
            "pushl %ecx \n\t"
            "pushl %edx \n\t"
            "pushl %edi \n\t"
            "pushl %esi \n\t"
            "movl 12(%ebp), %edx \n\t"
            "cmp $0, %edx  \n\t"
            "jz end_func1 \n\t"
            "movl 8(%ebp), %eax  \n\t" // start address of decoded data
            "movl 16(%ebp), %esi \n\t"
            "xor %ecx, %ecx \n\t"   // counter for 32bit-word number
            "xor %edi, %edi \n\t"   // counter for plus sign
    "loop_head1: \n\t"
            "movl (%eax,%ecx,4), %ebx \n\t"
            "cmp %esi, %ebx  \n\t"
            "jl pass11 \n\t"
            "add $1, %edi \n\t"    // faster than inc
    "pass11: \n\t"
            "add $1, %ecx  \n\t"  // next  data 
            "cmp %edx, %ecx \n\t"
            "jnz loop_head1 \n\t"
    "end_func1: \n\t"
            "movl 20(%ebp), %eax  \n\t"
            "movl %edi, (%eax) \n\t"
            "popl %esi \n\t"    // recover saved register 
            "popl %edi \n\t"
            "popl %edx \n\t"
            "popl %ecx \n\t"
            "popl %ebx \n\t"
            "popl %eax \n\t"
        );
    } 

    #endif

#else

    void hist_count4(int* idata, int npos, int xhalf, int* pcount)
    {
        int n,k;
        *pcount=0;
        for(n=0; n<npos; n+=4){
           if(idata[n] >= xhalf) *pcount+=1;
        }
    }

    void hist_count1(int* idata, int npos, int xhalf, int* pcount)
    {
        int n;
        *pcount=0;
        for(n=0; n<npos; n++){
           if(idata[n] >= xhalf) *pcount+=1;
        }
    }

#endif

void hist_count(K5VSSP_file_t *K5, int *xhist)
{
    /*  do count for statistics */
    int xhistw[8];     // 8=4ch*2  (" means sign+  sign- of data)
    int xhalf,n;
    xhalf=1;
    xhalf<<=K5->adbit-1;
    if(K5->numch==4){  // case of 4 channel data
         hist_count4(K5->idata, K5->npos, xhalf, &xhistw[1]);
         hist_count4(K5->idata+1, K5->npos, xhalf, &xhistw[3]);
         hist_count4(K5->idata+2, K5->npos, xhalf, &xhistw[5]);
         hist_count4(K5->idata+3, K5->npos, xhalf, &xhistw[7]);
         xhistw[0]=K5->npos/K5->numch-xhistw[1];
         xhistw[2]=K5->npos/K5->numch-xhistw[3];
         xhistw[4]=K5->npos/K5->numch-xhistw[5];
         xhistw[6]=K5->npos/K5->numch-xhistw[7];
         for(n=0; n<8; n++){
             xhist[n]+=xhistw[n];
         }
    } else {  // case of 1 ch data
         hist_count1(K5->idata, K5->npos, xhalf, &xhistw[1]);
         xhistw[0]=K5->npos-xhistw[1];
         xhist[0]+=xhistw[0];
         xhist[1]+=xhistw[1];
    }
}

void chkfilen2(char* fname)
{
       int usampl=200000;
       int i,n;
       int kword;
       int64_t tword;
       long int numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
       //int rword;
       int f_c;
       int xhist[8];     // 8=4ch*2  (" means sign+  sign- of data)
       int xhalf;
       char cbuf[10], fname2[80];
       char fmt[10];
       int total_frame, bad_frame;
       //int wordread;
       int headerbits;
       double dbytes;
       K5VSSP_file_t K5;
       vsspaux_t aux;

       K5.aux32=K5.head32+3;

       total_frame=0;
       bad_frame=0;

       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;
       for(i=0; i<8; i++) xhist[i]=0; 


       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       strcpy(K5.fname,fname);

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           goto exitend2;
       }
       printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);

       K5.twordread=0;  // reset word read counter
       K5.pos=0;        // reset current pointer position
       tword=0;

       //if(checkheader(&K5) < 0){
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       dbytes=kword*4.0+K5.headsizew*4.0;
       //printf("kword, headsizew, dbytes %d %d %f\n",kword,K5.headsizew,dbytes);

       tword+=K5.twordread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer
       K5.idata  = (int *)malloc(32*K5.nword*sizeof(int)); // K5 data read buffer

       if(kword >0){
           printf("First 64bit is not a header block!\n");
       }

       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %04X", K5.aux32[n]);
           }
           aux.size=K5.auxbyte;
           //strncpy(aux.bdat,(char *)K5.aux32,K5.auxbyte); // this is not good for in case of data 0 is included
           for(n=0; n<aux.size; n++) aux.bdat[n]=*((char *)K5.aux32+n);
           vssp_aux_parse(&aux);
           printf("\n        Format # = %d\n",aux.fmt);
           if(aux.fmt != 0)
             printf("        Filter (MHz) : %d\n",aux.filter);
           if(aux.fmt == 1)
             printf("        Station info : ID2(%s) name(%s)\n",aux.sid2,aux.station8);
           if(aux.fmt == 1 || aux.fmt == 2 )
             printf("        PC host name :  %s\n",aux.hostname8);

           printf("\n");
       }
       printf("\n");

       printf("                                                    %% of plus sign data  \n");
       printf(" FMT A/D CH  f(kHz)  TIME   seconds    # of bits    Ch1    Ch2    Ch3    Ch4\n");
       printf("%s %2d  %2d  %5d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);


       while(1){
           for(n=0; n<K5.imax; n++){
                K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
                parseADdata_3N(&K5);
                dbytes+=(double)K5.wordread*4.0;
                tword+=K5.wordread;

                /* sampling balance statistics (only check +/- unbalance) */
                hist_count(&K5, xhist);

                if (K5.wordread ==0 ) {// EOF
                    i=fclose( K5.stream ) ;
                    f_c++;
                    // continuous file name creation     XXXX.datN   where N=1,2,3......
                    strcpy(fname2,fname);
                    sprintf(cbuf,"%d\0",f_c);
                    strcat(fname2,cbuf);
                    /* file open */
                    if( (K5.stream  = fopen( fname2, "rb" )) == NULL ){
                        // file not found
                        goto exitend2;
                    }   
                    printf("\n continuous file (%s) opened!\n",fname2);  
                    printf("                                   ");  
                    strcpy(K5.fname,fname2);
                    K5.wordread = fread( K5.ubuf, sizeof( u_int32_t ), K5.nword, K5.stream );
                    parseADdata_3N(&K5);
                    dbytes+=(double)K5.wordread*4.0;
                    tword+=K5.wordread;

                    /* sampling balance statistics (only check +/- unbalance) */
                    hist_count(&K5, xhist);

                }
                if( ferror( K5.stream )  ) {
                    //printf("File Read Error!\n");
                    break;
                }

           }
           K5.twordread=0;  // reset word read counter
           if((kword=CheckK5header_with_Read(&K5)) < 0){
                tword+=K5.twordread;
                //printf("File read error! \n");
                //fgetpos(K5.stream,&K5.pos); 
                i=fclose( K5.stream ) ;
                goto exitend2;
           }
           dbytes+=kword*4.0+K5.headsizew*4.0;
           tword+=K5.twordread;
           // statistics print

           K5.vflag=1;
           xhalf=(int)(pow(2,K5.adbit-1)+0.5);
           total_frame++;
           //bitint=(numb-8)*8;
           //bitint=(numb-2)*32;
           numb=kword;
           //printf("kword, k5.headsizew %d %d\n",kword,K5.headsizew);
           bitsec=K5.sfreq*1000*K5.adbit*K5.numch;
           bitint=bitsec+numb*32;
           if(K5.numch==1){
               printf("%12d %6.1f",bitint,
                      100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]));
           } else {
               printf("%12d %6.1f %6.1f %6.1f %6.1f",bitint,
                      100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]),
                      100.0*(double)xhist[1*2+1]/(double)(xhist[1*2+0]+xhist[1*2+1]),
                      100.0*(double)xhist[2*2+1]/(double)(xhist[2*2+0]+xhist[2*2+1]),
                      100.0*(double)xhist[3*2+1]/(double)(xhist[3*2+0]+xhist[3*2+1]));
           } 
           if (numb != 0){
               bad_frame++;
               printf("NG "); 
               if(tflag ==0){
                   printf("Warning! (# of bits Error)\n"); 
               } else {
                   printf("Warning! (TIME DISCON and Bits error!)\n");
               }

           } else {
               if(tflag==0){
                   printf("  \n"); 
               } else {
                   printf(" Warning! (TIME DISCON)\n");
               }
           }
           if(nkai >0 && K5.seconds-secold !=1){
                printf("%s %2d  %2d  %5d %02d:%02d:%02d %6dNG", fmt,
                          K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
                tflag=1;
           } else {
                printf("%s %2d  %2d  %5d %02d:%02d:%02d %6d  ", fmt,
                          K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
                tflag=0;
           }
           for(i=0; i<8; i++) xhist[i]=0; 
           secold=K5.seconds;
           nkai++;
       }

 exitend2:
        free(K5.ubuf);
        free(K5.idata);
        printf("\n");
        printf("%lld bytes read\n",tword*4); 
        printf("%d frames out of %d frames (%f%%) are bad frames.\n",
           bad_frame, total_frame, 
           100.0*(double)bad_frame/(double)total_frame);
}


void chkfilen(char* fname) 
{
       int usampl=200000;
       int i,n,kword;
       int64_t tword;
       long int numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
       int headerbits;
       int f_c;
       int total_frame, bad_frame;
       char cbuf[10], fname2[80];
       char fmt[10];
       double dbytes;
       K5VSSP_file_t K5;
       vsspaux_t aux;
       //fpos_t size;

       K5.aux32=K5.head32+3;
       total_frame=0;
       bad_frame=0;
       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       headerbits=64;  // for K5/VSSP

       strcpy(K5.fname,fname);



       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           goto exitend2;
       }
       printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);

       K5.twordread=0;  // reset word read counter
       K5.pos=0;        // reset current pointer position
       tword=0;

       //if(checkheader(&K5) < 0){
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       tword+=K5.twordread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

       if(kword >0){
           printf("First 64bit is not a header block!\n");
       }

       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %04X", K5.aux32[n]);
           }
           aux.size=K5.auxbyte;
           //strncpy(aux.bdat,(char *)K5.aux32,K5.auxbyte); // this is not good for in case of data 0 is included
           for(n=0; n<aux.size; n++) aux.bdat[n]=*((char *)K5.aux32+n);
           vssp_aux_parse(&aux);
           printf("\n        Format # = %d\n",aux.fmt);
           if(aux.fmt != 0)
             printf("        Filter (MHz) : %d\n",aux.filter);
           if(aux.fmt == 1)
             printf("        Station info : ID2(%s) name(%s)\n",aux.sid2,aux.station8);
           if(aux.fmt == 1 || aux.fmt == 2 )
             printf("        PC host name :  %s\n",aux.hostname8);
           printf("\n");
       }
       printf("\n");

       printf(" FMT A/D CH  f(kHz)  TIME   seconds   # of bits\n");
       printf("%s %2d  %2d   %4d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);


loop1:
       //if((kword=fwdNsecK5(1, &K5))<0){
       if((kword=fwd1secK5(&K5))<0){
           tword+=K5.twordread;
           //printf("File EOF  %d\n", K5.twordread);
           goto exit2next;
       }
       //bitint = bitsec+kword*32;
       bitint=(int)(K5.twordread*32-headerbits);  //64bit is headder
       tword+=K5.twordread;
       K5.twordread=0;  // reset read counter
       total_frame++;

       printf("%10d",bitint);
       if (bitint != bitsec){
          bad_frame++;
          printf(" NG "); 
          if(tflag ==0){
              printf("# of bits Error\n"); 
          } else {
              printf("TIME DISCON and Bits error!\n");
          }
       } else {
          if(tflag==0){
              printf("  \n"); 
          } else {
              printf(" TIME DISCON\n");
          }
      }
       printf("%s %2d  %2d   %4d %02d:%02d:%02d %6d  ", fmt, 
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
       if(nkai >0 && K5.seconds-secold !=1){
           tflag=1;
       } else {
           tflag=0;
       }
       secold=K5.seconds;
       nkai++;

       goto loop1;

exit2next: // next file candidate read

       i=fclose( K5.stream ) ;
       f_c++;
       // continuous file name creation     XXXX.datN   where N=1,2,3......
       strcpy(fname2,fname);
       sprintf(cbuf,"%d\0",f_c);
       strcat(fname2,cbuf);
       strcpy(K5.fname,fname2);
       K5.size=get_file_size(K5.fname);
       // file open 
       if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ) {
           //printf("File Open Error! \n");
           goto exitend2;
       }
       K5.pos=0;
       tword-=K5.twordread;   //  to compensate too much count
       //K5.size=get_file_size(K5.stream);

       printf("\n continuous file (%s (%d bytes)) opened!\n",K5.fname,K5.size);  
       printf("                                    ");  
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            tword+=K5.twordread;
            goto exitend;
       }

       bitint=(int)(K5.twordread*32-headerbits);  //64bit is headder
       tword+=K5.twordread;
       K5.twordread=0;  // reset read counter
       total_frame++;
       printf("  %d",bitint);
       if (bitint != bitsec){
          bad_frame++;
          printf("NG "); 
          if(tflag ==0){
              printf("# of bits Error\n"); 
          } else {
              printf("TIME DISCON and Bits error!\n");
          }
       } else {
          if(tflag==0){
              printf("  \n"); 
          } else {
              printf(" TIME DISCON\n");
          }
       }

       printf("%s %2d  %2d   %4d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
       if(nkai >0 && K5.seconds-secold !=1){
           tflag=1;
       } else {
           tflag=0;
       }
       secold=K5.seconds;
       goto loop1;
    
 exitend:
       i=fclose( K5.stream ) ;
       //free(K5.ibuf);
       free(K5.ubuf);
       printf("\n");
 exitend2:;
       //tword+=K5.twordread;
       printf("\n");
       printf("%lld bytes read\n",tword*4);
       printf("%d frames out of %d frames (%f%%) are bad frames.\n",
           bad_frame, total_frame, 
           100.0*(double)bad_frame/(double)total_frame);
}

int main(int argc, char *argv[])
{
    char fname[60];
    char* dflt_fname="tds.data";
    int mode;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    mode=0;
    if(argc < 2) {
       //strcpy(fname,dflt_fname);
       printf("%s %s\n",PROGRAM_NAME,PROGRAM_VERSION);
       printf("How to run: \n");
       printf("  datachk32 datafile [mode]\n");
       printf("\n");
       printf("    where  datafile ----- K5 data file name to check\n");
       //printf("                          if 0 is given, internal dflt name will be used\n");
       printf("           mode      ---  display sampling statistics (zero balance)\n");
       printf("                           1: ON   0: no display (default)\n");
       exit(0);
    }
    if (argc>=2){
       strcpy(fname,*(argv+1));
    } 
    if(argc >=3){
       mode = atoi(*(argv + 2 ));
    }
    cstart1sec = clock();
    printf("************ %s  %s ************\n",PROGRAM_NAME,PROGRAM_VERSION);
    printf("     [Note: # of bits does not include Header]\n");
    if(mode==1){
       chkfilen2(fname);
    } else {
       chkfilen(fname);
    }
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
