/*******************************************************************/
/*                                                                 */
/*   Program Name :  inivssp32                                     */
/*                                                                 */
/*      Version:  0.00  2005-12-26                                 */
/*                                                                 */
/*      Copyright (c) 2005 T.Kondo/NICT All Right Reserved         */
/*                                                                 */
/*******************************************************************/
/*
/*
     Initialize and Time Set without 1PPS sync for K5/VSSP32  
        How to use:
          inivssp32 [year month day hour min sec]
          
        When all parameters are omitted, current IP-VLBI board time is read
        and set again.

*/
#define INIVSSP32_VERSION "Ver. 2005-12-26"

#include "chead.h"      // common header for IP-VLBI utilities

#include "libipvlbi.c"  // IP-VLBI functions (by T.Kondo)
#include "libipvlbi_board.c"  // IP-VLBI sampler board functions (by T.Kondo)

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int y2k(int yyyy, int mm, int dd);

int y2k(int yyyy, int mm, int dd)
{
  int i, sum = 0;
  int month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30 , 31, 30, 31};

  if(((yyyy % 4 == 0) && (yyyy % 100 != 0)) || (yyyy % 400 == 0))
    month[1] = 29;
  for (i=0; i<mm-1; i++) {
    sum += month[i];
  }
  sum += dd;

  return sum;
}


int l_year_chk(int year)   // leaf year check
{
       /* leaf year check */
      if (year-(year/4)*4 == 0) 
      { 
          if (year-(year/100)*100 == 0) 
          { 
               if (year-(year/400)*400 == 0) { return 0;}
               return -1;
           }
           return 0;
       }
       return -1;
}

void k2y(int year, int ddd, int* month, int* mday)
{
      int md[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
      int i,td,wd ;
       /* leaf year check */
      if (l_year_chk(year)==0)
      {
         md[1]=29;
      }
      *month=0;
      *mday=0;
      td=ddd;
      for (i=0; i< 12; i++)
      {
         wd=td-md[i];
         if (wd <=0)
         {
           *mday=td;
           *month=i+1;
           goto exithere;
          }
         td=wd;
       }
   exithere:;
}

unsigned int board_ref_check(int fd_in)
{
    /* board reference input signal check */
/*
    [IN}
       fd_in  :  device file directive (must be opened in advance)
    [return]
       reference signal status
*/
    int rv;
    unsigned int stat;
    unsigned int ref, pps, rhz, pll;
    stat=0;
    /* board status check */
    if((rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat))==-1){
        perror("   ioctl() status get failed ");
        goto end;
    }
    /* satatus get again just in case */
    if((rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat))==-1){
        perror("   ioctl() status get failed ");
        goto end;
    }
    //printf("ioctl() status get succeed\n");
    //printf(" status = %X\n",stat);   // debug
#ifdef _VSSP32
    // separate command
    //stat|=0x8;  // means this is VSSP32
    //ioctl(fd_in, TDSIO_GET_REFINPUT, &ref);
    //if(ref==1) stat|=1;
    //ioctl(fd_in, TDSIO_GET_10MHZINPUT, &rhz);
    //if(rhz==1) stat|=0x4;
    //ioctl(fd_in, TDSIO_GET_1PPSINPUT, &pps);
    //if(pps==1) stat|=0x2;
    
    /* set stat again to keep compatibility with K5/VSSP */
    ref=0; rhz=0; pps=0; pll=0;
    if((stat & 4) == 4) pps=1;        // 1PPS supplied
    if((stat & 8) == 8) ref=1;        // reference signal supplied
    if((stat & 0x20) == 0x20) rhz=1;  // reference signal is 5MHz
    if((stat & 0x10) == 0x10) pll=1;  // PLL locked
    stat=0;
    stat|=0x10;  // means this is VSSP32
    if(ref==1) stat|=1;
    if(pps==1) stat|=0x2;
    if(rhz==1) stat|=0x4;
    if(pll==1) stat|=0x8;
#else
    stat&=3;  // 2005-12-20
#endif
end:
    return stat;
}



/* board reference signal check 2 */
int board_ref_check2(int fd_in)
{
/*
    [IN}
       fd_in  :  device file directive (must be opened in advance)
    [return]
       -1 :  reference signal error
        0 :  OK
*/
    unsigned int stat;

    stat=board_ref_check(fd_in);
    if((stat & 0x10) == 0x10){ // this is VSSP32
        if((stat & 1) != 1){
            printf("   Warning! 10MHz or 5MHz signal is not supplied. \n");
        } else {
            if((stat & 4) == 4){
                printf("   Reference (5MHz) signal is OK!");
            } else {
                printf("   Reference (10MHz) signal is OK!");
            }
            if((stat & 8) == 8){
                printf("   PLL locked!\n");
            } else {
                printf("   Warning! PLL unlocked!\n");
            }
        }
    } else {   // this is VSSP
        if((stat & 1) != 1){
            printf("   Warning! 10MHz signal is not supplied. \n");
        } else {
            printf("   10MHz signal is OK!\n");
        }
    }
    if((stat & 2) != 2){
        printf("   Warning! 1PPS signal is not supplied. \n");
    } else {
        printf("   1PPS  signal is OK!\n");
    }
    if((stat & 3) == 0){
       printf("   Check signal connection and try again!\n");
       return -1;
    }
    if((stat & 3) == 1){
       printf("   Anyway we force to set time!\n");
       return -2;
    }
    return 0;
}

int main(int argc, char **argv)
{
    char dev[] = DEV_FILE; 
    int fd_in,rv, sig;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    unsigned int secold, stat;
    int  yyyy, mm, dd;      // Ԑݒ(timeset)ŗp
    int  h, m, s;           // Ԑݒ(timeset)ŗp
    int  y3; // 7 bits year  (0-127)
    int counter;
    int i;
    int rmode;
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[80];
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
         if(strncmp(*(argv+1),"?",1)==0){
           printf("inivssp32 %s\n  Usage: inivssp32 [year month day hour min sec]\n", INIVSSP32_VERSION);
           exit(0);
         }
         if(strncmp(*(argv+1),"only",3)==0){  // initialize only
             printf("execute initialize only\n");
             fd_in = open(dev, O_RDONLY);
             rv=ioctl(fd_in, TDSIO_INIT);
             close(fd_in);
             exit(0);
         }
    } 
    /* get host name */
    hostget(hostname);

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
//    if(getenv(K5LOGDIR) != NULL){
//        strcpy(logdir,getenv(K5LOGDIR));
//    } else {
//        strcpy(logdir,dflt_logdir);
//    }

    rmode=1;
    if(argc == 7){
        rmode=0;
        /* parameters get */
        yyyy = atoi(*(argv + 1 ));
        mm   = atoi(*(argv + 2 ));
        dd   = atoi(*(argv + 3 ));
        h    = atoi(*(argv + 4 ));
        m    = atoi(*(argv + 5 ));
        s    = atoi(*(argv + 6 ));
//        sprintf(cbuf,"inivssp32(%s) %d %d %d %d %d %d",INIVSSP32_VERSION,yyyy,mm,dd,h,m,s);
//        if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
    }



    fd_in = open(dev, O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
//        sprintf(cbuf,"device open() error");
//        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("Device open successd No.[%d]\n",fd_in);
    
    /* board reference signal check */
    printf("\n");
    if((sig=board_ref_check2(fd_in))==-1) goto end;
    
    /* initialize board */
    if((rv=ioctl(fd_in, TDSIO_INIT)) == -1){
        perror("ioctl() init failed ");
        return -1;
    } else {
        printf("ioctl() INIT succeed\n");
    }
    
    /* time read */
    if(rmode==1){
#ifdef _VSSP32
        rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
        yyyy=y3+2000;
#endif
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1) {
            perror("ioctl() time get failed ");
            return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        sec2hms(sec,&h,&m,&s);
        k2y(yyyy, day, &mm, &dd);
        printf("%d,%d,%d,%d,%d,%d\n",yyyy,mm,dd,h,m,s);  // debug
    }
    

    /* time set */
    //printf("%d,%d,%d,%d,%d,%d\n",yyyy,mm,dd,h,m,s);  // debug

    day = y2k(yyyy, mm, dd);
    sec = ((h * 60) + m) * 60 + s;
    time = TDS_MAKE_TIME(day,sec);

    printf("   Set time  => Total day : %5d    Seconds from 0h : %5d \n",day,sec);
#ifdef _VSSP32
    //year set
    y3=yyyy-2000;
    rv = ioctl(fd_in, TDSIO_SET_YEAR, &y3);  
    if (rv == -1) {
        perror("ioctl() year set failed ");
//        sprintf(cbuf,"ioctl() year set failed");
//        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
#endif

    //date and seconds set to sampler
    rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
    if (rv == -1) {
        perror("ioctl() time set failed ");
//        sprintf(cbuf,"ioctl() time set failed");
//        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("time set ok\n");

    //sync to 1PPS
//    if(sig==0){
//        rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
//        if (rv == -1) {
//            perror("ioctl() 1pps sync failed ");
//            sprintf(cbuf,"ioctl() 1pps sync failed");
//            if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
//            return -1;
//        }
//        printf("   1pps sync ok\n");
//    }

    time = 0; 
    secold=90000;
    counter=0;
    close(fd_in);
	
    
    // log out
//    if(sig==0){
//       sprintf(cbuf,"timesettk (time set OK)");
//    } else {
//       sprintf(cbuf,"timesettk (time set OK but 1PPS was not supplied! (no 1PPS sync made)");
//    }
//    if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");

    return 0; // debug
    // display time for given time seconds
    fd_in = open(dev, O_RDONLY);   //device open
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    printf("\n");
    do{
#ifdef _VSSP32
        rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
        yyyy=y3+2000;
#endif
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1) {
            perror("ioctl() time get failed ");
            return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        if (sec!=secold) {
             sec2hms(sec,&h,&m,&s);
             //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
             //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
#ifdef _VSSP32
             printf("  %04d/%03d %02d:%02d:%02d\n\033[A",yyyy,day,h,m,s);
#else
             printf("  %03d %02d:%02d:%02d\n\033[A",day,h,m,s);
#endif
             counter++;
         }
         secold=sec;
    }while(counter <=10);
#ifdef _VSSP32
    printf("\n\n*** time setting to K5/VSSP32 successed ");
#else
    printf("\n\n*** time setting successed ");
#endif
    if(sig !=0) printf("(but no 1PPS signal)");
    printf(" ***\n\n");
end:
    close(fd_in);
    return 0;
}
