/*********************************************************************/
/*                                                                   */
/*   Program Name :  k5tok5v32                                       */
/*                                                                   */
/*      Version:  0.00  2005-12-31                                   */
/*                                                                   */
/*                                                                   */
/*      Data format converter from K5/VSSP to K5/VSSP32              */
/*                                                                   */
/*                                                                   */
/*      Copyright (c) 2005- T.Kondo/NICT All Right Reserved          */
/*                                                                   */
/*********************************************************************/
/*
      usage
             k5tok5v32 file_name [ofile]
                  where   file_name -- K5/VSSP data file name
                          ofile      - outfile name (default datafile.v32)

        ver 0.00 2005-12-31  start development
*/
#define PROGRAM_VERSION "Ver. 2005-12-31"

#include "libk5.h"      // common header for IP-VLBI utilities

//#ifndef WIN32
//#include <stdint.h>
//#endif
//#define REWIND_BYTE  0  // define how many bytes rewind if failure in 
                          // header detection  on 1st trial (shoud be multiples of 4)
                          // 0 means rewind till previous header
#define REWIND_BYTE 200 
#define USE_ASM    // use inline assemble function

typedef struct K5VSSP_file     // originated from ip_vlbi_file type
                               // difference is displayed by **K5**
{
    /*
      Note: use of a fpos_t type variable was obsoleted because of
         the compatiblity ploblem of a fpos_t type variable between linux 2.6
         and previous one.
    */
	//unsigned char bit32[4];  // obsoleted in **K5**
	//unsigned char bit64[8]; // obsoleted in **K5**
	unsigned char *ibuf; // will be obsoleted in **K5**
	u_int32_t head32[8]; // **K5**  to contain full header including 
                         // in case of K5/VSSP32
	u_int32_t *aux32; // **K5**  to contain aux field start address in
                        // head 32
	u_int32_t *ubuf; // pointer to read buffer **K5**
    int nword;       // ubuf size allocated or to be allocated  **K5**
    int auxbyte;     // # of bytes in aux files  **K5**
                     // valid byte number in aux32  (deflt=20)
    int headsizew;    // header size in 32bit-word
    int *idata;   // pointer to decoded data (integer) container
    double *data;   // pointer to decoded data (R*8) container
	int sfreq;  //  sampling frequency in kHz
	int adbit;  // AD bits (1,2,4,8)
	int numch;  // # of channels
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit # of samples
	int kpos;  // should be obsoleted ?
	int npos;    //  valid number in idata or xdata **K5**
	int imax;    //  # of units in a second data
	int bytesec;    //  # of bytes in a second data  **K5**
	//char* fname; //pointer data file name obsoleted
	char fname[128]; //data file name opened or to be opened  **K5**
	double fsample;  // sampling frequency (Hz)
	double tsample; // sampling period (sec)
	double unitp;   // usampl*tsampl unit length 
    long int seconds; // time (sec) from mid night (00h00m00s)
	int hh, mm, ss;  // time in hour, minutes, seconds 
    int year; // 4 digits year (K5/VSSP32)
    int totalday; // (K5/VSSP32)
    int version1; // Major version (K5/VSSP32)
    int version2; // Minor version (K5/VSSP32)
	int spatn; // detected 2nd sync pattern detected 
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 K5/VSSP, =2 K5/VSSP32
                //         =2 is added **K5**
	FILE *stream;
	//fpos_t pos;   // changed to int64_t type 
	//fpos_t size;  // changed to int64_t type
                    // because fpos_t type is no more scalar value in linux 2.6
	int64_t pos;   // current pointer
	int64_t size;  // opened file size in bytes **K5**
	int fh;
	int ch;    
	int ierr;   // error code (general purpose)
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;  // number of bytes read
	int wordread;   // # of 32bit-words read counter (general) **K5**
	int64_t twordread;   // total # of 32bit-word read **K5**
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
} K5VSSP_file_t;

int CheckK5header_with_Read(K5VSSP_file_t *S);

int64_t get_file_size(char *fname)
{
    /*  get file size opened */
    struct stat info;
    stat(fname,&info);
    return (int64_t)info.st_size;
    // use %lld to print out whole number that has int64_t type
}

int header_aux_read(K5VSSP_file_t *K5)
{
   // read Aux field in K5/VSSP32 header
   // assumption : pointer is located just next to 64bit common header

       int i, i32, n;

       i32=0;

       /* next 32bit data read */
       K5->wordread = fread( K5->head32+2, sizeof( u_int32_t ), 1, K5->stream );
       K5->twordread+=K5->wordread;
       K5->pos+=K5->wordread*4;  // pointer position is in bytes
       if( ferror( K5->stream ) || K5->wordread==0 ){ // File EOF or read error
	       i=fclose( K5->stream ) ;
	       i32=-1;
           goto exit_here;
       }
       // parse header into each item
       K5->year=2000+(int)((K5->head32[2] >> 9) & 0x7F); // year
       K5->totalday=(int)(K5->head32[2] & 0x1FF);
       K5->version1=(int)((K5->head32[2] >> 28) & 0xF);
       K5->version2=(int)((K5->head32[2] >> 24) & 0xF);
       K5->auxbyte =(int)((K5->head32[2] >> 16) & 0xFF);
       //printf("year = %d auxbyte = %d\n",K5->year,K5->auxbyte);
       for(n=0; n<K5->auxbyte/4; n++){
           K5->wordread = fread( K5->head32+3+n, sizeof( u_int32_t ), 1, K5->stream );
           K5->twordread+=K5->wordread;
           K5->pos+=K5->wordread*4;  // pointer position is in bytes
           if( ferror( K5->stream ) || K5->wordread==0 ){ // File EOF or read error
	           i=fclose( K5->stream ) ;
		       i32=-1;
               goto exit_here;
	       }
       }
exit_here:
       return i32;

}

// K5 header check (updated from headerchkn2)
int CheckK5header(K5VSSP_file_t *K5)
/*
    K5 header common portion (1st 64bits) check

    if 1st 64bits show K5/VSSP32 pattern,  extended header and
    AUx field are read


    return 0 : K5/VSSP Header detected
          -1 : K5.head32 is not header
         -99 : EOF detected
*/
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; 
    int smplbit0, sfreq0, nch;
    u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
    u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
    int iok;
    iok=-1;

    K5->format=0;  // suppose K5/VSSP old format
    if(K5->head32[0]==0xFFFFFFFFUL){
        if((K5->head32[1] & 0xFF000000UL) == syncpat) K5->format=1; // this is sync block
        if((K5->head32[1] & 0xFF000000UL) == syncpat32) K5->format=2; // this is sync block
        //if((K5->head32[0]==0xFFFFFFFFUL) && ((K5->head32[1] & 0xFF000000UL) == syncpat)){ // this is sync block
        //if((K5->head32[0]==0xFFFFFFFFUL) && ((K5->head32[1] & 0xFF000000UL) == syncpat32)){ // this is sync block
        if(K5->format > 0){ 
            iok=0;
            /* parse sampling information and get time data */ 
            K5->spatn=(int)((K5->head32[1] >> 24) & 0xFF);       // spatn = 139 (0x8B) means sync pattern
            smplbit0 = (K5->head32[1] >> 22) & 0x3;   
            K5->adbit = sdat[smplbit0];      // AD bits
            sfreq0 = ( K5->head32[1] >> 18) & 0xF;
            K5->sfreq=fdat[sfreq0];
            nch = (K5->head32[1] >> 17) & 1;
            if (nch == 1) {
                K5->numch = 4; 
            } else {
                K5->numch = 1;
            }
            K5->seconds=(long int)(K5->head32[1] & 0x1FFFFUL);
            sec2hms(K5->seconds,&K5->hh,&K5->mm,&K5->ss);
        }
        if(K5->format==2){  // read further header
            K5->headsizew=8;
            if(header_aux_read(K5)<0) iok=-99;
       }

    }
    return iok;
}

void vssptovssp32(char* fname, char* oname)
{
       int usampl=200000;
       int i,n;
       int kword;
       int64_t tword;
       long int numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
       int nwrite;
       //int rword;
       int f_c;
       int xhist[8];     // 8=4ch*2  (" means sign+  sign- of data)
       int xhalf;
       char cbuf[10], fname2[80];
       char fmt[10];
       int total_frame, bad_frame;
       //int wordread;
       int headerbits;
       double dbytes;
       u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
       u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
       K5VSSP_file_t K5;
       FILE *fo;

       K5.aux32=K5.head32+3;

       total_frame=0;
       bad_frame=0;

       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;
       for(i=0; i<8; i++) xhist[i]=0; 


       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       strcpy(K5.fname,fname);

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           goto exitend2;
       }
       printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);

       K5.twordread=0;  // reset word read counter
       K5.pos=0;        // reset current pointer position
       tword=0;

       //if(checkheader(&K5) < 0){
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }

       if(K5.format==1){// nothing to do
           printf("File %s has K5/VSSP format. So nothing to do!\n");
            i=fclose( K5.stream ) ;
            goto exitend2;
       }

       // out file open

       if( (fo  = fopen( oname, "wb" )) == NULL ) {
           printf("File %s Open Error! \n",oname);
           goto exitend2;
       }

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       dbytes=kword*4.0+K5.headsizew*4.0;
       //printf("kword, headsizew, dbytes %d %d %f\n",kword,K5.headsizew,dbytes);

       tword+=K5.twordread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

       if(kword >0){
           printf("First 64bit is not a header block!\n");
       }

       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %0X", K5.aux32[n]);
           }
           printf("\n\n");
       }

       printf("                                                    %% of plus sign data  \n");
       printf(" FMT A/D CH  f(kHz)  TIME   seconds    # of bits    Ch1    Ch2    Ch3    Ch4\n");
       printf("%s %2d  %2d  %5d %02d:%02d:%02d %6d  ", fmt,
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);

       while(1){
           // header copy
           // At first, replace 2nd sync word to K5/VSSP pattern
           K5.head32[1]=(K5.head32[1] & 0xFFFFFFUL) | syncpat ;
           nwrite = fwrite( K5.head32, sizeof(u_int32_t), 2, fo);
           for(n=0; n<K5.imax; n++){
                K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
                if( ferror( K5.stream )  ) {
                    //printf("File Read Error!\n");
                    break;
                }
                nwrite = fwrite( K5.ubuf, sizeof(u_int32_t), K5.wordread, fo);
           }
           if((kword=CheckK5header_with_Read(&K5)) < 0){
                i=fclose( K5.stream ) ;
                goto exitend2;
           }
       }

 exitend2:
        i=fclose( fo ) ;
        free(K5.ubuf);
        printf("\n");
}


int CheckK5header_with_Read(K5VSSP_file_t *S)
{
    /* 
       Read header check to get sampling parameters 
    
       if first 64bit data is not header block, read file 
       untill finding header block.

       After call, stream pointer is set to just next to the header
       block.

       Return
               +ve   # of words read till header finding
               -1    no header found till EOF
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       S->headsizew=2; // for K5/VSSP
       //S->headsizew=8; // for K5/VSSP32

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       if ((i=CheckK5header(S)) == 0) {  // sync detection
           goto exit_loop;
       } else if(i==-99){
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}

int CheckK5header_with_Read2(K5VSSP_file_t *S, int maxword)
{
    /* 
       Read 64bit Data header check to get sampling parameters 
    
       if first 64bit data is not header block, read file at maximum 
       maxword 32bit-words untill finding header block.

       Return
               +ve   # of words  read till header finding
               -1    no header found after maxword read
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
       //                 &S->seconds,&S->hh,&S->mm,&S->ss);
       if ((i=CheckK5header(S)) == 0) {  // sync detection
           goto exit_loop;
       } else if(i==-99){
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
           if(i32+2 > maxword) return -1;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}


int fwdNsecK5(int nsec, K5VSSP_file_t *K5)
{
    /* 
       forward fast N sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind 1sec data
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one

    */
    int i,k;
    int maxword;
    int nbytes, nback;
    int64_t tword;
	int64_t pos;

    if(nsec < 1) return 0; // nothing to do

    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    nbytes=K5->bytesec*nsec;
    K5->pos+=nbytes;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,nbytes,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=nbytes/4;
    K5->twordread+=(int64_t)K5->wordread;


    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if((i=CheckK5header(&(*K5))) < 0){  // this is not the case of header
       if(i==-99) return -1;
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               if(nsec==1){
                   nback=(int)(K5->pos-pos);  // rewind to initial position
               } else {
                   nback=K5->bytesec;   // rewind just 1 sec
               }

               fseek(K5->stream,-nback,SEEK_CUR); // set position at initial position
               K5->twordread-=nback/4;            // word counter rewind
               K5->pos-=nback;  // initial position rewind
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

int fwd1secK5(K5VSSP_file_t *K5)
{
    /* 
       forward fast 1 sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind previous header position
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one
    */
    int i,k;
    int maxword;
    int64_t tword;
	int64_t pos;
    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    K5->pos+=K5->bytesec;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,K5->bytesec,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=K5->bytesec/4;
    K5->twordread+=(int64_t)K5->wordread;

    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if((i=CheckK5header(&(*K5))) < 0){  // this is not the case of header
       if(i==-99) return -1;
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
               K5->twordread=tword;            // word counter retrieve
               K5->pos=pos;  // retrieve initial position
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

int main(int argc, char *argv[])
{
    char fname[60];
    char oname[60];
    char* dflt_fname="tds.data";
    int mode;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    mode=0;
    if(argc < 2) {
       //strcpy(fname,dflt_fname);
       printf("k5tok5v32 %s\n",PROGRAM_VERSION);
       printf("How to run: \n");
       printf("  k5tok5v32 datafile [ofile]\n");
       printf("\n");
       printf("    where  datafile ----- K5/VSSP data file name to check\n");
       //printf("                          if 0 is given, internal dflt name will be used\n");
       printf("           ofile      --- outfile name (default datafile.v32)\n");
       exit(0);
    }
    if (argc>=2){
       strcpy(fname,*(argv+1));
    }
    oname[0]='\0';
    if(argc >=3){
       strcpy(oname,*(argv+2));
    }
    // oname creation
    if(oname[0]=='\0'){
        strcpy(oname,fname);
        strcat(oname,".k5");
    }
    cstart1sec = clock();
    printf("************ k5tok5v32  %s ************\n",PROGRAM_VERSION);
    printf("  format conversion start!\n");
    vssptovssp32(fname,oname);
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
