/*******************************************************************/
/*                                                                 */
/*   ho|ukahpCN[ht@C libipvlbi.h              */
/*          Tv[{[hANZXȊO                           */
/*                                                                 */
/*      Version:    2006-01-03                                     */
/*                                                                 */
/*      Copyright (c) 2002-2006 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*                                   
 ho|ukahpwb_[t@C  
      Tv[{[hANZXȊO   
           by T.Kondo 2001.8.26      
                      2002.5.26      
                      2002.7.06      
                      2003.6.16   hostget added
                      2003.10.12  definition of sampling frequency higher than
                                  or equal to 32MHz was added in "headerchkn"
                                  and "parsebit32n"
                      2005.6.10   divided into *.h and *.c
                      2005.12.28  support K5/VSSP32 data
                      2006.01.03  minor revision
*/

#ifndef _LIBIPVLBI_
#define _LIBIPVLBI_

#include "chead.h"  // common header 

#ifndef WIN32
#ifdef linux
  #include <sys/vfs.h>
#else
  #include <sys/param.h>
  #include <sys/mount.h>
#endif
#endif


#ifdef WIN32
//define gettimeofday to keep source code compatibility with linux/FreeBSD 
#ifndef _WINSOCKAPI_    /* to avoid duplicate include (winsock.h also include */
                        /*    struct timeval definition */
struct timeval {          
        long    tv_sec;         /* seconds */
        long    tv_usec;        /* and microseconds */
};
#endif
struct timezone {
        int    tz_minuteswest; /* minutes W of Greenwich */
        int    tz_dsttime;     /* type of dst correction */
};
int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif



typedef struct timeval_k5    // time related variables used in k5 utilities
{
    int year;      // 4 digit year
    int ddd;       // day of year
    int hh;        // hour
    int mm;        // minute
    int ss;        // sec in minute
    int second;    // sec in day
    double dtime;  // time in day (ddd+second/86400.0)
    int month;     // month
    int mday;      // day of month
} timeval_k5_t;


typedef struct listoutdir    //  (moved from libipvlbi_board.h)
{            
    int  dirnum;   // maximum number of directories
    int  ndir;     // next directory index#  2005-01-30
    char dir[10][120];
} listoutdir_t;

typedef struct logval    // structural variablle for log   (moved from libipvlbi_board.h)
{            
    char logdir[60];     // logfile directory
    char hostname[20];   // host name
    char logfile[80];    //log file name
    int logfiletype;     // logfile naming type
                         //  0: exocode_stationid.hostname.log
                         //  1: YYYY+hostname+".log"
                         //       where  YYYY is 4 digit year
                         //  -1: no log
} logval_t;

/* K5/VSSP and K5/VSSP32 format data handling type */
#define __LITTLE_ENDIAN_BITFIELD      // 2006-01-03
//#define __BIG_ENDIAN_BITFIELD

/* K5/VSSP and VSSP32 header definition */
struct VSSPHEADER {     // K5/VSSP and VSSP32 header   2006-01-03
    /* K5/VSSP portion (64bits) */
    u_int32_t syncptn1;		/* 0xffffffff */
    //unsigned int syncptn1;		/* 0xffffffff */
	u_int16_t totalsec0;
#if defined(__BIG_ENDIAN_BITFIELD)
    u_int16_t syncptn2 : 8;
	u_int16_t bitwidth : 2;
	u_int16_t fsample : 4;
	u_int16_t chmode : 1;
	u_int16_t totalsec1 : 1;
#endif
#if defined(__LITTLE_ENDIAN_BITFIELD)
	u_int16_t totalsec1 : 1;
	u_int16_t chmode : 1;
	u_int16_t fsample : 4;
	u_int16_t bitwidth : 2;
    u_int16_t syncptn2 : 8;    // use of unsigned char instead of uint16_t
                              //  results in nproper operation at Win32
#endif
	/* extended header portion for VSSP32 */
#if defined(__BIG_ENDIAN_BITFIELD)
	u_int16_t year : 7;
	u_int16_t totalday : 9;
    u_int16_t v_major  : 4;
    u_int16_t v_minor  : 4;
    u_int16_t auxsize  : 8;
#endif
#if defined(__LITTLE_ENDIAN_BITFIELD)
	u_int16_t totalday : 9;
	u_int16_t year : 7;
    u_int16_t auxsize  : 8;
    u_int16_t v_minor  : 4;
    u_int16_t v_major  : 4;
#endif
    /* AUX filed continued */
    u_int16_t fmt  : 8;     // AUX field format
    u_int16_t filter : 8;    // Filter info
    char sid2[2];       // 2 char Station ID
    char station[8];       // station name (A8)
    char hostname[8];       // Host PC name (A8)
#ifndef WIN32
} __attribute__((__packed__));
#else
};
#endif
typedef struct VSSPHEADER VSSPHEADER_T;

union HEAD_VSSP   //  2006-01-03
{
    unsigned char *bbuf;
    VSSPHEADER_T *hd;
};

/* K5/VSSP and K5/VSSP32 header information type */
typedef struct vssp_head  // VSSP header info
{
	int format; // =0 for old format , =1 K5/VSSP,  =2 K5/VSSP32
    unsigned char psync2;  // 2nd sync pattern (given)
                          //     0x8B for K5/VSSP
                          //     0x8C for K5/VSSP32
    unsigned char bhead[32];  // header up to 256 bits
    // Header info 
	int sfreq;   //sampling frequency (in kHz)
	int adbit;   // AD bit length
	int numch;   // # of CH
    int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
    // following is valid for K5/VSSP32
    int year;    // 3 digit year
    int totalday;    // total day
    int v_major; // Major version number
    int v_minor; // Minor version number
    int aux_size; // AUX field size in byte
    int aux_fmt;  // AUX field format
    int seconds; // second from 0h UTC
    int hh, mm, ss;
    char aux_sid2[3];       // 2 char Station ID
    char aux_station[9];       // station name (1st 8 char is valid)
    char aux_hostname[9];       // Host PC name (1st 8 char is valid)
} vssp_head_t;


typedef struct ip_vlbi_file  // moved from libcor.h on 2006-01-04
{
    /* headder info */
    vssp_head_t hd;

    /* followings  are to keep backward compatibility */
      /***************************************************/
        /* these info are included in vssp_head_t hd variable */
	    int sfreq;   //sampling frequency (in kHz)
	    int adbit;   // AD bit length
	    int numch;   // # of CH
	    int format; // =0 for old format , =1 K5/VSSP,  =2 K5/VSSP32
	    int spatn;   // sync pattern  (use hd.psync2)  

        /* use hd.bhead[32] instead of following buffers */
	    unsigned char bit32[4];
	    //unsigned char bit64[8];
	    unsigned char bit64[32];  // to include K5/VSSP32 header
      /***************************************************/
  
    /* file I/O variables */
	char* fname;   // file opened or to be opened
	FILE *stream;  // stream pointer
	unsigned char *ibuf;  // pointer to read buffer (char type)
	unsigned int bytesread;  // used for file I/O
    int open_flag;  // file open flag  
                    //   1: opened 
                    //   0: not opened or closed
	double dtimest;  // time (sec) at 1st header
    int seconds;     // time at recent header in seconds from 0h UTC
	int hh, mm, ss;  // time at recent header in hour, minute, second

    /* file read block info */
             //  unit is a block read at one time   
	int icnt;        //  current unit number in 1 sec data 
	int usampl;      // # of samples in a unit
	int numb;        //  # of bytes in a unit (=usampl*adbit*numch/8)
	int imax;        // # of units in a second data
	double fsample;  // sampling frequency (Hz)
	double tsample;  // sampling period
	double unitp;    // period (sec) of a unit (=usampl*tsampl)
	double dtime;    // time (sec) at begin of a unit

    /* data decoding */
    int *idata;  // pointer to buffer used for parsing each sampled data (int type)
	int kpos;    // valid number of data in idata array

	//int fh;  // not used
	//int ch;  // not used
    
    /* flags */
    int vflag;       // byte slip flag
                     //   1: sync detection success 
                         //   0: sync detection error (may be byte slip occurr)
    int vflag_old;       // Previous status of vflag
    int dtime_vflag;     // time (sec) at vflag decision made
    int dtime_vflag_old; // time (sec) at vflag_old decision made
	int ierr;        // general error code
} ip_vlbi_file_t;



/* K5/VSSP32 AUX field type used for set sampler */
typedef struct vsspaux  // VSSP32 AUX field type  for K5/VSSP32  2005-12-28 (moved from libipvlbi_board.h)
{
    int size;    // AUX filed size
    int fmt;     // AUX field format
    int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
    char bdat[256];   // AUX field data
    char sid[2];        // 1 char Station ID (no more use)
    char sid2[3];       // 2 char Station ID
    char station8[20];       // station name (1st 8 char is valid)
    char hostname8[20];       // Host PC name (1st 8 char is valid)
} vsspaux_t;

/* K5/VSSP and K5/VSSP32 sampler parameter type */
typedef struct vssp_param   // sampler parameter type variable  (moved from libipvlbi_board.h)
{
    int sfreq;
    int adbit;
    int numch;
    int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
    int freqkhz;  // frequency in kHz  (used for only return param)
    vsspaux_t vsspaux;  // AUX filed data for K5/VSSP32 2005-12-28
                        // vsspaux_t is defined in libipvlbi.h
} vssp_param_t;

/* structual variable for "sampleget" function */
typedef struct spget   // sampleget type variable  (moved from libipvlbi_board.h)
{
    int span;
    //int sfreq;
    //int adbit;
    //int numch;
    //int filter;  // filter frequency(MHz)  0: no use of K5/VSSP32
    vssp_param_t sp; 
    int f_size_mode;  // 0: no limitation on file size  
                      // 1: divided into 2GB size 
    char outfile[120];
    char fname[120]; 
    char outdir[120];
    listoutdir_t outdirtab;  //2005-01-30
    logval_t lg;
} spget_t;



int cmnd_get(int argc, char *argv[], char *command, char *param);
char *select_env_dflt(char *env, char *dflt);
void parse_dir_and_name_fm_fpath(char *fpath, char *dir, char *fname); 
int getenv_from_path(char *path, char *env, int nth);
int substitute_env_path_name(char *path, char *fpath);
void get_true_path(char *path, char *dfltenv, char *dfltdir2, char *fpath);


void hostget(char* hostname); // host name get using system command
long int nint(double x);
void read1line(char *line);
void sec2hms(long int sec, int* hh, int* mm, int* ss);
int leaf_year_check(int year);   // N̂΂O^[
void ddd2md(int year, int ddd, int* month, int* mday);
void md2ddd(int year, int* ddd, int month, int mday);
void timecorrect(int* year, int* month, int* mday,
                int* hh, int* mm, int* ss, int* secadd);
void parsebit32n(unsigned char* bit32, int* spatn, 
				int* smplbit, int* sfreq, int* numch,
				long int* seconds);
void headerchkn(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
			   int* sfreq, int* numch, int* seconds, int* hh,
			   int* mm, int* ss);
int headerchkn2(vssp_head_t *vhd); //2006-01-03
void headerchkn_old(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
			   int* sfreq, int* numch, long int* seconds, int* hh,
			   int* mm, int* ss);
void parseADdata(u_int32_t *ia, int numb, int adbit, int *xdata, int *npos);
void parseADdata_2(unsigned char *ia, int numb, int adbit, int *xdata, int *npos);
void parseADdata_3(unsigned char *ia, int numb, int adbit, int *xdata, int *npos);
void tk_sleep(int msec);  // suspend msec 

// following 2 functions uses "timeval" structure that is available only
//     on FreeBSD or linux.  So emulation is included in chead.h for WIN32
// 
void sec2ymds(struct timeval *tv, int *year, int* month, int* mday, int* hh, int* mm, int* ss);
void getsystimef(int* year, int* month, int* mday, int* hh, int* mm, int* ss, 
                      float *fsec, float *tofset);

// following function returns valid value only in case of linux/FreeBSD
// fixed value is returned for WIN32 just to emulate the program on Win32
double GetFreeInfo(char* dirnam);

void tmval_set(int *itime, timeval_k5_t *t);
void tmval_update_second(timeval_k5_t *t);  // convert t.seconds to time val
void tmval_update_md2ddd(timeval_k5_t *t);
void tmval_update_ddd2md(timeval_k5_t *t);


void vssp_aux_make(vsspaux_t *vsspaux);  // moved from libipvlbi_board.c

#endif
