/******************************************************************/
/*                                                                */
/*   ho|ukahpCN[h     libipvlbi_board.c           */
/*          Tv[{[h֌W                                  */
/*                                                                */
/*      Version:    2002-10-08                                    */
/*      Version:    2005-12-26                                    */
/*                                                                */
/*      Copyright (c) 2002-2005 T.Kondo/NICT All Right Reserved   */
/*                                                                */
/******************************************************************/
/*                                   
  K5/VSSP (IP-VLBI) sampler board  functions
         by T.Kondo 2001-08-26      
                    2002-05-26      
                    2002-10-21  sampleget was modified to handle file > 2GB      
                    2003-06-18   LogWrite2, LogWrite3 added
                    2003-08-09   miner message change in sampleget
                    2003-08-21   toffset_get is added
                    2004-05-23   LogWrite4 was added
                                sampleget was newnewed (use of structual variable as 
                                input parameter, and add log out function)
                                former sampleget was renamed as sampleget_old
                    2005-02-12   samplget was modified to more robust against disk full
                    2005-02-17   debug in samplget (cbuf[81]->cbuf[120]) to avoid
                                the error occurred at the case of very long file name
                    2005-02-12   libipvlbi_board.h was devided into *.h and *.c
                    2005-12-26  support VSSP32
*/

#include "libipvlbi_board.h"

/*---------  VSSP32 functions --------------*/

//void vssp_aux_make(vsspaux_t *vsspaux)  // moved to libipvlbi.c
//{
///*  generate AUX field data */
//    int n,len;
//    char cbuf1[8],cbuf2[8];
//    /* fill space exceeding 8char portion */
//    for(n=0; n<8; n++){
//        cbuf1[n]=' ';
//        cbuf2[n]=' ';
//    }
//    len=strlen(vsspaux->station8);
//    for(n=0; n<8 && n<len; n++) cbuf1[n]=vsspaux->station8[n];
//    len=strlen(vsspaux->hostname8);
//    for(n=0; n<8 && n<len; n++) cbuf2[n]=vsspaux->hostname8[n];
//    
//    /* format field set */
//    vsspaux->bdat[0]= (char)(vsspaux->fmt & 0xFF);
//    /* aux field set */
//    if(vsspaux->fmt==1){
//       /* station field set */
//       vsspaux->bdat[1]=vsspaux->sid[0];
//       vsspaux->bdat[2]=vsspaux->sid2[0];
//       vsspaux->bdat[3]=vsspaux->sid2[1];
//       for(n=0; n<8; n++) vsspaux->bdat[4+n]=cbuf1[n];
//       /* hostname field set */
//       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
//    } else if(vsspaux->fmt==2){
//       /* fill out data with 0x55 */
//       for(n=0; n<11; n++) vsspaux->bdat[1+n]= 0x55;
//       /* hostname field set */
//       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
//    } else if(vsspaux->fmt==85){
//       /* fill out all data with 0x55 */
//       for(n=0; n<19; n++) vsspaux->bdat[1+n]= 0x55;
//    } else if(vsspaux->fmt==170){
//       /* fill out all data with 0xAA */
//       for(n=0; n<19; n++) vsspaux->bdat[1+n]= 0xAA;
//    } else {
//       /* format 0 is adapted */
//       vsspaux->fmt=0;
//       /* fill out all data with 0x00 */
//       for(n=0; n<20; n++) vsspaux->bdat[n]= 0x00;
//    }
//}

int check_vssp(int fd_in, unsigned int *stat, int verbose)
{
/*     check reference signals of  K5/VSSP or K5/VSSP32
    [in]
       fd_in  --  device file descripter for K5/VSSP or K5/VSSP32
       verbose --  0: no message out   1: message out
    [return]
        0: OK   
        -1: both ref signal and 1PPS is not supplied or PLL unlocked | get status failed
        -2: only 1PPS is not supplied
        stat ---  detail status 
              bit0 :  reference signal supplied
              bit1 :  1PPS supplied
              bit2 :  reference signal is 5MHz (valid for K5/VSSP32)
              bit3 :  PLL is locked (valid for K5/VSSP32)
              bit4 :  this is K5/VSSP32  (valid for K5/VSSP32)
        
*/
    int rv;
    unsigned int sampling1, freq, width, stat0;
    unsigned int ref, pps, rhz, pll;

    *stat=0;
    /* board status check */
    rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat0);
    if (rv == -1) {
        //if(verbose==1)
           perror("  check_vssp: ioctl() status get failed ");
        return -1;
    }
    /* satatus get again just in case */
    rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat0);
    if (rv == -1) {
        //if(verbose==1)
           perror("  check_vssp: ioctl() status get failed ");
        return -1;
    }
    //printf("ioctl() status get succeed\n");
    // printf(" status = %X\n",stat);
#ifdef _VSSP32    
    /* set stat again to keep compatibility with K5/VSSP */
    ref=0; rhz=0; pps=0; pll=0;
    if((stat0 & 4) == 4) pps=1;        // 1PPS supplied
    if((stat0 & 8) == 8) ref=1;        // reference signal supplied
    if((stat0 & 0x20) == 0x20) rhz=1;  // reference signal is 5MHz
    if((stat0 & 0x10) == 0x10) pll=1;  // PLL locked
    stat0=0;
    stat0|=0x10;  // means this is VSSP32
    if(ref==1) stat0|=1;
    if(pps==1) stat0|=0x2;
    if(rhz==1) stat0|=0x4;
    if(pll==1) stat0|=0x8;
#else
    stat0&=3;  // 2005-12-20
#endif
    *stat=stat0;
    if((stat0 & 0x10) == 0x10){ // this is VSSP32
        if((stat0 & 1) != 1){
            if(verbose==1)
              printf("  check_vssp: Warning! 10MHz or 5MHz signal is not supplied. \n");
        } else {
            if((stat0 & 4) == 4){
                if(verbose==1)
                   printf("  check_vssp: Reference (5MHz) signal is OK!");
            } else {
                if(verbose==1)
                   printf("  check_vssp: Reference (10MHz) signal is OK!");
            }
            if((stat0 & 8) == 8){
                if(verbose==1)
                   printf("  check_vssp: PLL locked!\n");
            } else {
                if(verbose==1)
                   printf("  check_vssp: Warning! PLL unlocked!\n");
                return -1;
            }
        }
    } else {   // this is VSSP
        if((stat0 & 1) != 1){
            if(verbose==1)
               printf("  check_vssp: Warning! 10MHz signal is not supplied. \n");
        } else {
            if(verbose==1)
               printf("  check_vssp: 10MHz signal is OK!\n");
        }
    }
    if((stat0 & 2) != 2){
        if(verbose==1)
           printf("  check_vssp: Warning! 1PPS signal is not supplied. \n");
    } else {
        if(verbose==1)
           printf("  check_vssp: 1PPS  signal is OK!\n");
    }
    if((stat0 & 3) == 0){
    //   printf("  check_vssp:    Check signal connection and try again!\n");
         return -1;
    }
    if((stat0 & 3) == 1){
    //   printf("  check_vssp:    Anyway continue the process!\n");
         return -2;
    }
    return 0;
}

//int set_vssp_sparam(int fd_in, int sfreq, int adbit, int numch, int filter, int verbose)
int set_vssp_sparam(int fd_in, vssp_param_t *sp, int verbose)
{
/*     set sampling parameters for K5/VSSP or K5/VSSP32
    [in]
       fd_in  --  device file descripter for K5/VSSP or K5/VSSP32
       sp->sfreq  --  frequency parameter
       sp->adbit  --  AD bits
       sp->numch  --  # of CH
       sp->filter --  filter frequency (MHz) for K5/VSSP32  (0 means through)
       verbose --  0: no message out   1: message out
    [return]
        0: OK   -1: failed
*/
    int rv;
    unsigned int sfreq_u, sbit_u, numch_u, filter_u;  // for K5/VSSP32
    u_int8_t sampling, sfreq_t, sbit_t, numch_t;                // for K5/VSSP
    

#ifdef _VSSP32
    /* sampling frequency set */
    if      (sp->sfreq ==  40)  { sfreq_u =  TDSIO_SAMPLING_40KHZ; }
    else if (sp->sfreq == 100)  { sfreq_u =  TDSIO_SAMPLING_100KHZ; }
    else if (sp->sfreq == 200)  { sfreq_u =  TDSIO_SAMPLING_200KHZ; }
    else if (sp->sfreq == 500)  { sfreq_u =  TDSIO_SAMPLING_500KHZ; }
    else if (sp->sfreq ==   1)  { sfreq_u =  TDSIO_SAMPLING_1MHZ; }
    else if (sp->sfreq ==   2)  { sfreq_u =  TDSIO_SAMPLING_2MHZ; }
    else if (sp->sfreq ==   4)  { sfreq_u =  TDSIO_SAMPLING_4MHZ; }
    else if (sp->sfreq ==   8)  { sfreq_u =  TDSIO_SAMPLING_8MHZ; }
    else if (sp->sfreq ==  16)  { sfreq_u =  TDSIO_SAMPLING_16MHZ; }
    else if (sp->sfreq ==  32)  { sfreq_u =  TDSIO_SAMPLING_32MHZ; }
    else if (sp->sfreq ==  64)  { sfreq_u =  TDSIO_SAMPLING_64MHZ; }
    else { 
       //if(verbose==1){
          printf("  set_vssp_sparam: parameter sfreq  (sampling frequency) must be\n");
          printf("                   either one of followings:\n");
          printf("                   40,100,200,500 (for kHz)\n");
          printf("                   1,2,4,8,16,32,64 (for MHz)\n");
       //}
       return -1;
    }
    if ((rv = ioctl(fd_in, TDSIO_SET_FSAMPLE, &sfreq_u)) != 0) {
       //if(verbose ==1)
          perror("  set_vssp_sparam: ioctl() sampling rate set failed ");
       return -1;
    } else {
       if(verbose ==1)
          printf("  set_vssp_sparam: ioctl() sampling rate set succeed\n");
    }
    /* AD resolution set */
    if      (sp->adbit ==  1)  { sbit_u =  TDSIO_SAMPLING_1BIT; }
    else if (sp->adbit ==  2)  { sbit_u =  TDSIO_SAMPLING_2BIT; }
    else if (sp->adbit ==  4)  { sbit_u =  TDSIO_SAMPLING_4BIT; }
    else if (sp->adbit ==  8)  { sbit_u =  TDSIO_SAMPLING_8BIT; }
    else { 
       //if(verbose==1){
           printf("  set_vssp_sparam: parameter adbit (A/D resolution) must be\n");
           printf("                   either one of followings:  1, 2, 4, 8\n");
       //}
       return -1;
    }
    
    if ((rv = ioctl(fd_in, TDSIO_SET_RESOLUTIONBIT, &sbit_u)) != 0) {
       //if(verbose ==1)
          perror("  set_vssp_sparam: ioctl() AD resolution set failed ");
       return -1;
    } else {
       if(verbose ==1)
          printf("  set_vssp_sparam: ioctl() AD resolution set suceed\n");
    }
    /* number of channels */
    if      (sp->numch ==  1)  { numch_u =  TDSIO_SAMPLING_1CH; }
    else if (sp->numch ==  4)  { numch_u =  TDSIO_SAMPLING_4CH; }
    else { 
       //if(verbose==1){
          printf("  set_vssp_sparam: parameter numch (number of channels) must be\n");
          printf("                   either one of followings:   1, 4\n");
       //}
       return -1;
    }
    if ((rv = ioctl(fd_in, TDSIO_SET_CHMODE, &numch_u)) != 0) {
       //if(verbose ==1)
          perror("  set_vssp_sparam: ioctl() channels set failed ");
       return -1;
    } else {
       if(verbose ==1)
          printf("  set_vssp_sparam: ioctl() channels set succeed\n");
    }
    /* filter set */
    filter_u =  TDSIO_SAMPLING_THRU;
    if      (sp->filter ==   2)  { filter_u =  TDSIO_SAMPLING_2M; }
    else if (sp->filter ==   4)  { filter_u =  TDSIO_SAMPLING_4M; }
    else if (sp->filter ==   8)  { filter_u =  TDSIO_SAMPLING_8M; }
    else if (sp->filter ==  16)  { filter_u =  TDSIO_SAMPLING_16M; }
    if ((rv = ioctl(fd_in, TDSIO_SET_FILTER, &filter_u)) != 0) {
       //if(verbose ==1)
          perror("  set_vssp_sparam: ioctl() filter set failed ");
       return -1;
    } else {
       if(verbose ==1)
          printf("  set_vssp_sparam: ioctl() filter set succeed\n");
    }
#else
    /* sampling frequency set */
    if      (sp->sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
    else if (sp->sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
    else if (sp->sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
    else if (sp->sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
    else if (sp->sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
    else if (sp->sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
    else if (sp->sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
    else if (sp->sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
    else if (sp->sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
    else { 
       //if(verbose==1){
          printf("  set_vssp_sparam: parameter sfreq  (sampling frequency) must be\n");
          printf("                   either one of followings:\n");
          printf("                   40,100,200,500 (for kHz)\n");
          printf("                   1,2,4,8,16 (for MHz)\n");
       //}
       return -1;
    }
      
    /* AD resolution set */
    if      (sp->adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
    else if (sp->adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
    else if (sp->adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
    else if (sp->adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
    else { 
       //if(verbose==1){
           printf("  set_vssp_sparam: parameter adbit (A/D resolution) must be\n");
           printf("                   either one of followings:  1, 2, 4, 8\n");
       //}
       return -1;
    }

    /* number of channels */
    if      (sp->numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
    else if (sp->numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
    else { 
       //if(verbose==1){
          printf("  set_vssp_sparam: parameter numch (number of channels) must be\n");
          printf("                   either one of followings:   1, 4\n");
       //}
       return -1;
    }

    /* sampling parameters set */
    
    sampling =  sfreq_t | sbit_t | numch_t;

    //sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;

    rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
    if (rv == -1) {
       //if(verbose==1)
          perror("  set_vssp_sparam: ioctl() sampling set failed ");
       return -1;
    } else {
       if(verbose==1)
          printf("  set_vssp_sparam: ioctl() sampling set succeed\n");
    }
#endif
    return 0;
}


//int get_vssp_sparam(int fd_in, int *freqkhz, int *adbit, int *numch, int *filter, int verbose)
int get_vssp_sparam(int fd_in, vssp_param_t *sp, int verbose)
{
/* get sampling parameters from a board 
    [in]
       fd_in  --  device file descripter for K5/VSSP or K5/VSSP32
       verbose --  0: no message out   1: message out
    [return]
       0: OK   -1: failed
       sp->freqkhz  --  frequency (kHz)
       sp->adbit  --  AD bits
       sp->numch  --  # of CH
       sp->filter --  filter frequency (MHz) for K5/VSSP32  (0 means through)
*/
    int rv;
    unsigned int sampling1, freq, width, ufilter;
    int ch;
#ifdef _VSSP32
    /* get sampling frequency */
    if((rv = ioctl(fd_in,TDSIO_GET_FSAMPLE,&freq)) == -1){
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get sampling frequency failed ");
        return -1;
    } else {
        if(freq == TDSIO_SAMPLING_40KHZ)  sp->freqkhz = 40;
        else if(freq == TDSIO_SAMPLING_100KHZ) sp->freqkhz = 100;
        else if(freq == TDSIO_SAMPLING_200KHZ) sp->freqkhz = 200;
        else if(freq == TDSIO_SAMPLING_500KHZ) sp->freqkhz = 500;
        else if(freq == TDSIO_SAMPLING_1MHZ)   sp->freqkhz = 1000;
        else if(freq == TDSIO_SAMPLING_2MHZ)   sp->freqkhz = 2000;
        else if(freq == TDSIO_SAMPLING_4MHZ)   sp->freqkhz = 4000;
        else if(freq == TDSIO_SAMPLING_8MHZ)   sp->freqkhz = 8000;
        else if(freq == TDSIO_SAMPLING_16MHZ)  sp->freqkhz = 16000;
        else if(freq == TDSIO_SAMPLING_32MHZ)  sp->freqkhz = 32000;
        else if(freq == TDSIO_SAMPLING_64MHZ)  sp->freqkhz = 64000;
    }
    /* get filter frequency */
    if((rv = ioctl(fd_in,TDSIO_GET_FILTER,&ufilter)) == -1){
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get filter parameter failed ");
        return -1;
    } else {
        if(ufilter == TDSIO_SAMPLING_2M)  sp->filter = 2;
        else if(ufilter == TDSIO_SAMPLING_4M)  sp->filter = 4;
        else if(ufilter == TDSIO_SAMPLING_8M)  sp->filter = 8;
        else if(ufilter == TDSIO_SAMPLING_16M)  sp->filter = 16;
        else if(ufilter == TDSIO_SAMPLING_THRU)  sp->filter = 0;
    }
    /* get AD resolution bit */
    if((rv = ioctl(fd_in,TDSIO_GET_RESOLUTIONBIT,&width)) == -1){
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get AD resolution failed ");
        return -1;
    } else {
        if(width == TDSIO_SAMPLING_1BIT)  sp->adbit = 1;
        else if(width == TDSIO_SAMPLING_2BIT)  sp->adbit = 2;
        else if(width == TDSIO_SAMPLING_4BIT)  sp->adbit = 4;
        else if(width == TDSIO_SAMPLING_8BIT)  sp->adbit = 8;
    }
    /* get # of CH */
    if((rv = ioctl(fd_in,TDSIO_GET_CHMODE,&ch)) == -1){
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get # of CH failed ");
        return -1;
    } else {
        if(ch == TDSIO_SAMPLING_1CH)  sp->numch = 1;
        else if(ch == TDSIO_SAMPLING_4CH)  sp->numch = 4;
    }
    
#else
    sp->filter=0;
    if (rv == -1) {
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get sampling parameter failed ");
        return -1;
    }

    rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
    if (rv == -1) {
        //if(verbose==1)
           perror("  get_vssp_sparam: ioctl() get sampling parameter failed ");
        return -1;
    }
    else {
        if(verbose==1)
           printf("  get_vssp_sparam: ioctl() get sampling parameter succeed\n");
        freq = TDS_GET_SAMP_FREQ(sampling1);
        width = TDS_GET_SAMP_WIDTH(sampling1);
        ch = TDS_GET_SAMP_CH(sampling1);
        if(freq == TDS_SAMPLING_40KHZ)  sp->freqkhz = 40;
        if(freq == TDS_SAMPLING_100KHZ) sp->freqkhz = 100;
        if(freq == TDS_SAMPLING_200KHZ) sp->freqkhz = 200;
        if(freq == TDS_SAMPLING_500KHZ) sp->freqkhz = 500;
        if(freq == TDS_SAMPLING_1MHZ)   sp->freqkhz = 1000;
        if(freq == TDS_SAMPLING_2MHZ)   sp->freqkhz = 2000;
        if(freq == TDS_SAMPLING_4MHZ)   sp->freqkhz = 4000;
        if(freq == TDS_SAMPLING_8MHZ)   sp->freqkhz = 8000;
        if(freq == TDS_SAMPLING_16MHZ)  sp->freqkhz = 16000;
        if(width == TDS_SAMPLING_1BIT)  sp->adbit = 1;
        if(width == TDS_SAMPLING_2BIT)  sp->adbit = 2;
        if(width == TDS_SAMPLING_4BIT)  sp->adbit = 4;
        if(width == TDS_SAMPLING_8BIT)  sp->adbit = 8;
        if(ch == TDS_SAMPLING_4CH){
            sp->numch = 4;
        } else {
            sp->numch =1;
        }
    }   
#endif
    return 0;
}

int vssp_wait_nsec(int fd_in, int nsec)
{
/*
    wait N seconds using VSSP board data
*/
    unsigned int time,day,sec; 
    unsigned int seco;
    int ns;
    int rv;

    seco=-99;
    ns=-1;
    /* board time get */
 loop:
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
    day=TDS_GET_DAYS(time);
    sec=TDS_GET_SEC(time);
    if(sec != seco) {
       seco=sec;
       ns++;
       if(ns == nsec) goto exithere;
    }
    goto loop;
 exithere:
    return 0;
}



/*---------- END of VSSP32 Functions -------*/




/* Tv[{[h */
//int ini_sampler(int *fd_in,int sfreq, int adbit, int numch, long int *bitsec)
int ini_sampler(int *fd_in, vssp_param_t *sp, int *bitsec)
{
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    char cbuf[128];
    //int fd_in;
    int rv;
    int i;
    int headbit;
    //int headbit=64;  // header bit number  for a new sampler board
    long int allbyte;     // TvOoCg
    //long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    unsigned int sampling1, freq, width;
    int ch;
    u_int8_t sfreq_t, sbit_t, numch_t, sampling;
#ifdef _VSSP32
    struct tdsauxio tdsaux;
#endif
    
    headbit=64;  // header bit number  for K5/VSSP

    /* sampler open */
    *fd_in = open(dev, O_RDONLY);
    if (*fd_in == -1) {
        perror("  ini_sampler: device open() error ");
        return -1;
    }

#ifdef _VSSP32  // just for the case of VSSP32 only
    if((rv=ioctl(*fd_in, TDSIO_INIT)) == -1){
        perror("  ini_sampler: ioctl() INIT failed ");
        goto end;
    } else {
        printf("  ini_sampler: ioctl() INIT ");
    }
    /* set AUX field data */
    if(sp->vsspaux.size > 0) {    // default size should be 20
        tdsaux.auxsize=sp->vsspaux.size;
        for(i=0; i<tdsaux.auxsize; i++) tdsaux.auxfield[i]=sp->vsspaux.bdat[i];
        if((rv=ioctl(*fd_in, TDSIO_SET_AUX, &tdsaux)) == -1){
            perror("  ini_sampler: ioctl() SET_AUX failed ");
            goto end;
        } else {
            printf("SET_AUX ");
        }
    }
    /* get AUX field data */
    if((rv=ioctl(*fd_in, TDSIO_GET_AUX, &tdsaux)) == -1){
       perror("  ini_samplert: ioctl() GET_AUX failed ");
        goto end;
    } else {
            printf("GET_AUX ");
    }
    printf("succeed\n");
    headbit=64+32+8*tdsaux.auxsize;   // header bit calc
    if(headbit != 256){
        printf("  ini_sampler:Warning!! header bits (%d) is different from default value (256)\n",headbit);
    } else {
        printf("  ini_sampler: header bits = %d\n",headbit);
    }
#endif

    /* sampling parameters set */
    //if((rv=set_vssp_sparam(fd_in,sfreq,adbit,numch,filter,0))==-1){
    if((rv=set_vssp_sparam(*fd_in, sp,0))==-1){
        printf("  ini_sampler: sampling parameter set failed \n");
        goto end;
    }
    
    
    if (sp->sfreq == 40 || sp->sfreq == 100 || sp->sfreq == 200 || sp->sfreq == 500 ) {     //2005-12-21
        *bitsec=sp->sfreq*1000 ;
    } else { 
        *bitsec=sp->sfreq*1000*1000;
    }

    *bitsec=(*bitsec*sp->adbit*sp->numch)+headbit;   // bit number per second

    bytesec=*bitsec/8;


    /* get sampling parameters from a board */
    if ((rv=get_vssp_sparam(*fd_in, sp,0))== -1) {
        perror("  ini_sampler: ioctl() get sampling parameter failed ");
        goto end;
    }  

#ifdef _VSSP32
    sprintf(cbuf,"ini_sampler: sampling parameters are %d kHz (filter %d MHz) %d BIT %d CH",
                         sp->freqkhz,sp->filter,sp->adbit,sp->numch);
#else
    sprintf(cbuf,"ini_sampler: sampling parameters are %d kHz %d BIT %d CH",
                         sp->freqkhz,sp->adbit,sp->numch);
#endif
    printf("  %s\n",cbuf);

end:
    return rv;
}


/* TvOX^[g */
int start_sampler(int *fd_in)
{
    int rv;
       
    rv = ioctl(*fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("  start_sampler : ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("  start_sampler : ioctl() FIFO clear succeed\n");
    }

    /* sampling start */

    rv = ioctl(*fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("  start_sampler : ioctl() start sampling failed ");
        printf("                  Probably time was not set for the IP-VLBI board.\n");
        printf("                  Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("  start_sampler : ioctl() start sampling succeed\n");
    }
end:
    return rv;
}


/* TvOXgbv */
int stop_sampler(int *fd_in)
{
    int rv;

    rv = ioctl(*fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("  stop_sampler : ioctl() stop sampling failed ");
    } else {
        printf("  stop_sampler : ioctl() stop sampling succeed\n");
    }

    close(*fd_in);
       
    return rv;
}


/* Tv[{[hуf[^WgK[ */
//int ini_and_start_sampler(int *fd_in,int sfreq, int adbit, int numch, long int *bitsec)
int ini_and_start_sampler(int *fd_in, vssp_param_t *sp, int *bitsec)
{
    char dev[] = DEV_FILE;
    //int fd_in;
    int rv;

/*
    printf(" span = %d\n",span);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
*/

    //rv=ini_sampler(&(*fd_in),sfreq, adbit, numch, &(*bitsec));
    rv=ini_sampler(fd_in, sp, bitsec);
    if (rv < 0) {
        printf("  ini_and_start_sampler : sampler initialize error \n");
        return rv;
    }

    rv=start_sampler(fd_in);

    if (rv < 0) {
        printf("  ini_and_start_sampler : start sampling failed ");
    }
    
    return rv;
}


/*  f[^W֐
   NۂɃf[^W܂łRbII
    outfile ---- f[^o̓t@C
    span    ---- f[^Wԁ@ibj
    sfreq   ---- TvOg
                    40,100,200,500 (for kHz)
                    1,2,4,8,16 (for MHz)
    adbit   ---- `crbg
                    1,2,4,8
    numch   ---- `l
                    1, 4
    2003-11-20 no limitation on file size
*/
int sampleget_big(char* outfile, int span, int sfreq, int adbit, int numch)
{
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit=64;  // header bit number  for a new sampler board
    //int headbit=32;  // header bit number  for a old sampler board
    long int allbyte;     // TvOoCg
    long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    long int c;
    unsigned int sampling1, freq, width;
    int ch;
    // 2002-10-21
    double dallbyte, dc;
    int f_c;   // file number counter  in case of file exceeds > 2Gbyte
    char cbuf[10];
    char outname2[80]; // for extra output
    // ----------
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    u_int32_t buf[BUFSIZ * 2];
        //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI

   /*
    printf(" span = %d\n",span);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
   */

       /* sampling duration check */
       if (span <= 0) {
          printf(" parameter span (duration in sec) should be positive!\n");
          // nothing to do
          return -1;
       }


       /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }

       /* output file open */
    fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
    if (fd_out == -1) {
        perror("output file open() error");
        goto end;
    }

       /* sampling frequency set */

       if      (sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
       else if (sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
       else if (sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
       else if (sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
       else if (sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
       else if (sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
       else if (sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
       else if (sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
       else if (sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
       else { 
       printf(" parameter sfreq  (sampling frequency) must be either one of followings:\n");
       printf("              40,100,200,500 (for kHz)\n");
       printf("              1,2,4,8,16 (for MHz)\n");
          return -1;
       }
      
       /* AD resolution set */
       if      (adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
       else if (adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
       else if (adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
       else if (adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
       else { 
       printf(" parameter adbit (A/D resolution) must be either one of followings:\n");
       printf("              1,2,4,8\n");
          return -1;
       }

       /* number of channels */
       if      (numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
       else if (numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
       else { 
       printf(" parameter numch (number of channels) must be either one of followings:\n");
       printf("              1,4\n");
          return -1;
       }

       /* sampling parameters set */
       sampling =  sfreq_t | sbit_t | numch_t;

       //if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}   
       //else { bitsec=sfreq*1000*1000 + headbit ;}
       if (sfreq >= 40) { bitsec=sfreq*1000 ;}          //2002-9-15
       else { bitsec=sfreq*1000*1000 ;}                 //2002-9-15

       //bitsec=bitsec*adbit*numch;   // bit number per second
        bitsec=(bitsec*adbit*numch)+headbit;   // bit number per second  !! 2002-9-15

       bytesec=bitsec/8;
       allbyte=bytesec*span;
       dallbyte=(double)bytesec*(double)span;   // 2002-10-21

       /* File maximum bytes check */
       //if (allbyte > FILEMAXSIZE){
 //      if (dallbyte > (double)FILEMAXSIZE){  // 2002-10-21
 //         allbyte= FILEMAXSIZE;
 //         secforce=allbyte/bytesec;
 //         printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
//              allbyte,FILEMAXSIZE);  // debug
//          printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
//        }
       printf("  sampleget: bits/sec=%d  bytes/sec=%d  all_bytes=%g\n",bitsec, bytesec,dallbyte);  // debug

    //sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;

    rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
    if (rv == -1) {
       perror("  sampleget: ioctl() sampling set failed ");
       goto end;
    }
    else {
       printf("  sampleget: ioctl() sampling set succeed\n");
    }


       /* get sampling parameters from a board */

    rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
    if (rv == -1) {
        perror("  sampleget: ioctl() get sampling parameter failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() get sampling parameter succeed\n");
        freq = TDS_GET_SAMP_FREQ(sampling1);
        width = TDS_GET_SAMP_WIDTH(sampling1);
        ch = TDS_GET_SAMP_CH(sampling1);
        if(freq == TDS_SAMPLING_40KHZ)
            freq = 40;
        if(freq == TDS_SAMPLING_100KHZ)
            freq = 100;
        if(freq == TDS_SAMPLING_200KHZ)
            freq = 200;
        if(freq == TDS_SAMPLING_500KHZ)
            freq = 500;
        if(freq == TDS_SAMPLING_1MHZ)
            freq = 1000;
        if(freq == TDS_SAMPLING_2MHZ)
            freq = 2000;
        if(freq == TDS_SAMPLING_4MHZ)
            freq = 4000;
        if(freq == TDS_SAMPLING_8MHZ)
            freq = 8000;
        if(freq == TDS_SAMPLING_16MHZ)
            freq = 16000;
        if(width == TDS_SAMPLING_1BIT)
            width = 1;
        if(width == TDS_SAMPLING_2BIT)
            width = 2;
        if(width == TDS_SAMPLING_4BIT)
            width = 4;
        if(width == TDS_SAMPLING_8BIT)
            width = 8;
        if(ch == TDS_SAMPLING_4CH)
            ch = 4;
        else
            ch =1;
        printf("  sampleget: sampling parameters are %d kHz %d BIT %d CH\n",freq,width,ch);
    }   



       /* board time get */
 /*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
    //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
    printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("  sampleget: ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() FIFO clear succeed\n");
    }
/*       
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);           // Poorւ̓BV{[h(2002.3ȍ~)ł
    if (rv == -1) {                               // ȂĂ̍ŏɂPΗǂ
        perror("ioctl() 1pps sync failed ");   //  (Ȃǂj
        goto end;
    }
    else {
        printf("ioctl() 1pps sync succeed\n");
    }
*/
/*  rv = ioctl(fd_in, TDSIO_SYNC_TIME);            // O͎͂gpȂ
    if (rv == -1) {
        perror("ioctl() time sync failed ");
        goto end;
    }
    else {
        printf("ioctl() time sync succeed\n");
    }
*/

       /* sampling start */

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("  sampleget: ioctl() start sampling failed ");
              printf("  sampleget:  Probably time was not set for the IP-VLBI board.\n");
              printf("  sampleget:  Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() start sampling succeed\n");
    }
        printf("  sampleget: This is new version (2002-10-21) of sampleget\n");
    c = 0;
    dc=0.0;   // 2002-10-21
    f_c=0;    // 2002-10-21
    //while (c <=  allbyte) {
    while (1) {                   // 2002-10-21
       int nbyte = sizeof(buf);   // nbyte = 8192

       rv = read(fd_in, buf, nbyte);
       if (rv == -1) {
          perror("  sampleget: read() failed ");
          goto end;
       }
          //printf(" nbyte = %d\n",nbyte);   // debug
       rv = write(fd_out, buf, rv);
       c= c+ nbyte;
    /* add on 2002-10-21 */
        dc+=(double)nbyte;
        if( dc > dallbyte) {   // sampling end
            goto exithere;
        }
//      if(c >  FILEMAXSIZE) { // newfile creation
//          c=0;
//          close(fd_out);
//          f_c++;
//          // new file name creation     XXXX.datN   where N=1,2,3......
//          strcpy(outname2,outfile);
//          sprintf(cbuf,"%d\0",f_c);
//          strcat(outname2,cbuf);
//          printf("  sampleget: outfile %d %s\n",f_c,outname2);  //debug
//          /* output file open */
//          fd_out = open(outname2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
//          if (fd_out == -1) {
//             perror("  sampleget: output file open() error");
//             goto end;
//         }
//      }
            
    }

exithere:
       /* sampling stop */

    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("  sampleget: ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() stop sampling succeed\n");
    }

    close(fd_out);

 end:
    close(fd_in);

    return rv;
}
        

/*  f[^W֐
   NۂɃf[^W܂łRbII

  []
    spg->outfile ---- f[^o̓t@C
    spg->span    ---- f[^Wԁ@ibj
    spg->sfreq   ---- TvOg
                    40,100,200,500 (for kHz)
                    1,2,4,8,16 (for MHz)
    spg->adbit   ---- `crbg
                    1,2,4,8
    spg->numch   ---- `l
                    1, 4
    spg->f_size_mode ---- 0: 2GBȏ̃t@CT|[g
                          1: 2GB̃t@C쐬
    spg->lg.logdir --- Ot@CfBNg
    spg->lg.hostname -- zXg
    spg->lg.logfile ---Ot@C
    spg->lg.logfiletype -- Ot@C
                          0: exocode+station_id+"."+hostname+".log"
                          1: YYYY+hostname+".log"
                               where  YYYY is 4 digit year
                          -1: Oo͗}

  [^[]
    -1:  G[
     0:  I
     ntry >0   read errorntry񔭐Bőntrmaxŗ}

    2004-05-23 Oo͋@\ǉBrbOt@Cуt@C[h𓝍
    2005-02-05 more robust against disk full
*/
int sampleget(spget_t *spg)
{
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit;
//#ifdef _VSSP32
//    int headbit=256;  // default header bit number  for K5/VSSP32
//#else
//    int headbit=64;  // header bit number  for K5/VSSP
//#endif
    long int allbyte;     // TvOoCg
    long int bitsec;       // Pb̃rbg@iwb_[rbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[rbg܂ށj
    long int c;
    int sfreq, adbit, numch, span;
    int filter;
    int err;
    int ntry, ntrymax;
    int nbyte;
    int sampling1, freq, width, filter0;
    int ch;
    double dallbyte, dc;
    int f_c;   // file number counter  in case of file exceeds > 2Gbyte
    int oflag,iflag;  //oflag =1 :output file opened
                      //iflag =1 :input file opened
    char cbuf[160];
    char outfile[120];
    char outname2[120]; // for extra output
    // ----------
    unsigned int sfreq_u, sbit_u, numch_u, filter_u;
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
#ifdef _VSSP32
    char buf[64*1024];
    struct tdsauxio tdsaux;
#else
    u_int32_t buf[BUFSIZ * 2];
#endif
    logval_t lg;
     //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ was 1024

    headbit=64;  // header bit number  for K5/VSSP
    oflag=0;
    iflag=0;
    span=spg->span;
    sfreq=spg->sp.sfreq;
    adbit=spg->sp.adbit;
    numch=spg->sp.numch;
    filter=spg->sp.filter;
    strcpy(outfile,spg->outfile);
    lg=spg->lg;

    //ntrymax=5;  // the number to try again when read error occured
    ntrymax=10;  // the number to try again when read error occured
    ntry=0;     // try again counter (if read failure, try again)
    nbyte = sizeof(buf);   // nbyte = 8192  !! check
    
    printf("  sampleget: Ver. 2005-12-28\n");
    /*
    printf(" span = %d\n",span);    // debug
    printf(" sfreq= %d\n",sfreq);   // debug
    printf(" adbit= %d\n",adbit);   // debug
    printf(" numch= %d\n",numch);   // debug
   */

    /* sampling duration check */
    if (span <= 0) {
       printf(" parameter span (duration in sec) should be positive!\n");
       // nothing to do
       return -1;
    }


    /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
       perror("device open() error ");
       return -1;
    }
    iflag=1;

    /* output file open */
    fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
    if (fd_out == -1) {
       perror("output file open() error");
       goto end;
    }
    oflag=1;
    sprintf(cbuf,"sampleget: output file = %s",outfile);
    printf("  %s\n",cbuf);
    LogWrite4(&lg,0,0.0,0,cbuf);

#ifdef _VSSP32  // just for the case of VSSP32 only
    if((rv=ioctl(fd_in, TDSIO_INIT)) == -1){
        perror("  sampleget: ioctl() INIT failed ");
        goto end;
    } else {
        //printf("  sampleget: ioctl() INIT succeed\n");
        printf("  sampleget: ioctl() INIT ");
    }
    /* set AUX field data */
    if(spg->sp.vsspaux.size > 0) {    // default size should be 20
        tdsaux.auxsize=spg->sp.vsspaux.size;
        for(i=0; i<tdsaux.auxsize; i++) tdsaux.auxfield[i]=spg->sp.vsspaux.bdat[i];
        if((rv=ioctl(fd_in, TDSIO_SET_AUX, &tdsaux)) == -1){
            perror("  sampleget: ioctl() SET_AUX failed ");
            goto end;
        } else {
            //printf("  sampleget: ioctl() SET_AUX succeed\n");
            printf("SET_AUX ");
        }
    }
    /* get AUX field data */
    if((rv=ioctl(fd_in, TDSIO_GET_AUX, &tdsaux)) == -1){
       perror("  sampleget: ioctl() GET_AUX failed ");
            goto end;
    } else {
            printf("GET_AUX ");
    }
    printf("succeed\n");
    headbit=64+32+8*tdsaux.auxsize;   // header bit calc
    if(headbit != 256){
        printf("  sampleget:Warning!! header bits (%d) is different from default value (256)\n",headbit);
    } else {
        printf("  sampleget: header bits = %d\n",headbit);
    }
#endif

    /* sampling parameters set */
    //if((rv=set_vssp_sparam(fd_in,sfreq,adbit,numch,filter,0))==-1){
    if((rv=set_vssp_sparam(fd_in, &spg->sp,0))==-1){
        printf("  sampleget: sampling parameter set failed \n");
        goto end;
    }

    /* check file size etc */
    //if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}   
    //else { bitsec=sfreq*1000*1000 + headbit ;}
    if (sfreq == 40 || sfreq == 100 || sfreq == 200 || sfreq == 500 ) {     //2005-12-21
        bitsec=sfreq*1000 ;
    } else { 
        bitsec=sfreq*1000*1000;
    }

    bitsec=(bitsec*adbit*numch)+headbit;   // bit number per second (including header bits)

    bytesec=bitsec/8;
    allbyte=bytesec*span;
    dallbyte=(double)bytesec*(double)span; 

    /* File maximum bytes check */
       //if (allbyte > FILEMAXSIZE){
//      if (dallbyte > (double)FILEMAXSIZE){  // 2002-10-21
//         allbyte= FILEMAXSIZE;
//         secforce=allbyte/bytesec;
//         printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
//              allbyte,FILEMAXSIZE);  // debug
//         printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
//      }
    sprintf(cbuf,"sampleget: bits/sec=%d  bytes/sec=%d  all_bytes=%g",bitsec, bytesec,dallbyte);  // debug
    printf("  %s\n",cbuf);

    /* get sampling parameters from a board */
    //if ((rv=get_vssp_sparam(fd_in,&freq,&width,&ch,&filter0,0))== -1) {
    if ((rv=get_vssp_sparam(fd_in, &spg->sp,0))== -1) {
        perror("  sampleget: ioctl() get sampling parameter failed ");
        goto end;
    }  

#ifdef _VSSP32
    sprintf(cbuf,"sampleget: sampling parameters are %d kHz (filter %d MHz) %d BIT %d CH",
                         spg->sp.freqkhz,spg->sp.filter,spg->sp.adbit,spg->sp.numch);
#else
    sprintf(cbuf,"sampleget: sampling parameters are %d kHz %d BIT %d CH",
                         spg->sp.freqkhz,spg->sp.adbit,spg->sp.numch);
#endif
    printf("  %s\n",cbuf);
    LogWrite4(&lg,0,0.0,0,cbuf);

       /* board time get */
 /*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
    //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
    printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("  sampleget: ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() FIFO clear succeed\n");
    }

    /* sampling start */

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("  sampleget: ioctl() start sampling failed ");
        printf("  sampleget:  Probably time was not set for the IP-VLBI board.\n");
        printf("  sampleget:  Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() start sampling succeed\n");
    }
    c = 0;
    dc= 0.0;   // 2002-10-21
    f_c=0;    // 2002-10-21
    //ntry=0;   // try again counter (if read failure, try again)
    //while (c <=  allbyte) {
    while (1) {                   // 2002-10-21
try_again:
       rv = read(fd_in, buf, nbyte);
#ifdef _VSSP32
       if (rv == -1) {
          printf("  sampleget: read() failed ");
          ioctl(fd_in, TDSIO_GET_ERROR, &err); printf("Error code = %d\n",err);  // debug for VSSP32
          sprintf(cbuf,"sampleget: read() failed! Error code = %d",err);  // 2006-01-24
          //LogWrite4(&lg,0,0.0,1,"sampleget: read() failed");
          LogWrite4(&lg,0,0.0,1,cbuf);
          //goto end;
//          if((rv = ioctl(fd_in, TDSIO_SAMPLING_STOP))==-1) goto end;
//          printf("STOP ");
//          //if((rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR))==-1) goto end;
//          //printf("FIFO_CLEARED ");
//          if((rv = ioctl(fd_in, TDSIO_SAMPLING_START))==-1) goto end;
//          printf("START ");
          ntry++;                                  // 2006-01-24
          if(ntry > ntrymax) {
              printf("  sampleget: !! retry number (%d) exceeds maximum (%d)\n",ntry,ntrymax);
              sprintf(cbuf,"sampleget: !! retry number (%d) exceeds maximum (%d)\0",ntry,ntrymax);
              LogWrite4(&lg,0,0.0,1,cbuf);
              printf("  sampleget: !! so give up retry\n");
              sprintf(cbuf,"sampleget: !! so give up retry\0");
              LogWrite4(&lg,0,0.0,1,cbuf);
              goto end;
          }

          goto try_again;
       }
#else
       if (rv == -1) {
          perror("  sampleget: read() failed ");
          ioctl(fd_in, TDSIO_GET_ERROR, &rv); printf("Error code = %d\n",rv);  // debug for VSSP32
          
          LogWrite4(&lg,0,0.0,1,"sampleget: read() failed");
          //try sampling start again 
          ntry++;
          if(ntry > ntrymax) {
              printf("  sampleget: !! retry number (%d) exceeds maximum (%d)\n",ntry,ntrymax);
              sprintf(cbuf,"sampleget: !! retry number (%d) exceeds maximum (%d)\0",ntry,ntrymax);
              LogWrite4(&lg,0,0.0,1,cbuf);
              printf("  sampleget: !! so give up retry\n");
              sprintf(cbuf,"sampleget: !! so give up retry\0");
              LogWrite4(&lg,0,0.0,1,cbuf);
              goto end;
          }
          rv = ioctl(fd_in, TDSIO_SAMPLING_START);
          if (rv == -1) {
              perror("  sampleget: ioctl() re-start sampling failed ");
              sprintf(cbuf,"sampleget: re-start sampling failed at retry %d/%d\0",ntry,ntrymax);
              LogWrite4(&lg,0,0.0,1,cbuf);
              goto end;
          } else {
              printf("  sampleget: ioctl() re-start sampling succeed at retry %d/%d\n",ntry,ntrymax);;
              sprintf(cbuf,"sampleget: re-start sampling at retry %d/%d\0",ntry,ntrymax);
              LogWrite4(&lg,0,0.0,1,cbuf);
          } 
          goto try_again;
       }
#endif
       //printf(" nbyte = %d  rv = %d\n",nbyte,rv);   // debug
       rv = write(fd_out, buf, rv);  // should check error (file full etc)
       /* add on 2005-01-30 */
       if (rv == -1) {
          perror("  sampleget: write() failed ");
          sprintf(cbuf,"sampleget: write() failed : %s\0",strerror(errno));
          LogWrite4(&lg,0,0.0,1,cbuf);
          //try file close and move to another disk 
          //  this is supposing write error occurred due to disk full 
          c=0;
          close(fd_out);
          oflag=0;
          f_c++;
          //new directory name set in out file name
          if(spg->outdirtab.ndir < spg->outdirtab.dirnum){ //2005-01-30
              strcpy(spg->outdir,spg->outdirtab.dir[spg->outdirtab.ndir]); 
              spg->outdirtab.ndir++;   
              sprintf(cbuf,"OUT directory changed  :  %s",spg->outdir);
              LogWrite4(&lg,0,0.0,0,cbuf);
          } else {
              goto end;
          }
          strcpy(outfile,spg->outdir);
          strcat(outfile,spg->fname);

          // new file name creation     XXXX.datN   where N=1,2,3......

          strcpy(outname2,outfile);
          sprintf(cbuf,"%d\0",f_c);
          strcat(outname2,cbuf);
          printf("  sampleget: outfile %d %s\n",f_c,outname2);  //debug
          /* output file open */
          fd_out = open(outname2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
          if (fd_out == -1) {
             perror("  sampleget: output file open() error");
             goto end;
          }
          oflag=1;
          sprintf(cbuf,"sampleget: output file = %s",outname2);
          printf("  %s\n",cbuf);
          LogWrite4(&lg,0,0.0,0,cbuf);
       }

       /****** add end *******/
       c+= nbyte;

       /* add on 2002-10-21 */
        dc+=(double)nbyte;
        if( dc > dallbyte) {   // sampling end
            goto exithere;
        }
        if(spg->f_size_mode==1 && c >  FILEMAXSIZE) { // newfile creation
            c=0;
            close(fd_out);
            oflag=0;
            f_c++;
            // new file name creation     XXXX.datN   where N=1,2,3......
            strcpy(outname2,outfile);
            sprintf(cbuf,"%d\0",f_c);
            strcat(outname2,cbuf);
            printf("  sampleget: outfile %d %s\n",f_c,outname2);  //debug
            /* output file open */
            fd_out = open(outname2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out == -1) {
               perror("  sampleget: output file open() error");
               goto end;
            }
            oflag=1;
            sprintf(cbuf,"sampleget: output file = %s",outname2);
            printf("  %s\n",cbuf);
            LogWrite4(&lg,0,0.0,0,cbuf);
        }
            
    }

exithere:
       /* sampling stop */

    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("  sampleget: ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() stop sampling succeed\n");
    }

end:
    if(oflag == 1)  close(fd_out);
    if(iflag == 1)  close(fd_in);

    return ntry;
}
        
        

/*  f[^W֐
   NۂɃf[^W܂łRbII
    outfile ---- f[^o̓t@C
    span    ---- f[^Wԁ@ibj
    sfreq   ---- TvOg
                    40,100,200,500 (for kHz)
                    1,2,4,8,16 (for MHz)
    adbit   ---- `crbg
                    1,2,4,8
    numch   ---- `l
                    1, 4
    2002-10-21 2Gbytesȏ̊ϑ悤ɕύX
*/
int sampleget_old(char* outfile, int span, int sfreq, int adbit, int numch)
{
    /* former sampleget till 2004-05-23 */

    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit=64;  // header bit number  for a new sampler board
    //int headbit=32;  // header bit number  for a old sampler board
    long int allbyte;     // TvOoCg
    long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    long int c;
    unsigned int sampling1, freq, width;
    int ch;
    // 2002-10-21
    double dallbyte, dc;
    int f_c;   // file number counter  in case of file exceeds > 2Gbyte
    char cbuf[10];
    char outname2[80]; // for extra output
    // ----------
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    u_int32_t buf[BUFSIZ * 2];
        //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI

   /*
    printf(" span = %d\n",span);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
   */

       /* sampling duration check */
       if (span <= 0) {
          printf(" parameter span (duration in sec) should be positive!\n");
          // nothing to do
          return -1;
       }


       /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }

       /* output file open */
    fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
    if (fd_out == -1) {
        perror("output file open() error");
        goto end;
    }

       /* sampling frequency set */

       if      (sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
       else if (sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
       else if (sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
       else if (sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
       else if (sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
       else if (sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
       else if (sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
       else if (sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
       else if (sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
       else { 
       printf(" parameter sfreq  (sampling frequency) must be either one of followings:\n");
       printf("              40,100,200,500 (for kHz)\n");
       printf("              1,2,4,8,16 (for MHz)\n");
          return -1;
       }
      
       /* AD resolution set */
       if      (adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
       else if (adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
       else if (adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
       else if (adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
       else { 
       printf(" parameter adbit (A/D resolution) must be either one of followings:\n");
       printf("              1,2,4,8\n");
          return -1;
       }

       /* number of channels */
       if      (numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
       else if (numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
       else { 
       printf(" parameter numch (number of channels) must be either one of followings:\n");
       printf("              1,4\n");
          return -1;
       }

       /* sampling parameters set */
       sampling =  sfreq_t | sbit_t | numch_t;

       //if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}   
       //else { bitsec=sfreq*1000*1000 + headbit ;}
       if (sfreq >= 40) { bitsec=sfreq*1000 ;}          //2002-9-15
       else { bitsec=sfreq*1000*1000 ;}                 //2002-9-15

       //bitsec=bitsec*adbit*numch;   // bit number per second
        bitsec=(bitsec*adbit*numch)+headbit;   // bit number per second  !! 2002-9-15

       bytesec=bitsec/8;
       allbyte=bytesec*span;
       dallbyte=(double)bytesec*(double)span;   // 2002-10-21

       /* File maximum bytes check */
       //if (allbyte > FILEMAXSIZE){
 //      if (dallbyte > (double)FILEMAXSIZE){  // 2002-10-21
 //         allbyte= FILEMAXSIZE;
 //         secforce=allbyte/bytesec;
 //         printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
//              allbyte,FILEMAXSIZE);  // debug
//          printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
//        }
       printf("  sampleget: bits/sec=%d  bytes/sec=%d  all_bytes=%g\n",bitsec, bytesec,dallbyte);  // debug

    //sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;

    rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
    if (rv == -1) {
       perror("  sampleget: ioctl() sampling set failed ");
       goto end;
    }
    else {
       printf("  sampleget: ioctl() sampling set succeed\n");
    }


       /* get sampling parameters from a board */

    rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
    if (rv == -1) {
        perror("  sampleget: ioctl() get sampling parameter failed ");
        goto end;
    }
    else {
        printf("  sampleget: ioctl() get sampling parameter succeed\n");
        freq = TDS_GET_SAMP_FREQ(sampling1);
        width = TDS_GET_SAMP_WIDTH(sampling1);
        ch = TDS_GET_SAMP_CH(sampling1);
        if(freq == TDS_SAMPLING_40KHZ)
            freq = 40;
        if(freq == TDS_SAMPLING_100KHZ)
            freq = 100;
        if(freq == TDS_SAMPLING_200KHZ)
            freq = 200;
        if(freq == TDS_SAMPLING_500KHZ)
            freq = 500;
        if(freq == TDS_SAMPLING_1MHZ)
            freq = 1000;
        if(freq == TDS_SAMPLING_2MHZ)
            freq = 2000;
        if(freq == TDS_SAMPLING_4MHZ)
            freq = 4000;
        if(freq == TDS_SAMPLING_8MHZ)
            freq = 8000;
        if(freq == TDS_SAMPLING_16MHZ)
            freq = 16000;
        if(width == TDS_SAMPLING_1BIT)
            width = 1;
        if(width == TDS_SAMPLING_2BIT)
            width = 2;
        if(width == TDS_SAMPLING_4BIT)
            width = 4;
        if(width == TDS_SAMPLING_8BIT)
            width = 8;
        if(ch == TDS_SAMPLING_4CH)
            ch = 4;
        else
            ch =1;
        printf("  sampleget: sampling parameters are %d kHz %d BIT %d CH\n",freq,width,ch);
    }   



       /* board time get */
 /*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
    //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
    printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("  sampleget_old: ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("  sampleget_old: ioctl() FIFO clear succeed\n");
    }
/*       
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);           // Poorւ̓BV{[h(2002.3ȍ~)ł
    if (rv == -1) {                               // ȂĂ̍ŏɂPΗǂ
        perror("ioctl() 1pps sync failed ");   //  (Ȃǂj
        goto end;
    }
    else {
        printf("ioctl() 1pps sync succeed\n");
    }
*/
/*  rv = ioctl(fd_in, TDSIO_SYNC_TIME);            // O͎͂gpȂ
    if (rv == -1) {
        perror("ioctl() time sync failed ");
        goto end;
    }
    else {
        printf("ioctl() time sync succeed\n");
    }
*/

       /* sampling start */

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("  sampleget: ioctl() start sampling failed ");
              printf("  sampleget_old:  Probably time was not set for the IP-VLBI board.\n");
              printf("  sampleget_old:  Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("  sampleget_old: ioctl() start sampling succeed\n");
    }
        printf("  sampleget_old: Ver. 2002-10-21\n");
    c = 0;
    dc=0.0;   // 2002-10-21
    f_c=0;    // 2002-10-21
    //while (c <=  allbyte) {
    while (1) {                   // 2002-10-21
       int nbyte = sizeof(buf);   // nbyte = 8192

       rv = read(fd_in, buf, nbyte);
       if (rv == -1) {
          perror("  sampleget_old: read() failed ");
          // if necessary, put try again code here
          goto end;
       }
          //printf(" nbyte = %d\n",nbyte);   // debug
       rv = write(fd_out, buf, rv);
       c= c+ nbyte;
    /* add on 2002-10-21 */
        dc+=(double)nbyte;
        if( dc > dallbyte) {   // sampling end
            goto exithere;
        }
        if(c >  FILEMAXSIZE) { // newfile creation
            c=0;
            close(fd_out);
            f_c++;
            // new file name creation     XXXX.datN   where N=1,2,3......
            strcpy(outname2,outfile);
            sprintf(cbuf,"%d\0",f_c);
            strcat(outname2,cbuf);
            printf("  sampleget_old: outfile %d %s\n",f_c,outname2);  //debug
            /* output file open */
            fd_out = open(outname2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out == -1) {
               perror("  sampleget_old: output file open() error");
               goto end;
           }
        }
            
    }

exithere:
       /* sampling stop */

    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("  sampleget_old: ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("  sampleget_old: ioctl() stop sampling succeed\n");
    }

    close(fd_out);

 end:
    close(fd_in);

    return rv;
}
        

/*  f[^W֐ sampleget̃IWiŁiQfbytes/fileȏ͂߁Ij
   NۂɃf[^W܂łRbII
    outfile ---- f[^o̓t@C
    span    ---- f[^Wԁ@ibj
    sfreq   ---- TvOg
                    40,100,200,500 (for kHz)
                    1,2,4,8,16 (for MHz)
    adbit   ---- `crbg
                    1,2,4,8
    numch   ---- `l
                    1, 4
*/
int sampleget0(char* outfile, int span, int sfreq, int adbit, int numch)
{
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit=64;  // header bit number  for a new sampler board
    //int headbit=32;  // header bit number  for a old sampler board
    long int allbyte;     // TvOoCg
    long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    long int c;
    unsigned int sampling1, freq, width;
    int ch;
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    u_int32_t buf[BUFSIZ * 2];
        //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI

   /*
    printf(" span = %d\n",span);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
   */

       /* sampling duration check */
       if (span <= 0) {
          printf(" parameter span (duration in sec) should be positive!\n");
          // nothing to do
          return -1;
       }


       /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }

       /* output file open */
    fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
    if (fd_out == -1) {
        perror("output file open() error");
        goto end;
    }

       /* sampling frequency set */

       if      (sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
       else if (sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
       else if (sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
       else if (sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
       else if (sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
       else if (sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
       else if (sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
       else if (sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
       else if (sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
       else { 
       printf(" parameter sfreq  (sampling frequency) must be either one of followings:\n");
       printf("              40,100,200,500 (for kHz)\n");
       printf("              1,2,4,8,16 (for MHz)\n");
          return -1;
       }
      
       /* AD resolution set */
       if      (adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
       else if (adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
       else if (adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
       else if (adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
       else { 
       printf(" parameter adbit (A/D resolution) must be either one of followings:\n");
       printf("              1,2,4,8\n");
          return -1;
       }

       /* number of channels */
       if      (numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
       else if (numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
       else { 
       printf(" parameter numch (number of channels) must be either one of followings:\n");
       printf("              1,4\n");
          return -1;
       }

       /* sampling parameters set */
       sampling =  sfreq_t | sbit_t | numch_t;

       //if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}   
       //else { bitsec=sfreq*1000*1000 + headbit ;}
       if (sfreq >= 40) { bitsec=sfreq*1000 ;}          //2002-9-15
       else { bitsec=sfreq*1000*1000 ;}                 //2002-9-15

       //bitsec=bitsec*adbit*numch;   // bit number per second
        bitsec=(bitsec*adbit*numch)+headbit;   // bit number per second  !! 2002-9-15

       bytesec=bitsec/8;
       allbyte=bytesec*span;

       /* File maximum bytes check */
       if (allbyte > FILEMAXSIZE){
          allbyte= FILEMAXSIZE;
          secforce=allbyte/bytesec;
          printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
              allbyte,FILEMAXSIZE);  // debug
          printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
        }
       printf(" bitsec  = %d\n",bitsec);  // debug
       printf(" bytesec = %d\n",bytesec);  // debug
       printf(" allbyte = %d\n",allbyte);  // debug

    //sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;

    rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
    if (rv == -1) {
       perror("ioctl() sampling set failed ");
       goto end;
    }
    else {
       printf("ioctl() sampling set succeed\n");
    }


       /* get sampling parameters from a board */

    rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
    if (rv == -1) {
        perror("ioctl() get sampling parameter failed ");
        goto end;
    }
    else {
        printf("ioctl() get sampling parameter succeed\n");
        freq = TDS_GET_SAMP_FREQ(sampling1);
        width = TDS_GET_SAMP_WIDTH(sampling1);
        ch = TDS_GET_SAMP_CH(sampling1);
        if(freq == TDS_SAMPLING_40KHZ)
            freq = 40;
        if(freq == TDS_SAMPLING_100KHZ)
            freq = 100;
        if(freq == TDS_SAMPLING_200KHZ)
            freq = 200;
        if(freq == TDS_SAMPLING_500KHZ)
            freq = 500;
        if(freq == TDS_SAMPLING_1MHZ)
            freq = 1000;
        if(freq == TDS_SAMPLING_2MHZ)
            freq = 2000;
        if(freq == TDS_SAMPLING_4MHZ)
            freq = 4000;
        if(freq == TDS_SAMPLING_8MHZ)
            freq = 8000;
        if(freq == TDS_SAMPLING_16MHZ)
            freq = 16000;
        if(width == TDS_SAMPLING_1BIT)
            width = 1;
        if(width == TDS_SAMPLING_2BIT)
            width = 2;
        if(width == TDS_SAMPLING_4BIT)
            width = 4;
        if(width == TDS_SAMPLING_8BIT)
            width = 8;
        if(ch == TDS_SAMPLING_4CH)
            ch = 4;
        else
            ch =1;
        printf("sampling setting is %d kHz %d BIT %d CH\n",freq,width,ch);
    }   



       /* board time get */
 /*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
    //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
    printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("ioctl() FIFO clear succeed\n");
    }
/*       
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);           // Poorւ̓BV{[h(2002.3ȍ~)ł
    if (rv == -1) {                               // ȂĂ̍ŏɂPΗǂ
        perror("ioctl() 1pps sync failed ");   //  (Ȃǂj
        goto end;
    }
    else {
        printf("ioctl() 1pps sync succeed\n");
    }
*/
/*  rv = ioctl(fd_in, TDSIO_SYNC_TIME);            // O͎͂gpȂ
    if (rv == -1) {
        perror("ioctl() time sync failed ");
        goto end;
    }
    else {
        printf("ioctl() time sync succeed\n");
    }
*/

       /* sampling start */

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("ioctl() start sampling failed ");
              printf("Probably time was not set for the IP-VLBI board.\n");
              printf("Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("ioctl() start sampling succeed\n");
    }

    c = 0;
    //while (c < BUFSIZ * BUFSIZ) {
    while (c <=  allbyte) {
       int nbyte = sizeof(buf);   // nbyte = 8192

       rv = read(fd_in, buf, nbyte);
       if (rv == -1) {
          perror("read() failed ");
          goto end;
       }
          //printf(" nbyte = %d\n",nbyte);   // debug
       rv = write(fd_out, buf, rv);
       c= c+ nbyte;
    }

       /* sampling stop */

    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("ioctl() stop sampling succeed\n");
    }

    close(fd_out);

 end:
    close(fd_in);

    return rv;
}
        
        
        
int sampleget2(char* outfile, int span, int sfreq, int adbit, int numch, char* dirtemp, int mode)
{
/*
                mode    -- t@Co̓[h
                       1: ꊇt@Ĉ݁@iftHgj
                       2: Pbf[^t@Ĉ
                       3:  ꊇt@C{Pbƃt@C
                       4:  Pbf[^t@CóAϑꊇt@C쐬
                       5:  3+4  for debug purpose ꊇt@C debug.dat 쐬
                       6: Pbf[^t@Ĉ (Ԑj
                       7: UƓit@Co͖jVXe`FbNp
*/
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out, fd_out_tmp;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv, rv2, rv3, rv2s;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit=64;  // header bit number  for a new sampler board
    //int headbit=32;  // header bit number  for a old sampler board
    long int allbyte;     // TvOoCg
    long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    long int c, c2;
    unsigned int sampling1, freq, width;
    int ch;
    int sec_counter;    // second counter for tempolary file creation
    int sec_counter_max;
    int fmode;         // file out mode
    int endflg, nbyte, nfile;
    char* namnow="sampler.now"; //1sec file name sampling out now 
    char* namlast="sampler.dat"; //1sec file name sampling out just finished 
    char tmpnow[80],tmplast[80];
    char tmpfile1[80],tmpfile2[80], outfile2[80];
    char fname1[20],fname2[20];
    char filetable[1000][80];  // 1sec file name container  
    int nfilemax;
    int fvalid;   // 1 sec file number exceeds 3 then set 1 
    int fpos;
    int ierr;
    unsigned char bit64[8];  // for header check
    int hspatn, hiok, hsmplbit, hsfreq, hnumch, hhh,hmm,hss;
    long int hseconds;
    char* fdebug="debug.dat";
    FILE *fin;
    
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    u_int32_t buf[BUFSIZ * 2];
    
    nfilemax=1000;
    sec_counter_max=1000;  // sec_counter reset if exceeds this value
    fvalid=0;

    //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI

/*
    printf(" span = %d\n",span);    // debug
    printf(" sfreq= %d\n",sfreq);   // debug
    printf(" adbit= %d\n",adbit);   // debug
    printf(" numch= %d\n",numch);   // debug
*/
    /* File out mode check */
    if(mode <=0 || mode > 4) {
        fmode=1;
    } else {
        fmode=mode;
    }
    if(mode == 5) {
        fmode=3;
    }
    if(mode == 6 || mode ==7 ) {
        fmode=2;
        nfilemax=10;
        sec_counter_max=10;
    }

    /* sampling duration check */
    if (span <= 0) {
        printf(" parameter span (duration in sec) should be positive!\n");
        // nothing to do
        return -1;
    }


    /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }

    /* output file open */
    if(fmode == 1 || fmode ==3){
        fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
        if (fd_out == -1) {
            perror("output file open() error");
            goto end;
        }
    }
    sec_counter=0;
    /* tempolary file name creation */
    strcpy(tmpnow,dirtemp);
    strcpy(tmplast,dirtemp);
    strcat(tmpnow,namnow);
    strcat(tmplast,namlast);
                        
    //sprintf(fname1,"smp%04d.now\0",sec_counter);  
    sprintf(fname2,"smp%04d.dat\0",sec_counter);  
    //strcat(tmpfile1,fname1);
    strcpy(tmpfile2,dirtemp);
    strcat(tmpfile2,fname2);
    
    /* tempolary output file open */
    if(mode !=7 ){
        
        if(fmode == 2 || fmode ==3 || fmode == 4){
            fd_out_tmp = open(tmpnow, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out_tmp == -1) {
                perror("temprary output file open() error");
                goto end;
            }
        }
    }
       /* sampling frequency set */

       if      (sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
       else if (sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
       else if (sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
       else if (sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
       else if (sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
       else if (sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
       else if (sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
       else if (sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
       else if (sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
       else { 
       printf(" parameter sfreq  (sampling frequency) must be either one of followings:\n");
       printf("              40,100,200,500 (for kHz)\n");
       printf("              1,2,4,8,16 (for MHz)\n");
          return -1;
       }
      
       /* AD resolution set */
       if      (adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
       else if (adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
       else if (adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
       else if (adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
       else { 
       printf(" parameter adbit (A/D resolution) must be either one of followings:\n");
       printf("              1,2,4,8\n");
          return -1;
       }

       /* number of channels */
       if      (numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
       else if (numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
       else { 
       printf(" parameter numch (number of channels) must be either one of followings:\n");
       printf("              1,4\n");
          return -1;
       }

       /* sampling parameters set */
       sampling =  sfreq_t | sbit_t | numch_t;

       //if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}   
       //else { bitsec=sfreq*1000*1000 + headbit ;}
       if (sfreq >= 40) { bitsec=sfreq*1000 ;}          //2002-9-15
       else { bitsec=sfreq*1000*1000 ;}                 //2002-9-15

     /* sampling byte number calc */
        //bitsec=bitsec*adbit*numch;   // bit number per second
        bitsec=(bitsec*adbit*numch)+headbit;   // bit number per second  !! 2002-9-15

       bytesec=bitsec/8;
       allbyte=bytesec*span;

       /* File maximum bytes check */
       if(mode !=6 && mode !=7 ){
            if (allbyte > FILEMAXSIZE || allbyte < 0){
                allbyte= FILEMAXSIZE;
                secforce=allbyte/bytesec;
         
                allbyte=secforce*bytesec;
                printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
                allbyte,FILEMAXSIZE);  // debug
                printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
            }
       }
       printf(" bitsec  = %d\n",bitsec);  // debug
       printf(" bytesec = %d\n",bytesec);  // debug
       printf(" allbyte = %d\n",allbyte);  // debug

    //sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;

    rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
    if (rv == -1) {
       perror("ioctl() sampling set failed ");
       goto end;
    }
    else {
       printf("ioctl() sampling set succeed\n");
    }


       /* get sampling parameters from a board */

    rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
    if (rv == -1) {
        perror("ioctl() get sampling parameter failed ");
        goto end;
    }
    else {
        printf("ioctl() get sampling parameter succeed\n");
        freq = TDS_GET_SAMP_FREQ(sampling1);
        width = TDS_GET_SAMP_WIDTH(sampling1);
        ch = TDS_GET_SAMP_CH(sampling1);
        if(freq == TDS_SAMPLING_40KHZ)
            freq = 40;
        if(freq == TDS_SAMPLING_100KHZ)
            freq = 100;
        if(freq == TDS_SAMPLING_200KHZ)
            freq = 200;
        if(freq == TDS_SAMPLING_500KHZ)
            freq = 500;
        if(freq == TDS_SAMPLING_1MHZ)
            freq = 1000;
        if(freq == TDS_SAMPLING_2MHZ)
            freq = 2000;
        if(freq == TDS_SAMPLING_4MHZ)
            freq = 4000;
        if(freq == TDS_SAMPLING_8MHZ)
            freq = 8000;
        if(freq == TDS_SAMPLING_16MHZ)
            freq = 16000;
        if(width == TDS_SAMPLING_1BIT)
            width = 1;
        if(width == TDS_SAMPLING_2BIT)
            width = 2;
        if(width == TDS_SAMPLING_4BIT)
            width = 4;
        if(width == TDS_SAMPLING_8BIT)
            width = 8;
        if(ch == TDS_SAMPLING_4CH)
            ch = 4;
        else
            ch =1;
        printf("sampling setting is %d kHz %d BIT %d CH\n",freq,width,ch);
    }   



       /* board time get */
 /*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
    //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
    printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("ioctl() FIFO clear failed ");
        goto end;
    }
    else {
        printf("ioctl() FIFO clear succeed\n");
    }
/*       
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);           // Poorւ̓BV{[h(2002.3ȍ~)ł
    if (rv == -1) {                               // ȂĂ̍ŏɂPΗǂ
        perror("ioctl() 1pps sync failed ");   //  (Ȃǂj
        goto end;
    }
    else {
        printf("ioctl() 1pps sync succeed\n");
    }
*/
/*  rv = ioctl(fd_in, TDSIO_SYNC_TIME);            // O͎͂gpȂ
    if (rv == -1) {
        perror("ioctl() time sync failed ");
        goto end;
    }
    else {
        printf("ioctl() time sync succeed\n");
    }
*/

       /* sampling start */

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("ioctl() start sampling failed ");
              printf("Probably time was not set for the IP-VLBI board.\n");
              printf("Please set time using timesettk and try again.\n");
        goto end;
    }
    else {
        printf("ioctl() start sampling succeed\n");
    }

    c = 0;
    c2= 0;
    endflg=0;
    nfile=0;
    nbyte = sizeof(buf);   // nbyte = 8192
    //while (c < BUFSIZ * BUFSIZ) {
    while (1) {

        rv = read(fd_in, buf, nbyte);
        if (rv == -1) {
            perror("read() failed ");
            goto end;
        }
        if(rv != nbyte) {printf("read byte # difference %d\n",rv);}
        //printf(" nbyte = %d\n",nbyte);   // debug
        /* monitor 1st 64bit data */
        if(c == 0) {
            printf("%x %x \n",*(buf), *(buf+1));
        }
        
        c += rv;      // already read byte number
        //printf(" nbyte,rv,c  %d %d %d\n",nbyte,rv,c);   // debug
        if(mode !=6 && mode !=7 ){
            if( c >= allbyte ){
                rv3=c-allbyte;
                rv2=rv-rv3;
                endflg=1;  // end flag set
                if(fmode == 1 || fmode ==3){
                    rv2 = write(fd_out, buf, rv2);
                    close(fd_out);
                }
            } else {
                if(fmode == 1 || fmode ==3){
                    rv = write(fd_out, buf, rv);
                }
            }
        }
            
        if(fmode == 2 || fmode ==3 || fmode == 4){
            c2+=rv;
            if(c2 >= bytesec) {
                rv3=c2-bytesec;
            
                rv2=rv-rv3;
                //printf("1st c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  // debug
                if(mode !=7 ){
                    rv2=write(fd_out_tmp,buf,rv2);
                    /* tempolary file update */
                    close(fd_out_tmp);
                    if(fvalid == 1){
                        fpos=nfile-1;
                        if(fpos < 0){ fpos=fpos+nfilemax;}
                    
                        //printf("%d %s\n",fpos,filetable[fpos]);
                        /* folowing is not necessary for FreeBSD */
                    /*
                        if((ierr=remove(filetable[fpos])) <0){
                        if(ierr != -1 ) perror("File remove error");
                        }
                    */
                        if (rename(tmplast,filetable[fpos]) <0) {
                        perror("Rename tmplast error");
                        }
                    }else{
                        if(nfile >=0) fvalid=1;
                    }
                
                    /* folowing (remove) is not necessary for FreeBSD */
                    /*
                    if((ierr=remove(tmplast)) <0){
                    if(ierr != -1 ) perror("File remove error");
                    }
                    */
                
                    if (rename(tmpnow,tmplast) <0){
                        perror("File tmpnow rename error");
                    }
                
                }
                strcpy(filetable[nfile],tmpfile2); // save file name info
                
                //printf("nfile %d   %s %s\n",nfile,filetable[nfile],tmpfile2);
                nfile++;
                if(nfile >= nfilemax) nfile=0;
                /* new templary file nmae creation */
                sec_counter++;
                if(sec_counter >= sec_counter_max) sec_counter=0;
                //sprintf(fname1,"smp%04d.now\0",sec_counter);  
                sprintf(fname2,"smp%04d.dat\0",sec_counter);  
                //strcpy(tmpfile1,dirtemp);
                //strcat(tmpfile1,fname1);
                strcpy(tmpfile2,dirtemp);
                strcat(tmpfile2,fname2);
                
                if(endflg !=1 ){
                    if(mode !=7 ){
                        /* tempolary output file open */
                        //fd_out_tmp = open(tmpfile1, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
                        fd_out_tmp = open(tmpnow, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
                        if (fd_out_tmp == -1) {
                            perror("temprary output file open() error");
                            goto end;
                        }
                    }
                    /* rest of data write */
                    if(rv3 > 0) {
                        rv2s=rv2/sizeof(u_int32_t);
                        if(mode !=7 ){
                            rv3=write(fd_out_tmp,buf+rv2s,rv3);
                        }
                        /* monitor 1st 64bit data */
                        printf("%x %x \n",*(buf+rv2s), *(buf+rv2s+1));
                        
                        /* synck pattern check (this is too heavy for sampling gt 1MHz (no more use) */
                        /*
                        strncpy(bit64,(char *)(buf+rv2s),8);
                        headerchkn(bit64,&hiok,&hspatn,&hsmplbit,&hsfreq,&hnumch,
                            &hseconds,&hhh,&hmm,&hss);
                        if (hiok == 0) {  // sync detection
                            printf("Sync OK!!\n");
                            printf("A/D(bits) %d  CHs %d  SFreq(kHz) %d",hsmplbit,hnumch, hsfreq);
                            printf("  Time %02d:%02d:%02d  sec %d\n",hhh,hmm,hss,hseconds);
                        } else {
                            printf("First 64bit is not header block!\n");
                        }
                        */
                    }
                        //printf("after  c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  // debug
                    c2=rv3;
                    //printf("final  c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  //debug
                }
            } else {
                if(mode !=7 ){
                    rv2=write(fd_out_tmp,buf,rv);
                }
            }
        }
            
        if(endflg == 1) goto loopexit;
            
    }

loopexit:

       /* sampling stop */

    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("ioctl() stop sampling succeed\n");
    }
    
 end:
    close(fd_in);
    
    if(mode !=7 ){
        
    if(fmode == 2 || fmode ==3 || fmode == 4){
        if(fvalid == 1){
            fpos=nfile-1;
            if(fpos < 0) fpos=fpos+nfilemax;
            /*
            if(remove(filetable[fpos]) <0)
            perror("File remove error");
            */
            /* just rename */
        //  if (rename(tmplast,filetable[fpos]) <0) 
        //      perror("Rename tmplast error");
            
            /* Copy file to leave last file name */
            fd_out = open(filetable[fpos], O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out == -1) {
                perror("output file open() error");
                goto end2;
            }
            if((fin = fopen(tmplast, "rb"))==NULL){
                perror("Data file open error!\n");
                goto end2;
            }
            while(1){
                rv = fread(buf,1,nbyte,fin);
                if (rv == -1) {
                    perror("read() failed ");
                    goto end2;
                }
                rv2=write(fd_out,buf,rv);
                if (feof(fin) !=0)  // Reach EOF
                {
                    fclose(fin);
                    goto exit2;
                }
            }
        exit2:;
            
            close(fd_out);      
        }
    }
        
    }
    //close(fd_out);
    //close(fd_out_tmp);
    //if (rename(tmpfile1,tmpfile2) <0) 
    //  perror("Rename error");
    
    /* 1sec data file merges into one file big file */
    
    if(fmode == 4 || mode ==5) {  
        printf("File merging .....\n");
        if(mode == 5) {  // for debug
            strcpy(outfile2,dirtemp);
            strcat(outfile2,fdebug);
        } else {
            strcpy(outfile2,outfile);
        }
        printf("File merging into  %s\n",outfile2);
        
        fd_out = open(outfile2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
        if (fd_out == -1) {
            perror("output file open() error");
            goto end2;
        }
        for(i=0; i<nfile; i++){
            strcpy(tmpfile1,filetable[i]);
            printf("%d %d : %s\n",nfile,i+1,tmpfile1);
            if((fin = fopen(tmpfile1, "rb"))==NULL){
                perror("Data file open error!\n");
                goto end2;
            }
            while(1){
                rv = fread(buf,1,nbyte,fin);
                if (rv == -1) {
                    perror("read() failed ");
                    goto end2;
                }
                rv2=write(fd_out,buf,rv);
                if (feof(fin) !=0)  // Reach EOF
                {
                    fclose(fin);
                    goto exit1;
                }
            }
        exit1:;
        }
        close(fd_out);
    }

 end2:;
    return rv;
}



//int sampleget4(char* outfile, int span, int sfreq, int adbit, int numch, char* dirtemp, int mode)
int sampleget4(spget_t *spg, int mode)
{
/*      sampleget3 ̉ǔ  (₦VNp^[̌osj

                mode    -- t@Co̓[h
                       1: ꊇt@Ĉ iftHgj
                       2: Pbf[^t@Ĉ
                       3:  ꊇt@C{Pbƃt@C
                       4:  Pbf[^t@CóAϑꊇt@C쐬
                       5:  3+4  for debug purpose ꊇt@C debug.dat 쐬
                       6: Pbf[^t@Ĉ (Ԑj
                       7: UƓit@Co͖jVXe`FbNp
*/
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in;
    int fd_out, fd_out_tmp;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv, rv2, rv3, rv2s;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    int i,h,m,s;
    int secforce;
    int headbit=64;  // header bit number  for a new sampler board
    //int headbit=32;  // header bit number  for a old sampler board
    int allbyte;     // TvOoCg
    int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
    int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
    int c, c2;
    unsigned int sampling1, freq, width;
    int ch;
    int sec_counter;    // second counter for tempolary file creation
    int sec_counter_max;
    long int byteinter;   // inter header byte counter
    int rv32;
    int bytepos; //  byte position for start of sync pattern
    int fmode;         // file out mode
    int endflg, nbyte, nfile;
    int flag1sec;     // 1sec (sync) detection flag  1:detected
    int flag1st;
    char* namnow="sampler.now"; //1sec file name sampling out now 
    char* namlast="sampler.dat"; //1sec file name sampling out just finished 
    char tmpnow[80],tmplast[80];
    char tmpfile1[80],tmpfile2[80], outfile2[80];
    char fname1[20],fname2[20];
    char filetable[1000][80];  // 1sec file name container  
    int nfilemax;
    int fvalid;   // 1 sec file number exceeds 3 then set 1 
    int fpos;
    int ierr;
    unsigned char bit64[8];  // for header check
    int hspatn, hiok, hsmplbit, hsfreq, hnumch, hhh,hmm,hss;
    int hseconds, seconds, secold;
    char* fdebug="debug.dat";
    FILE *fin;
    
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    u_int32_t buf[BUFSIZ * 2], lastbuf;
    
    nfilemax=1000;
    sec_counter_max=1000;  // sec_counter reset if exceeds this value
    fvalid=0;
    flag1sec=0;
    flag1st=0;
    bytepos=0;
    byteinter=0;
    //lastbuf = (u_int32_t *)malloc(sizeof(u_int32_t));
    //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI

/*
    printf(" span = %d\n",span);    // debug
    printf(" sfreq= %d\n",sfreq);   // debug
    printf(" adbit= %d\n",adbit);   // debug
    printf(" numch= %d\n",numch);   // debug
*/
    /* File out mode check */
    if(mode <=0 || mode > 4) {
        fmode=1;
    } else {
        fmode=mode;
    }
    if(mode == 5) {
        fmode=3;
    }
    if(mode == 6 || mode ==7 ) {
        fmode=2;
        nfilemax=10;
        sec_counter_max=10;
    }

    /* sampling duration check */
    if (spg->span <= 0) {
        printf(" parameter span (duration in sec) should be positive!\n");
        // nothing to do
        return -1;
    }


    /* output file (1 big file) open */
    if(fmode == 1 || fmode ==3){
        fd_out = open(spg->outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
        if (fd_out == -1) {
            perror("output file open() error");
            goto end;
        }
    }
    sec_counter=0;
    /* tempolary file (for 1 sec data write name) creation */
    //strcpy(tmpnow,dirtemp);
    strcpy(tmpnow,spg->outdir);
    //strcpy(tmplast,dirtemp);
    strcpy(tmplast,spg->outdir);
    strcat(tmpnow,namnow);
    strcat(tmplast,namlast);
    sprintf(fname2,"smp%04d.dat\0",sec_counter);  
    //strcpy(tmpfile2,dirtemp);
    strcpy(tmpfile2,spg->outdir);
    strcat(tmpfile2,fname2);
    
    /* tempolary output file open */
    if(mode !=7 ){
        
        if(fmode == 2 || fmode ==3 || fmode == 4){
            fd_out_tmp = open(tmpnow, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out_tmp == -1) {
                perror("temprary output file open() error");
                goto end;
            }
        }
    }


    /* sampling set and start sampling */
    //if((rv=ini_and_start_sampler(&fd_in, sfreq, adbit, numch, &bitsec))<0) {
    if((rv=ini_and_start_sampler(&fd_in, &spg->sp, &bitsec))<0) {
         goto end2;
    }

     bytesec=bitsec/8;
     allbyte=bytesec*spg->span;

     /* File maximum bytes check */
     if(mode !=6 && mode !=7 ){
        if (allbyte > FILEMAXSIZE || allbyte < 0){
            allbyte= FILEMAXSIZE;
            secforce=allbyte/bytesec;
         
            allbyte=secforce*bytesec;
            printf(" # of bytes force to set to %d Bytes because it exceeds %d Bytes\n",
            allbyte,FILEMAXSIZE);  // debug
            printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
        }
    }
    printf(" bitsec  = %d\n",bitsec);  // debug
    printf(" bytesec = %d\n",bytesec);  // debug

    printf(" allbyte = %d\n",allbyte);  // debug
    
    //rv=start_sampler(&fd_in);
    //if(rv < 0) goto end;

    c = 0;   // byte counter for one big file
    c2= 0;   
    endflg=0;
    nfile=0;
    nbyte = sizeof(buf);   // nbyte = 8192
    //while (c < BUFSIZ * BUFSIZ) {
    while (1) {
        
        rv = read(fd_in, buf, nbyte);
        if (rv == -1) {
            perror("read() failed ");
            goto end;
        }
        if(rv != nbyte) {printf("read byte # difference %d\n",rv);}
        //printf(" nbyte = %d\n",nbyte);   // debug
        
        /* sync pattern check */
        rv32=rv/sizeof(u_int32_t);
        if(c > 0) {
#ifdef _VSSP32
            if(lastbuf==0xFFFFFFFF && ((buf[0] & 0xFF000000)==0x8C000000)) { // this is sync pattern for VSSP32
#else
            if(lastbuf==0xFFFFFFFF && ((buf[0] & 0xFF000000)==0x8B000000)) { // this is sync pattern for VSSP
#endif
                seconds=(int)(buf[0] & 0x1FFFF);
                printf("Sync detected  byteinter %d  seconds %d\n",byteinter, seconds);
                if( secold+1 != seconds){ // time error
                    if((secold != 86399) && (seconds != 0)){
                         printf("Time increment error!\n");
                    }
                }
                if(bytesec != byteinter){
                    printf("%d bytes data missed\n",bytesec-byteinter);
                }
                bytepos=-4;
                flag1sec=1;
                secold=seconds;
                byteinter=0;
            } 
            byteinter+=4;
        }
            
        for(i=0; i<rv32-1; i++){
#ifdef _VSSP32
            if(buf[i]==0xFFFFFFFF && ((buf[i+1] & 0xFF000000)==0x8C000000)) { // this is sync pattern
#else
            if(buf[i]==0xFFFFFFFF && ((buf[i+1] & 0xFF000000)==0x8B000000)) { // this is sync pattern
#endif
                seconds=(long int)(buf[i+1] & 0x1FFFF);
                printf("Sync detected  byteinter %d  seconds %d \n",byteinter, seconds);
                if( c > 0 && ( secold+1 != seconds)){ // time error
                    if((secold != 86399) && (seconds != 0)){
                        printf("Time increment error!\n");
                    }
                }
                if(c > 0 && (bytesec != byteinter)){
                    printf("%d bytes data missed\n",bytesec-byteinter);
                }
                bytepos=i*sizeof(u_int32_t);
                flag1sec=1;
                secold=seconds;
                byteinter=0;
            } 
            byteinter+=4;
        }
        lastbuf=buf[rv32-1];
        /*
        if(c == 0) {
            printf("%x %x ",*(buf), *(buf+1));
            headerchkn((unsigned char*)(buf),&hiok,&hspatn,&hsmplbit,&hsfreq,&hnumch,
                    &hseconds,&hhh,&hmm,&hss);
            if (hiok == 0) {  // sync detection
                printf("OK ");
                printf("%dbit %dch %dkHz",hsmplbit,hnumch, hsfreq);
                printf(" %02d:%02d:%02d %d\n",hhh,hmm,hss,hseconds);
            } else {
                printf("First 64bit is not header block!\n");
            }
        }
        */
        c += rv;      // already read byte number
        //printf(" nbyte,rv,c  %d %d %d\n",nbyte,rv,c);   // debug
        if(mode !=6 && mode !=7 ){
            if( c >= allbyte ){
                rv3=c-allbyte;
                rv2=rv-rv3;
                endflg=1;  // end flag set
                if(fmode == 1 || fmode ==3){
                    rv2 = write(fd_out, buf, rv2);
                    close(fd_out);
                }
            } else {
                if(fmode == 1 || fmode ==3){
                    rv = write(fd_out, buf, rv);
                }
            }
        } else {
            c=1;   // for inifinite operation for mode=6 or =7
        }
        
        /* every 1 sec file creation */
        if(fmode == 2 || fmode ==3 || fmode == 4){
            if(flag1st ==1 && flag1sec ==1 ) {
                rv3=rv-bytepos;
                rv2=bytepos;
                //printf("1st c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  // debug
                if(mode !=7){
                    if(bytepos > -4 ) {
                        if(flag1st != 0) c2=write(fd_out_tmp,&lastbuf,4);
                    }
                    if(bytepos > 0 ) {
                         rv2=write(fd_out_tmp,buf,rv2);
                    }
                    /* tempolary file update */
                    close(fd_out_tmp);
                    if(fvalid == 1){
                        fpos=nfile-1;
                        if(fpos < 0){ fpos=fpos+nfilemax;}
                    
                        //printf("%d %s\n",fpos,filetable[fpos]);
                        /* folowing is not necessary for FreeBSD */
                    /*
                        if((ierr=remove(filetable[fpos])) <0){
                        if(ierr != -1 ) perror("File remove error");
                        }
                    */
                        if (rename(tmplast,filetable[fpos]) <0) {
                            perror("Rename tmplast error");
                        }
                    }else{
                        if(nfile >=0) fvalid=1;
                    }
                
                    /* folowing (remove) is not necessary for FreeBSD */
                    /*
                    if((ierr=remove(tmplast)) <0){
                    if(ierr != -1 ) perror("File remove error");
                    }
                    */
                
                    if (rename(tmpnow,tmplast) <0){
                        perror("File tmpnow rename error");
                    }
                
                }
                strcpy(filetable[nfile],tmpfile2); // save file name info
                
                //printf("nfile %d   %s %s\n",nfile,filetable[nfile],tmpfile2);
                nfile++;
                if(nfile >= nfilemax) nfile=0;
                /* new templary file nmae creation */
                sec_counter++;
                if(sec_counter >= sec_counter_max) sec_counter=0;
                //sprintf(fname1,"smp%04d.now\0",sec_counter);  
                sprintf(fname2,"smp%04d.dat\0",sec_counter);  
                //strcpy(tmpfile2,dirtemp);
                strcpy(tmpfile2,spg->outdir);
                strcat(tmpfile2,fname2);
                
                if(endflg !=1 ){
                    if(mode !=7 ){
                        /* tempolary output file open */
                        //fd_out_tmp = open(tmpfile1, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
                        fd_out_tmp = open(tmpnow, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
                        if (fd_out_tmp == -1) {
                            perror("temprary output file open() error");
                            goto end;
                        }
                    }
                    /* rest of data write */
                    if(bytepos > 0) {
                        rv2s=rv2/sizeof(u_int32_t);
                        if(mode !=7 ){
                            rv3=write(fd_out_tmp,buf+rv2s,rv3-4);
                        }
                    } else {
                        if(mode !=7 ){
                            rv3=write(fd_out_tmp,buf,rv-4);
                        }
                    }
                        //printf("after  c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  // debug
                    c2=rv3;
                    //printf("final  c2,rv3,rv2   %d %d %d\n",c2,rv3,rv2);  //debug
                }
            } else {
                if(mode !=7 ){
                    if(flag1st !=0) c2=write(fd_out_tmp,&lastbuf,4);
                    rv2=write(fd_out_tmp,buf,rv-4);
                }
            }
            flag1sec=0; //flag reset
        }
        flag1st=1;
        if(endflg == 1) goto loopexit;
            
    }

loopexit:

       /* sampling stop */
end:
    rv = stop_sampler(&fd_in);
    
    if(mode !=7 ){
        
    if(fmode == 2 || fmode ==3 || fmode == 4){
        if(fvalid == 1){
            fpos=nfile-1;
            if(fpos < 0) fpos=fpos+nfilemax;
            /*
            if(remove(filetable[fpos]) <0)
            perror("File remove error");
            */
            /* just rename */
        //  if (rename(tmplast,filetable[fpos]) <0) 
        //      perror("Rename tmplast error");
            
            /* Copy file to leave last file name */
            fd_out = open(filetable[fpos], O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
            if (fd_out == -1) {
                perror("output file open() error");
                goto end2;
            }
            if((fin = fopen(tmplast, "rb"))==NULL){
                perror("Data file open error!\n");
                goto end2;
            }
            while(1){
                rv = fread(buf,1,nbyte,fin);
                if (rv == -1) {
                    perror("read() failed ");
                    goto end2;
                }
                rv2=write(fd_out,buf,rv);
                if (feof(fin) !=0)  // Reach EOF
                {
                    fclose(fin);
                    goto exit2;
                }
            }
        exit2:;
            
            close(fd_out);
        }
    }
        
    }
    //close(fd_out);
    //close(fd_out_tmp);
    //if (rename(tmpfile1,tmpfile2) <0) 
    //  perror("Rename error");
    
    /* 1sec data file merges into one file big file */
    
    if(fmode == 4 || mode ==5) {  
        printf("File merging .....\n");
        if(mode == 5) {  // for debug
            //strcpy(outfile2,dirtemp);
            strcpy(outfile2,spg->outdir);
            strcat(outfile2,fdebug);
        } else {
            strcpy(outfile2,spg->outfile);
        }
        printf("File merging into  %s\n",outfile2);
        
        fd_out = open(outfile2, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
        if (fd_out == -1) {
            perror("output file open() error");
            goto end2;
        }
        for(i=0; i<nfile; i++){
            strcpy(tmpfile1,filetable[i]);
            printf("%d %d : %s\n",nfile,i+1,tmpfile1);
            if((fin = fopen(tmpfile1, "rb"))==NULL){
                perror("Data file open error!\n");
                goto end2;
            }
            while(1){
                rv = fread(buf,1,nbyte,fin);
                if (rv == -1) {
                    perror("read() failed ");
                    goto end2;
                }
                rv2=write(fd_out,buf,rv);
                if (feof(fin) !=0)  // Reach EOF
                {
                    fclose(fin);
                    goto exit1;
                }
            }
        exit1:;
        }
        close(fd_out);
    }

 end2:;
    return rv;
}





        
int ip_board_time(unsigned int *day, unsigned int *sec, int *hh, int *mm, int *ss)
{
     int fd_in,rv;
     unsigned int time;
     //fd_in = open("/dev/tds0", O_RDONLY);  //foCX̃I[v
     fd_in = open(DEV_FILE, O_RDONLY);  //foCX̃I[v
     if (fd_in == -1) {
        perror("ip_board_time: device open() error ");
        return -1;
     }
     //foCX玞̎擾 
     rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
     if (rv == -1) {
         perror("ip_board_time: ioctl() time get failed ");
         return -1;
     }
     *day=TDS_GET_DAYS(time);
     *sec=TDS_GET_SEC(time);
     sec2hms(*sec,&(*hh),&(*mm),&(*ss));
    close(fd_in);  // ho|ukah{[hN[Y
    return 0;
}

int toffset_get(int *yearPC, float *tofset) 
// get time difference between IP-BOARD time and PC time   2003-08-21
{ 
    int fd_in,rv;
    int year,month,mday,hh,mm,ss,day,h,m,s,sec,secold,counter;
    unsigned int time;
    float fsec;
     //fd_in = open("/dev/tds0", O_RDONLY);  //foCX̃I[v
     fd_in = open(DEV_FILE, O_RDONLY);  //foCX̃I[v
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    //foCX玞̎擾 and display while
    secold=100000;
    counter=0;
    
    do{
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1) 
        {
            perror("ioctl() time get failed ");
            return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        if (sec!=secold)        // to catch exact second tic
        {
            sec2hms(sec,&h,&m,&s);
            printf("%03d %02d:%02d:%02d\n",day,h,m,s);
            if (counter >=1 ) { goto exith1 ;}
            counter++; 
        }
        secold=sec;
    }while(counter <=10);
exith1:
    
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, tofset);
    *tofset=(float)sec-((float)(hh*3600+mm*60+ss)+fsec); 
    printf("toffset_get: System Time %04d/%02d/%02d %02d:%02d:%02d\n",year,month,mday,hh,mm,ss);
    printf("toffset_get: Offset %f\n",*tofset);
    *yearPC=year;  // save year info get from PC
    
    close(fd_in);  // ho|ukah{[hN[Y
    return 0;
}
    
int LogWrite2(char* logdir, char* hostname, int time_mode, 
              float secofset, int mode, char* cbuf)
{
/*
  [in]
     logdir  -----  logfile directory
     hostname ----  host PC name
     time_mode ---  0: time get from PC
                    1: time get from IP-VLBI board
     mode   ------  logfile separator mode
                    0:  : 
                    1:  /
                    2:  "
                    3:  ?
                    4:  #
                    5:  (space)
     cbuf           log statement
 
  ---------
       logfilename  logdir/YYYYhostname.log

       TIME(UT) get from host
       log format
            YYDDDHHMMSSXY cbuf
    where    YY   -- year (2 digits)
             DDD  -- total day  (day of year)
             HH   -- hour
             MM   -- minute
             SS   -- sec
             X    -- separator mode
             Y    -- time_mode   (space) for IP-Board Time
                                  p  for PC-TIME
             cbuf -- Any comment
    
       A logfile will be opened as an append mode every time.

*/
    int i,ilen,ddd,y1,y2,imode;
    int hh,mm,ss,month,mday,year;
    unsigned int day, secnow;
    float fsec,tofset;
    int ip[7];
    char logfile[90], wbuf[30];
    char septab[][2]={":","/","\"","?","#"," "};
    FILE* flog;
    //time_t tnow;              // for getting date and time info
    //struct tm tmnow;          //        "

    // current time get
    tofset=secofset;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    secnow=hh*3600+mm*60+ss;
    md2ddd(year, &ddd, month, mday);
    if(time_mode == 1) {
       if(ip_board_time(&day, &secnow, &hh, &mm, &ss)==-1) return -1 ;
       ddd=day;
       ddd2md(year, ddd, &month, &mday);
    }           

    ip[0]=year;
    ip[1]=ddd;
    ip[2]=hh;
    ip[3]=mm;
    ip[4]=ss;
    ip[5]=month;
    ip[6]=mday;
    y1=ip[0] % 10;
    y2=ip[0] % 100;
    //printf("CURRENT TIME(UT) %s\n", asctime(&tmnow)); //debug
    //for (i=0; i<7; i++) printf("%d ",ip[i]); printf("\n"); //debug

    /* logfilename create */
    strcpy(logfile,logdir);
    ilen=strlen(logfile);
    if (logfile[ilen-1] != '/'){
        logfile[ilen]='/';
        logfile[ilen+1]='\0';
    }
    sprintf(wbuf,"%04d%s.log\0",year,hostname);
    strcat(logfile,wbuf);
    if( (flog  = fopen( logfile, "a" )) == NULL ){
        printf("LogWrite: Log File (%s) Open Error! \n",logfile);
        return -1;
    }
    //printf("LogWrite: Log File (%s) Opened! \n",logfile);  // debug
    
    imode=mode;
    if(imode <0 || imode >5) imode=5;
    if(time_mode==1){
       sprintf(wbuf,"%02d%03d%02d%02d%02d%s  \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    } else {
       sprintf(wbuf,"%02d%03d%02d%02d%02d%sp \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    }
    fprintf(flog,"%s%s\n",wbuf,cbuf);
    i=fclose(flog);
    return 0;
}
    
void LogWrite3(char* logdir, char* hostname, char* logfile, int logfiletype,
              int time_mode,  float secofset, int mode, char* cbuf)
{
/*
  [in]
     logdir  -----  logfile directory
     hostname ----  host PC name
     logfile  -----  logfile name
     logfiletype --  logfile name definitiion
                      0: logfile
                      1: logdir/YYYY+hostname+".log"
                         where  YYYY is 4 digit year
     time_mode ---  0: time get from PC
                    1: time get from IP-VLBI board
     secofset ----  second offset
     mode   ------  logfile separator mode
                    0:  : 
                    1:  /
                    2:  "
                    3:  ?
                    4:  #
                    5:  (space)
     cbuf           log statement
 
  ---------

       TIME(UT) get from host
       log format
            YYDDDHHMMSSXY cbuf
    where    YY   -- year (2 digits)
             DDD  -- total day  (day of year)
             HH   -- hour
             MM   -- minute
             SS   -- sec
             X    -- separator mode
             Y    -- time_mode   (space) for IP-Board Time
                                  p  for PC-TIME
             cbuf -- Any comment
    
       A logfile will be opened as an append mode every time.

*/
    int i,ilen,ddd,y1,y2,imode;
    int hh,mm,ss,month,mday,year;
    unsigned int day, secnow;
    float fsec,tofset;
    int ip[7];
    char wbuf[30];
    char septab[][2]={":","/","\"","?","#"," "};
    FILE* flog;
    //time_t tnow;              // for getting date and time info
    //struct tm tmnow;          //        "
    if(logfiletype < 0) goto exit_end;  // nothing to do
    // current time get
    tofset=secofset;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    secnow=hh*3600+mm*60+ss;
    md2ddd(year, &ddd, month, mday);
    if(time_mode == 1) {
        if(ip_board_time(&day, &secnow, &hh, &mm, &ss)==-1) {
           printf("LogWrite3: IP-BOARD TIME GET ERROR! \n");
           goto exit_end;
        }
       ddd=day;
       ddd2md(year, ddd, &month, &mday);
    }           

    ip[0]=year;
    ip[1]=ddd;
    ip[2]=hh;
    ip[3]=mm;
    ip[4]=ss;
    ip[5]=month;
    ip[6]=mday;
    y1=ip[0] % 10;
    y2=ip[0] % 100;
    //printf("CURRENT TIME(UT) %s\n", asctime(&tmnow)); //debug
    //for (i=0; i<7; i++) printf("%d ",ip[i]); printf("\n"); //debug

    /* logfilename create */
    if(logfiletype==1){
       strcpy(logfile,logdir);
       ilen=strlen(logfile);
       if (logfile[ilen-1] != '/'){
           logfile[ilen]='/';
           logfile[ilen+1]='\0';
       } 
       sprintf(wbuf,"%04d%s.log\0",year,hostname);
       strcat(logfile,wbuf);
    }

    /* logfile open */
    if( (flog  = fopen( logfile, "a" )) == NULL ){
        printf("LogWrite3: Log File (%s) Open Error! \n",logfile);
        goto exit_end;
    }
    //printf("LogWrite: Log File (%s) Opened! \n",logfile);  // debug
    
    imode=mode;
    if(imode <0 || imode >5) imode=5;
    if(time_mode==1){
       sprintf(wbuf,"%02d%03d%02d%02d%02d%s  \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    } else {
       sprintf(wbuf,"%02d%03d%02d%02d%02d%sp \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    }
    fprintf(flog,"%s%s\n",wbuf,cbuf);
    i=fclose(flog);
exit_end:;

}
    
void LogWrite4(logval_t *lg, int time_mode,  float secofset, int mode, char* cbuf)
{
/*
  [in]
     log->logdir  -----  logfile directory
     log->hostname ----  host PC name
     log->logfile  -----  logfile name
     log->logfiletype --  logfile name definitiion
                      0: logfile
                      1: logdir/YYYY+hostname+".log"
                         where  YYYY is 4 digit year
                     -1:  no log
     time_mode ---  0: time get from PC
                    1: time get from IP-VLBI board
     mode   ------  logfile separator mode
                    0:  : 
                    1:  /
                    2:  "
                    3:  ?
                    4:  #
                    5:  (space)
     cbuf           log statement
 
  ---------

       TIME(UT) get from host
       log format
            YYDDDHHMMSSXY cbuf
    where    YY   -- year (2 digits)
             DDD  -- total day  (day of year)
             HH   -- hour
             MM   -- minute
             SS   -- sec
             X    -- separator mode
             Y    -- time_mode   (space) for IP-Board Time
                                  p  for PC-TIME
             cbuf -- Any comment
    
       A logfile will be opened as an append mode every time.

*/
    int i,ilen,ddd,y1,y2,imode;
    int hh,mm,ss,month,mday,year;
    unsigned int day, secnow;
    float fsec,tofset;
    int ip[7];
    char wbuf[30];
    char septab[][2]={":","/","\"","?","#"," "};
    FILE* flog;
    //time_t tnow;              // for getting date and time info
    //struct tm tmnow;          //        "
    if(lg->logfiletype < 0) goto exit_end;  // nothing to do
    // current time get
    tofset=secofset;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    secnow=hh*3600+mm*60+ss;
    md2ddd(year, &ddd, month, mday);
    if(time_mode == 1) {
        if(ip_board_time(&day, &secnow, &hh, &mm, &ss)==-1) {
           printf("LogWrite4: IP-BOARD TIME GET ERROR! \n");
           goto exit_end;
        }
       ddd=day;
       ddd2md(year, ddd, &month, &mday);
    }           

    ip[0]=year;
    ip[1]=ddd;
    ip[2]=hh;
    ip[3]=mm;
    ip[4]=ss;
    ip[5]=month;
    ip[6]=mday;
    y1=ip[0] % 10;
    y2=ip[0] % 100;
    //printf("CURRENT TIME(UT) %s\n", asctime(&tmnow)); //debug
    //for (i=0; i<7; i++) printf("%d ",ip[i]); printf("\n"); //debug

    /* logfilename create */
    if(lg->logfiletype==1){
       strcpy(lg->logfile,lg->logdir);
       ilen=strlen(lg->logfile);
       if (lg->logfile[ilen-1] != '/'){
           lg->logfile[ilen]='/';
           lg->logfile[ilen+1]='\0';
       } 
       sprintf(wbuf,"%04d%s.log\0",year,lg->hostname);
       strcat(lg->logfile,wbuf);
    }

    /* logfile open */
    if( (flog  = fopen( lg->logfile, "a" )) == NULL ){
        printf("LogWrite4: Log File (%s) Open Error! \n",lg->logfile);
        goto exit_end;
    }
    //printf("LogWrite: Log File (%s) Opened! \n",logfile);  // debug
    
    imode=mode;
    if(imode <0 || imode >5) imode=5;
    if(time_mode==1){
       sprintf(wbuf,"%02d%03d%02d%02d%02d%s  \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    } else {
       sprintf(wbuf,"%02d%03d%02d%02d%02d%sp \0",y2,ddd,ip[2],ip[3],ip[4],septab[imode]);
    }
    fprintf(flog,"%s%s\n",wbuf,cbuf);
    i=fclose(flog);
exit_end:;

}

