/**************************************************************/
/*                                                            */
/*   File name :  libskdvex.h                                 */
/*                                                            */
/*      Version:   2006-05-09                                 */
/*                                                            */
/*   Copyright (c) 2005,2006 T.Kondo/NICT All Right Reserved  */
/*                                                            */
/**************************************************************/
/*

2006-05-09
  das_def_t das[20] in vex type structure was changed to das_def_t das[40]
2006-03-17
  new member (usb) was added to schedule type variable to set
  sideband in case of SKED file
2005-07-25
  To reflect "tape early start parameter" in SCHED and VEX,
  das_def_t type was introduced and sked_data_t and vex_t types were 
  modified. Moreover DAS_anal and DAS_mon functions were added and
  vex_read function was updated.  
  Readskd_param function was added and readsked fucntion was updated.
  (line_parse function was also added.)
2005-06-12
  libsked.h and libvex.h are merged into libskdvex.h.
  libskdvex.h is divided into libskdvex.h and libskdvex.c

*/
#ifndef _LIBSKDVEX_
#define _LIBSKDVEX_

#include "chead.h"
#include "libipvlbi.h"
#include <math.h>
#include <ctype.h>

extern void ddd2md(int year, int ddd, int* month, int* mday);

typedef struct obs_file_naming
{   // file naming input type variable
     int naming_type;       // naming type;
     char expcode[11];     // experiment code
     char statid[4];       // station ID  A1 or A2
     char statid2[4];      // station ID  A2
     char scanid[20];      // scan ID
     char freqg[4];        // frequency group ID a|b|c|d
     char pcid[4];         // PC ID  '1'|'2'|'3'|'4'

     int numch;   // # of Ch  (1 or 4)
     int obsnum;  // current obs# (1,2,3....)

     //int year;    // 4 digit year
     //int dddf;   // totalday at 1st scan
     //int ddd;    // totalday at current scan
     //int hh;     // hour at the start of current scan
     //int mm;     // minutes at the start of current scan
     //int ss;     // second at the start of current scan
     timeval_k5_t tf;  // time info at 1st scan
     timeval_k5_t tn;  // time info at current scan

     
} obs_file_naming_t;

    /* data structure for SKED (DRUDG) file data */
typedef struct schedule
{
     char expcode[11];           // experiment code
     char starnam1[MAXSTAR][11]; // star name1
     char starnam2[MAXSTAR][11]; // star name2
     double ra[MAXSTAR];         // star position RA  in degree
     double dec[MAXSTAR];        //       "       DEC in degree
     double epo[MAXSTAR];        //       "       Epoch
     int    nstar;               // number of stars 

     char statnam[MAXSTAT][11];       // station name
     double xyz[MAXSTAT][3];          // station position (x,y,z) in m
     char statid[MAXSTAT][3];         // 1 char or 2 char station ID 
     char statid2[MAXSTAT][3];         // 1 char or 2 char station ID 
     int nstation;               // number of stations

     double freqtab[16];         // RF frequency table (Hz)
     double pcalf[10][16];      // PCAL frequency table (Hz)  modified on 2003-07-18
                                //  up to 10 th PCAL frequency
     int nfreq;                  // number of frequency
     // scan info
     char obsstar[MAXOBS][11];   // Observation sourcer name
     int starid[MAXOBS];         // Star ID  (entry at starpos table)
     int start[MAXOBS][5];       // start time   YY, DDD, HH, MM, SS
     int dura[MAXOBS];           // obs duration (sec)
     char obsid[MAXOBS][21];     // station IDs participated in this obs
     int nobs;                   // Number of observations (scans)
     char scanid[MAXOBS][21];    // added on 2005-01-13
     char skdname[61];           // schedule file name
     char skedtype[10];          // sked type SKED|VEX|OVEX
     //double x_clock_offset;

     char sideband[16][4];       // side band  U|L or USB|LSB
     int datastart[MAXOBS][MAXSTAT];   // data start time offset 
                                 //   from obs start time (sec) 
     int datastop[MAXOBS][MAXSTAT];    // data stop  time offset from obs start time (sec)
     int station_obs[MAXOBS][MAXSTAT]; // station participated in scan info (1: obs  0: no obs)

     /* following is added  to contain early start info */ //2005-07-25
     int earlysec; // early start parameter (sec)
     /* following is added  to set sideband from operator */ //2006-03-17
     int usb; // 1: USB  0: LSB
     
} sked_data_t;

typedef struct skedline  // structure for each scan data
{
    char source[10];   // radio source
    int start[5];      // start time  YY,DDD,HH,MM,SS
    int dura;          // max scan length in second
    int nstat;         // participated stations in this scan
    char id_stat[MAXSTAT][3]; // participated station id
    int dura_stat[MAXSTAT];   // scan length by station
} skedline_t;

/*******/
/* primitive block structure for VEX analysis*/

typedef struct global
{
    char exper[30];     // link id to $EXPER block
    char eop[30];       // link id to $EOP block
} global_t;

typedef struct exper
{
    char def[30];         // def id
    char exper_name[20];  // experiment code (name)
} exper_t;

typedef struct mode_item
{
    char mode[30];   // major link id to other block
    char id[20];     // link id to STATION block
} mode_item_t;

typedef struct fanout_def
{
    char sub_pass[2];      // sub pass character (usually A|B )
    char chan_id[20];      // link id to chan_def in $FREQ block
    char stream_info[5];   // sign | mag
    int headstack;         // head stack number
    int track[4];          // track number from first multiplex to last multiplex
    int fanout;            // fanout
} fanout_def_t;
 

typedef struct chan_def
{
    double rf;         // RF frequency (Hz)
    char sideband[2];  // net side band  U|L
    double vbw;        // band width (Hz)
    char chan_id[20];  // Channel ID link to $TRACK
    char bbc_id[20];   // BBC ID  link to $BBC
    char pcal_id[20];  // Pcal ID  link to $PHASE_CAL
} chan_def_t;

typedef struct tracks
{
    char     def[50];            // def id
    char  format[20];            // track frame format ex. Mark4
    int    multiplex;            // multiplex 1|2|4 
    int    bitssampl;            // bits/sampl
    //fanout_def_t fanout_def[32]; // fanout def info (fanout_def type)
    fanout_def_t fanout_def[64]; // fanout def info (fanout_def type) 2003-10-10
    int nfanout_def;             // number of fanout_def
} tracks_t;

typedef struct freq
{
    char   def[50];              // def id
    double samplerate;           // sampling rate (Hz/s)
    int numch;                   // number of channels
    //chan_def_t chan_def[16];     // channel def info (chan_def type)
    chan_def_t chan_def[64];     // channel def info (chan_def type)
} freq_t;

typedef struct if_def            // added on 2003-07-17
{
    char if_id[20];              // IF ID link to $BBC
    char pol[2];                 // Polarization R|L
    double lo;                   // total local frequency (Hz)
    char sideband[2];            // net side band  U|L
    double pcal_space;           // PCAL freq spacing (Hz)
    double pcal_base;            // PCAL freq base (Hz)
} if_def_t;

typedef struct if_block          // added on 2003-07-17
{
    char   def[50];              // def id
    int nif;                   // number of IF definition
    if_def_t if_def[10];         // IF def info (if_def type)
} if_block_t;

typedef struct pcal_detect
{
    char pcal_id[20];     // Pcal id link to $FREQ block
    int ntone;            // number of tones
    int tone[10];  // tone number info for pcal detection
} pcal_detect_t;

typedef struct pcal_block             // added on 2003-07-17
{
    char def[50];                   // def id
    int npcal;                         // number of PCAL definition
    pcal_detect_t pcal[20];         // PCAL def info
} pcal_block_t;

typedef struct bbc          // added on 2003-07-17
{
    char bbc_id[20];     // BBC id link to $FREQ block
    char if_id[20];     // IF id link to $IF block
    int bbcch;        // Physical BBC number
} bbc_t;

typedef struct bbc_block          // added on 2003-07-17
{
    char   def[50];       // def id
    char nbbc;     // number of BBC block
    bbc_t bbc_def[20];    // BBC def 
} bbc_block_t;

typedef struct siteclock
{
    char   def[50];        // def id
    int validity_epoch[5]; // yyyy,ddd,hh,mm,ss
    double clock_offset;   // clock offset wrt UTC  >0 for formatter tick early
    int clock_epoch[5];    // yyyy,ddd,hh,mm,ss
    double clock_rate;     // s/s
} siteclock_t;

typedef struct site
{
    char   def[50];        // def id
    char   type[20];       // site type 'fixed' or others
    char   name[20];       // site name
    char   id[3];          // 2 chars site id
    double position[3];    // site position X,Y,Z (m)
    siteclock_t sclock;    // site clock info [siteclock_t type]
} site_t;


typedef struct source
{
    char    def[50];         // def id
    char    type[20];        // star|satellite
    char    name[20];        // source name
    char    IAU_name[20];    // IAU name
    double  ra;              // RA in degree
    double  dec;             // DEC in degree
    char    epoch[10];       // epoch  J2000|B1950 etc
} source_t;

typedef struct eop
{
    char  def[50];     // def id
    int  epoch[5];     // EOP epoch yyyy, ddd, hh, mm, ss
    int    points;     // number of eop data points
    int interval_hr;   // eop data interval (unit is hour)
    double ut1[10];    // UT1-UTCs (sec)
    double xwob[10];   // Wobbling X parameters (asec)
    double ywob[10];   // Wobbling Y parameters (asec)
} eop_t;

typedef struct station_def
{
    char def[20];      // def id
    char das[20];      // link id to DAS block
    char clock[20];    // link id to CLOCK block
    char site[20];     // link id to SITE block
    char antenna[20];  // link id to ANTENNA block
} station_def_t;

typedef struct das_def  // added on 2005-07-25
{
    char def[50];      // def id
    int earlysec;      // tape early start param (sec)
    // other parameters are omitted
} das_def_t;

typedef struct station_in_scan
{
    char id[3];       // 2 chars station id
    int data_start;   // sec  valid data offset from start
    int data_stop;    // sec  stop time from start
} station_in_scan_t;

typedef struct scan
{
    char scanid[20];  // scan id (used as a Mark 5 data file name)
    char mode[30];    // link id to MODE block (describing track, freq link)
    int number;       // scan (obs) number in an experiment
    int start[5];     // scan start time   yyyy,ddd,hh,mm,ss
    char source[20];  // source name
    station_in_scan_t station[MAXSTAT];  // scan period by station
    int numstat;      // acutual stations in station[MAXSTAT}
} scan_t;

typedef struct roll
{
    char    def[50];      // def ID
    char    rolldef[4];   // on|off
} roll_t;

typedef struct vexmode    // structure for MODE block data
{
    char def[30];                    // def id
    mode_item_t mode_freq[MAXSTAT];  // link to $FREQ block info by station (link id to $STATION)
    mode_item_t mode_track[MAXSTAT]; // link to $TRACK block info by station ( " )
    mode_item_t mode_roll[MAXSTAT];  // link to $ROLL block info by station  ( " )
    mode_item_t mode_pcal_detect[MAXSTAT]; // link to PCAL block info by station ( " )
    mode_item_t mode_if[MAXSTAT]; // link to $IF block info by station ( " )
    mode_item_t mode_bbc[MAXSTAT]; // link to $BBC block info by station ( " )
    int nmode;                // used only for count mode_freq and mode_track count
} vexmode_t;
   

typedef struct vex
{
    char vex_file_name[60];    // VEX file name
    char block[50][20];        // found $block name table  50x20chars
    int nblock;                // found $block number
    char flag[10];             // vex type identifier VEX|OVEX
    global_t global;           // $GLOBAL block info
    exper_t exper[10];         // $EXPER block info up to 10
    int nexper;                // number of def in $EXPER block
    vexmode_t mode[10];        // $MODE block info
    int nmode;                 // number of def in $MODE block
    site_t site[MAXSTAT];      // $SITE block info
    int  numstat;              // number of def in $SITE block
    source_t source[MAXSTAR];  // $SOURCE block info
    int  numstar;              // number of def in $SOURCE block
    //freq_t freq_def[5];        // $FREQ block info
    freq_t freq_def[20];        // $FREQ block info   2003-07-04
    int  nfreq_def;            // number of def in $FREQ block
    tracks_t track_def[10];    // $TRACK block info
    int ntrack_def;            // number of def in $TRACK block
    scan_t  scan[MAXOBS];      // $SCHED info
    int  numobs;               // number of observations in $SCHED block
    station_def_t station[MAXSTAT]; // $STATION block info
    int nstation;                   // number of def in $STATION block
    siteclock_t siteclock[MAXSTAT]; // $CLOCK block info
    int nclock;                     // number of def in $CLOCK block
    eop_t   eop[10];           // $EOP block info
    int neop;                  // number of def in $EOP block
    roll_t  roll[10];          // $ROLL block info
    int nroll;                 // number of def in $ROLL block
    pcal_block_t pcal[20];    // $PCAL block info
    int npcal;                 // number of def in $PCAL block
    bbc_block_t  bbc[20];    // $BBC block info
    int nbbc;                 // number of def in $BBC block
    if_block_t  ifb[20];    // $IF block info
    int nifb;                 // number of def in $IF block

    /* following is added  to contain early start info */ //2005-07-25

    //das_def_t das[20]; // DAS info including early start parameter
    das_def_t das[40]; // DAS info including early start parameter //2005-05-09
    int ndas;          // number of def in $DAS block
} vex_t;


typedef struct sidtable
{
    char sid[10];
    char site[20]; // site name 2005-01-13
} sidtable_t;

typedef struct freq_info
{
    char chanid[10];       // channel ID
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    int nch;              // channel # for this info
    int chmax;            // maximum channels in original table
    int bbc;              // BBC# 2004-12-25
    char bbcid[20];       //  BBC ID  2004-12-25
} freq_info_t;


typedef struct track_freq_sub
{
    int fan_out_pos;      // 1 for 1st   2 for secondary
    int headstack;    // head stack position  1,2,3,4  added 2004-09-13
    int track;            // track number
    char chan_id[20];     // chan id
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    char bitstream[10];       //  sign|mag
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    int bbc;              // BBC#  2004-12-25
    char bbcid[20];       //  BBC ID  2004-12-25
    char sub_pass[2];    // pass info (A|B) 2004-06-29
} track_freq_sub_t;

typedef struct track_freq
{
    //track_freq_sub_t trk[32];
    track_freq_sub_t trk[64];     // 2003-10-10
    char trackdef[30];
    char freqdef[30];
    int ntrack;
    int nfreq;
    int adbit;
    double samplrate;     // Sampling rate sampls/sec
    int fanout;        // fanout 1|2|4
    char roll[30];
} track_freq_t;

typedef struct vparam
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} vparam_t;


double clock_get(siteclock_t clock, int *prt);  //2003-07-13
void sa_clock_get(vex_t *vex, char *stat_id1, char *stat_id2, int *prt, 
                  double *coffset, double *crate, double *xcoff);  //2005-05-19
void eop_get2(eop_t eop, int *prt, double *ut1, double *xwob, double *ywob);  //2003-07-13
void eop_get(vex_t *vex, int *prt, 
                  double *ut1, double *xwob, double *ywob);  //2003-07-13
void parsetime1(char *ctime, int *itime);  //2003-03-21
void asc_ra2deg(char *ra, double *radeg); //2003-03-21
void asc_dec2deg(char *dec, double *decdeg); //2003-03-21
int vex_parse(char *cbuf, vparam_t *vparam);  //2003-03-21
int GLOBAL_ana(FILE *fin, char *line, vex_t *vex); //2003-03-21
int EXPER_ana(FILE *fin, char *line, vex_t *vex);  //2003-03-21
int SKED_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int TRACKS_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int SITE_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int STATION_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int SOURCE_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int CLOCK_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int EOP_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int ROLL_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int FREQ_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int MODE_ana(FILE* fin, char *line, vex_t *vex);  //2003-07-17
int PCAL_ana(FILE* fin, char *line, vex_t *vex); /* 2004-06-17 */
int BBC_ana(FILE* fin, char *line, vex_t *vex); //2003-07-17
int IF_ana(FILE* fin, char *line, vex_t *vex); //2003-07-17
int DAS_ana(FILE* fin, char *line, vex_t *vex); //2005-07-25
void MODE_mon(vex_t *vex);  //2003-03-21
void SKED_mon(vex_t *vex);  //2003-03-21
void TRACKS_mon(vex_t *vex);  //2003-03-21
void SITE_mon(vex_t *vex);  //2003-03-21
void STATION_mon(vex_t *vex);  //2003-03-21
void SOURCE_mon(vex_t *vex);  //2003-03-21
void FREQ_mon(vex_t *vex);  //2003-03-21
void CLOCK_mon(vex_t *vex);  //2003-03-21
void CLOCK_mon2(vex_t *vex, char *stat_id);  //2003-07-13
void CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock);  //2003-07-19
void EOP_mon(vex_t *vex);  //2003-03-21
void EOP_mon2(vex_t *vex);  //2003-03-21
void PCAL_mon(vex_t *vex);  //2003-07-16
void BBC_mon(vex_t *vex);  //2003-07-16
void IF_mon(vex_t *vex);  //2003-07-16
void DAS_mon(vex_t *vex);  //2005-07-25
void vex_read(vex_t *vex); 
int search_site(vex_t *vex, char *key, site_t *site);  /* 2004-06-17 */
int search_site2(vex_t *vex, char *key, site_t *site);  /* 2005-01-12 */
freq_info_t vex_freq_get(vex_t *vex, char *mode, char *sid, char *chanid);
freq_info_t vex_chinfo_get(vex_t *vex, char *mode, char *sid, int nch);
int search_track_freq(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq);
int search_track_freq2(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq);
void create_siteid_table(vex_t *vex, int *nstation, sidtable_t *siteid);  //2003-03-21
int check_block(vex_t *vex);  //2003-03-21


/*******/
int line_parse(char *line, vparam_t *vparam);  //2005-07-25

void SkdMonit(sked_data_t *skd);
void stcode_get(char *stcode_file, char *sid2, char *sid1);
void vex2sked(vex_t *vex, char *stcode_file, sked_data_t *skd);
void vex2sked2(vex_t *vex, sked_data_t *skd);
void vex2sked3(vex_t *vex, char *stat_id, sked_data_t *skd);
void skedlineana(char *line, skedline_t *skl);
int readskd_source(FILE *fin, char *line, sked_data_t *skd);
int readskd_station(FILE *fin, char *line, sked_data_t *skd);
int readskd_freq(FILE *fin, char *line, sked_data_t *skd);
int readskd_sked(FILE *fin, char *line, sked_data_t *skd);
int readskd_param(FILE *fin, char *line, sked_data_t *skd); //added on 2005-07-25
void readskd(sked_data_t *skd);   
void readskd_for_nozomi(char *skdname, char *baseid, char *apefile, int *iok);
void check_sked_type(char* skdname, char* skedtype);
void pickup_skd(sked_data_t *skd, char *stat_id);
void setscanid_skd(sked_data_t *skd);
void scan_file_naming(obs_file_naming_t *ofn, char *fname);


#endif
