/**************************************************************/
/*                                                            */
/*   File name :  libspeplot.h                                */
/*                                                            */
/*      Version:   2006-01-31                                 */
/*                                                            */
/*   Copyright (c) 2006 T.Kondo/NICT All Right Reserved       */
/*                                                            */
/**************************************************************/
/*

2006-01-31
  libspeplot.h includes functions used for spectrum plot.

*/
#ifndef _LIBSPEPLOT_
#define _LIBSPEPLOT_

void plotspe_a(float *spow, int npos, int sfreq, int adbit, 
               char *lab, char *lab2, int mode2,
               float ymaxdb, float ymindb);
void plotspe_a2(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int mode, 
               double f1khz, double f2khz, float *pmaxdat, float *fdatmax, int nmax);
void plotautocor(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int amode);

/******************************************/

void plotspe_a(float *spow, int npos, int sfreq, int adbit, 
               char *lab, char *lab2, int mode2,
               float ymaxdb, float ymindb)
/*
               mode2 -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale
                          Y: amplitude axis mode
                             =0: log scale 
                             =1: linear scale (common for 4 channels)
                                  (use ymaxdb, ymindb)
                             =2: linear scale (each max and min)
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin;
      float fscale;
      float *fdat, *powdat;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax;
      int mode,ymode;

      ymode=mode2/10;
      mode=mode2 % 10;
      strcpy(htitl,"POWER SPECTRUM");

      /* frequency axis generation */

      npmax=npos/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      /* frequency axis */
      fsampl=(float)1000.0*(float)sfreq;
      df=fsampl/(float)npos;
      if(sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(mode == 0) {    // log scale
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++){fdat[n]=(float)log10((float)n*df/fscale); }
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         for(n=0; n<npmax; n++){fdat[n]=(float)((float)n*df/fscale); }
      }

      /* amplitude axis */

      /* data copy, if necessary convert to linear scale */
      if(ymode == 0){    // log scale
         for(n=0; n<npmax; n++) powdat[n]=spow[n];
      } else {   // linear scale
         /* convert to linear scale */
         for(n=0; n<npmax; n++) powdat[n]=(float)pow(10.0,spow[n]/10.0);
      }

      /* data max, min check */
      for(n=0; n<npmax; n++){
         //powdat[n]=spow[n];
         if(n==0){
            pmax=powdat[n];
            pmin=powdat[n];
         } else {
            if(powdat[n] > pmax){ pmax=powdat[n]; }
            if(powdat[n] < pmin){ pmin=powdat[n]; }
         }
      }

      if(ymode == 0){    // log scale
         strcpy(ytit,"POWER (dBm)");
         if( adbit==1 ){strcpy(ytit,"POWER (dB)"); }
         ymax=10.0;
         ymin=-80.0;
         if (adbit == 8 ) ymin=-100;     // extend dynamic range 
      } else {   // linear scale
         strcpy(ytit,"POWER (linear)");
         ymax=pmax;
         ymin=pmin;
         if(ymode==1){
            ymax=(float)pow(10.0,ymaxdb/10.0);
            ymin=(float)pow(10.0,ymindb/10.0);
         }
      }


      //maxdbm=pmax;
      //mindbm=pmin;

      //goto exitend; //debug

      /* open graphics device */
      /*   move to main */
       //if (cpgopen("?") < 1)
      // if (cpgopen("/GF") < 1)
      // {
      //    goto exitend;
      // }

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(mode == 0){   
         cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,lab);
       cpgmtxt("T",0.5,0.5,0.5,lab2);
       cpgsch(2.0);
      
      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }
      /* close the graphics device */
      /* move to main */
      //cpgclos();
  //exitend: 
     free(powdat);
     free(fdat);
}

void plotspe_a2(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int mode, 
               double f1khz, double f2khz, float *pmaxdat, float *fdatmax, int nmax)
/*
        mode = 0; log frequency scale   1; linear frequency scale
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float fscale;
      float *fdat, *powdat;
      float fatmax;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax,k, ncount,nmaxpos;
      //float pmaxdat[4],fdatmax[4];
	  //nmax=4;
      strcpy(htitl,"POWER SPECTRUM");

      /* frequency axis generation */

      npmax=npos/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      fsampl=(float)(1000.0*(float)sfreq);
      df=fsampl/(float)npos;
      if(sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(mode == 0) {    // log scale
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++){fdat[n]=(float)log10((float)n*df/fscale); }
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         //fmaxkhz=990.0;
         //fminkhz=920.0;
      	 if(f1khz != 0.0) fmin=(float)(f1khz/(fscale/1000.0));
      	 if(f2khz != 0.0) fmax=(float)(f2khz/(fscale/1000.0));
         for(n=0; n<npmax; n++) fdat[n]=(float)(n*df/fscale); 
      }

      strcpy(ytit,"POWER (dBm)");
      if( adbit==1 ){strcpy(ytit,"POWER (dB)"); }
      ymax=10.0;
      ymin=-80.0;
      //ymax=-34.0;
      //ymin=-40.0;
      if (adbit == 8 ){ ymin=-100; }    // extend dynamic range 

      /* data max, min check */
	  //printf("max search start %d\n",npmax);
	  
	  for(n=0; n<npmax; n++) powdat[n]=spow[n];
	
      pmin= (float)1.0e30;
	  for(ncount=0; ncount<nmax; ncount++){
          pmax=(float)(-1.0e30);
		  k=0;
		  for(n=0; n<npmax; n++){
            if(fdat[n]>=fmin && fdat[n]<=fmax){
               if(powdat[n] > pmax){ 
                   pmax=powdat[n];
                   fatmax=fdat[n];
               	   pmaxdat[ncount]=pmax;
               	   fdatmax[ncount]=(float)(fatmax*(fscale/1000.0));  // to obtain in kHz
               	   nmaxpos=n;
               }
           	   if(ncount==0) {
           		    if(powdat[n] < pmin) pmin=powdat[n]; 
           	   }
               k++;
           	}
          }
		 powdat[nmaxpos]=(float)(-10.0e30);
	  	 //printf("pmax,fatmax, i %f %f %d\n",pmax,fatmax,i);
      }
	 /*
      printf("Number of points = %d\n",k);
      printf("1st Max %fdbm at %fkHz\n",pmaxdat[0],fdatmax[0]);
      printf("2nd Max %fdbm at %fkHz\n",pmaxdat[1],fdatmax[1]);
      printf("3rd Max %fdbm at %fkHz\n",pmaxdat[2],fdatmax[2]);
      printf("4th Max %fdbm at %fkHz\n",pmaxdat[3],fdatmax[3]);
	 */
      maxdbm=pmaxdat[0];
      mindbm=pmin;
      ymax=pmaxdat[0];
      ymin=pmin;
	  //printf("ymax,ymin %f %f\n",ymax,ymin);
	
	  for(n=0; n<npmax; n++) powdat[n]=spow[n];
      //goto exitend; //debug

      /* open graphics device */
      /*   move to main */
       //if (cpgopen("?") < 1)
      // if (cpgopen("/GF") < 1)
      // {
      //    goto exitend;
      // }

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(mode == 0){   
         cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,lab);
       cpgmtxt("T",0.5,0.5,0.5,lab2);
       cpgsch(2.0);
      
      /* check data */
      if (pmax == pmin)
      {
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      }
      else
      {


          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }
      /* close the graphics device */
      /* move to main */
      //cpgclos();
     free(powdat);
     free(fdat);
}


void plotautocor(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int amode)
/*
        mode = 0; log frequency scale   1; linear frequency scale
*/
{
      //float cofst,tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin;
      float *tdat, *pdat;
      //float x,y;
      char htitl[60], xtit[60], ytit[60];
      int n, k, npmax, nhalf;

      strcpy(htitl,"AUTO CORRELATION (ABS)");

      /* time axis generation */

      if(amode <= 1) {
         npmax=npos;
      } else {
         npmax=amode;
      }
      tdat = (float *)malloc(npmax*sizeof(float));
      pdat = (float *)malloc(npmax*sizeof(float));
      nhalf=npmax/2;

      fsampl=(float)(1000.0*(float)sfreq);
      dt=(float)(1.0/fsampl);
      for(n=0; n<npmax; n++){
          tdat[n]=(float)((float)(n-nhalf)*dt*1.0e6);   // usec unit
      }
      tmax=tdat[npmax-1];
      tmin=tdat[0];

      strcpy(xtit,"DELAY TIME (us)");
      strcpy(ytit,"CORRELATION AMP");
      ymax=(float)1.1;
      ymin=0.0;

      /* data range limited copy and max, min check */
     
      for(n=0; n<npmax; n++)
      {
         k=n-nhalf;
         if(k < 0) k=npos+k;
         pdat[n]=spow[k];
         if(pdat[n]<0) pdat[n]=-pdat[n];  // to take absolute value
         if(n==0)
         {
            pmax=pdat[n];
            pmin=pdat[n];
         }
         else
         {
            if(pdat[n] > pmax){ pmax=pdat[n]; }
            if(pdat[n] < pmin){ pmin=pdat[n]; }
         }
      }
      

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      cpgenv(tmin, tmax, ymin, ymax, 0, 0);   // linear scale
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,lab);
       cpgmtxt("T",0.5,0.5,0.5,lab2);
       cpgsch(2.0);
      /* check data */
      if (pmax == pmin)
      {
          cpgmtxt("B",-6.0,0.5,0.5,"NO DATA FOR PLOT");
          
      }
      else
      {
      
        /* plot the line graph */
         cpgline(npmax,tdat,pdat); 
      }

      /* move to main */
  //exitend: 
     free(pdat);
     free(tdat);
}


#endif
