/**************************************************************/
/*                                                            */
/*   File name :  libvssp32.h                                 */
/*                                                            */
/*      Version:   2006-01-31                                 */
/*                                                            */
/*   Copyright (c) 2006 T.Kondo/NICT All Right Reserved       */
/*                                                            */
/**************************************************************/
/*

2006-01-31
  libvssp32.h includes type definitions and functions 
  for utilities under src directory those use K5/VSSP32 format data.

*/
#ifndef _LIBVSSP32_
#define _LIBVSSP32_

#define REWIND_BYTE 200 
//#define USE_ASM    // use inline assemble function

typedef struct K5VSSP_file     // originated from ip_vlbi_file type
                               // difference is displayed by **K5**
{
    /*
      Note: use of a fpos_t type variable was obsoleted because of
         the compatiblity ploblem of a fpos_t type variable between linux 2.6
         and previous one.
    */
	//unsigned char bit32[4];  // obsoleted in **K5**
	//unsigned char bit64[8]; // obsoleted in **K5**
	unsigned char *ibuf; // will be obsoleted in **K5**
	u_int32_t head32[8]; // **K5**  to contain full header including 
                         // in case of K5/VSSP32
	u_int32_t *aux32; // **K5**  to contain aux field start address in
                        // head 32
	u_int32_t *ubuf; // pointer to read buffer **K5**
    int nword;       // ubuf size allocated or to be allocated  **K5**
    int auxbyte;     // # of bytes in aux files  **K5**
                     // valid byte number in aux32  (deflt=20)
    int headsizew;    // header size in 32bit-word
    int *idata;   // pointer to decoded data (integer) container
    double *data;   // pointer to decoded data (R*8) container
	int sfreq;  //  sampling frequency in kHz
	int adbit;  // AD bits (1,2,4,8)
	int numch;  // # of channels
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit # of samples
	int kpos;  // should be obsoleted ?
	int npos;    //  valid number in idata or xdata **K5**
	int imax;    //  # of units in a second data
	int bytesec;    //  # of bytes in a second data  **K5**
	//char* fname; //pointer data file name obsoleted
	char fname[128]; //data file name opened or to be opened  **K5**
	double fsample;  // sampling frequency (Hz)
	double tsample; // sampling period (sec)
	double unitp;   // usampl*tsampl unit length 
    long int seconds; // time (sec) from mid night (00h00m00s)
	int hh, mm, ss;  // time in hour, minutes, seconds 
    int year; // 4 digits year (K5/VSSP32)
    int totalday; // (K5/VSSP32)
    int version1; // Major version (K5/VSSP32)
    int version2; // Minor version (K5/VSSP32)
	int spatn; // detected 2nd sync pattern detected 
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 K5/VSSP, =2 K5/VSSP32
                //         =2 is added **K5**
	FILE *stream;
	//fpos_t pos;   // changed to int64_t type 
	//fpos_t size;  // changed to int64_t type
                    // because fpos_t type is no more scalar value in linux 2.6
	int64_t pos;   // current pointer
	int64_t size;  // opened file size in bytes **K5**
	int fh;
	int ch;    
	int ierr;   // error code (general purpose)
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;  // number of bytes read
	int wordread;   // # of 32bit-words read counter (general) **K5**
	int64_t twordread;   // total # of 32bit-word read **K5**
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
} K5VSSP_file_t;


int CheckK5header_with_Read(K5VSSP_file_t *K5);
int64_t get_file_size(char *fname);
void vssp_aux_parse(vsspaux_t *vsspaux);
int CheckK5header(K5VSSP_file_t *K5);
void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit);
void parseADdata_3N(K5VSSP_file_t *K5);
int CheckK5header_with_Read2(K5VSSP_file_t *K5, int maxword);
int fwdNsecK5(int nsec, K5VSSP_file_t *K5);
int fwd1secK5(K5VSSP_file_t *K5);

/******************************************/

int64_t get_file_size(char *fname)
{
    /*  get file size opened */
    struct stat info;
    stat(fname,&info);
    return (int64_t)info.st_size;
    // use %lld to print out whole number that has int64_t type
}


void vssp_aux_parse(vsspaux_t *vsspaux)
{
/*  parse AUX field data 
    [in]
      vsspaux->size  ----  AUX filed size in bytes
      vsspaux->bdat  ----  AUX field
    [return]
      vsspaux->fmt;  ----  AUX field format
      vsspaux->filter;  ----  FILTER parameter ( when fmt !=0)
      vsspaux->sid2; ----  2 char Station ID ( when fmt=1)
      vsspaux->station8;  -- station name (8 char)
      vsspaux->hostname8; -- Host PC name (8 char)
*/

    int n;
    /* fill out with space code */
    strcpy(vsspaux->sid," ");
    strcpy(vsspaux->sid2,"  ");
    strcpy(vsspaux->station8,"        ");
    strcpy(vsspaux->hostname8,"        ");
    
    /* format field get */
    vsspaux->fmt=(int)vsspaux->bdat[0];
    /* filter field get */
    vsspaux->filter=(int)vsspaux->bdat[1];
    if(vsspaux->fmt==1){
       /* station field get */
       vsspaux->sid[0]=vsspaux->bdat[1];
       vsspaux->sid[1]='\0';
       vsspaux->sid2[0]=vsspaux->bdat[2];
       vsspaux->sid2[1]=vsspaux->bdat[3];
       vsspaux->sid2[2]='\0';
       for(n=0; n<8; n++) vsspaux->station8[n]=vsspaux->bdat[4+n];
       vsspaux->station8[8]='\0';
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->hostname8[n]=vsspaux->bdat[12+n];
       vsspaux->hostname8[8]='\0';
    } else if(vsspaux->fmt==2){
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->hostname8[n]=vsspaux->bdat[12+n];
       vsspaux->hostname8[8]='\0';
    }
}



int header_aux_read(K5VSSP_file_t *K5)
{
   // read Aux field in K5/VSSP32 header
   // assumption : pointer is located just next to 64bit common header

       int i, i32, n;

       i32=0;

       /* next 32bit data read */
       K5->wordread = fread( K5->head32+2, sizeof( u_int32_t ), 1, K5->stream );
       K5->twordread+=K5->wordread;
       K5->pos+=K5->wordread*4;  // pointer position is in bytes
       if( ferror( K5->stream ) || K5->wordread==0 ){ // File EOF or read error
	       i=fclose( K5->stream ) ;
	       i32=-1;
           goto exit_here;
       }
       // parse header into each item
       K5->year=2000+(int)((K5->head32[2] >> 9) & 0x7F); // year
       K5->totalday=(int)(K5->head32[2] & 0x1FF);
       K5->version1=(int)((K5->head32[2] >> 28) & 0xF);
       K5->version2=(int)((K5->head32[2] >> 24) & 0xF);
       K5->auxbyte =(int)((K5->head32[2] >> 16) & 0xFF);
       //printf("year = %d auxbyte = %d\n",K5->year,K5->auxbyte);
       for(n=0; n<K5->auxbyte/4; n++){
           K5->wordread = fread( K5->head32+3+n, sizeof( u_int32_t ), 1, K5->stream );
           K5->twordread+=K5->wordread;
           K5->pos+=K5->wordread*4;  // pointer position is in bytes
           if( ferror( K5->stream ) || K5->wordread==0 ){ // File EOF or read error
	           i=fclose( K5->stream ) ;
		       i32=-1;
               goto exit_here;
	       }
       }
exit_here:
       return i32;

}

// K5 header check (updated from headerchkn2)
int CheckK5header(K5VSSP_file_t *K5)
/*
    K5 header common portion (1st 64bits) check

    if 1st 64bits show K5/VSSP32 pattern,  extended header and
    AUx field are read


    return 0 : K5/VSSP Header detected
          -1 : K5.head32 is not header
         -99 : EOF detected
*/
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; 
    int smplbit0, sfreq0, nch;
    u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
    u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
    int iok;
    iok=-1;

    K5->format=0;  // suppose K5/VSSP old format
    if(K5->head32[0]==0xFFFFFFFFUL){
        if((K5->head32[1] & 0xFF000000UL) == syncpat) K5->format=1; // this is sync block
        if((K5->head32[1] & 0xFF000000UL) == syncpat32) K5->format=2; // this is sync block
        //if((K5->head32[0]==0xFFFFFFFFUL) && ((K5->head32[1] & 0xFF000000UL) == syncpat)){ // this is sync block
        //if((K5->head32[0]==0xFFFFFFFFUL) && ((K5->head32[1] & 0xFF000000UL) == syncpat32)){ // this is sync block
        if(K5->format > 0){ 
            iok=0;
            /* parse sampling information and get time data */ 
            K5->spatn=(int)((K5->head32[1] >> 24) & 0xFF);       
            smplbit0 = (K5->head32[1] >> 22) & 0x3;   
            K5->adbit = sdat[smplbit0];      // AD bits
            sfreq0 = ( K5->head32[1] >> 18) & 0xF;
            K5->sfreq=fdat[sfreq0];
            nch = (K5->head32[1] >> 17) & 1;
            if (nch == 1) {
                K5->numch = 4; 
            } else {
                K5->numch = 1;
            }
            K5->seconds=(long int)(K5->head32[1] & 0x1FFFFUL);
            sec2hms(K5->seconds,&K5->hh,&K5->mm,&K5->ss);
        }
        if(K5->format==2){  // read further header
            K5->headsizew=8;
            if(header_aux_read(K5)<0) iok=-99;
       }

    }
    return iok;
}

#ifdef USE_ASM

    #ifdef WIN32
        void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)  // for VC++ inline assembler
        {
        /*
                eax  --  K5pos
                ebx  --  wd
                ecx (cl)  --  adbit
                edx  --  ilast
                esi  --  loop counter
                edi  --  temporary use of wd
        
                ebp+14h --  pat

        */
                __asm{
                    mov edx, ilast
                    mov eax, K5pos
                    mov ebx, wd
                    mov cl, byte ptr [adbit]
                    //mov ch, byte ptr [pat]  //not necessary
                    xor esi,esi  
            loop_1:
                    mov edi, ebx
                    and edi, pat
                    mov [eax+esi*4], edi
                    shr ebx, cl
                    inc esi
                    cmp esi, edx
                    jnz loop_1
                }
        }
    #else  // for gcc
        void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)
        {
        /*
                K5pos --  8(%ebp)
                wd    --  12(%ebp)
                ilast --  16(%ebp)
                pat   --  20(%ebp)
                adbit --  24(%ebp)

                eax  --  K5pos
                ebx  --  wd
                ecx (cl)  --  adbit
                edx  --  ilast
                esi  --  loop counter
                edi  --  temporary use of wd
        

        */
            asm volatile (
                 "pushl %eax \n\t"     // save register used in this function
                 "pushl %ebx \n\t"
                 "pushl %ecx \n\t"
                 "pushl %edx \n\t"
                 "pushl %edi \n\t"
                 "pushl %esi \n\t"

                 "movl 16(%ebp), %edx \n\t"
                 "movl  8(%ebp), %eax \n\t"
                 "movl 12(%ebp), %ebx \n\t"

                 "movb 24(%ebp), %cl \n\t"
                 "xor %esi, %esi \n\t"  
            "loop_10: \n\t"
                 "movl %ebx, %edi \n\t"
                 "and  20(%ebp), %edi \n\t"
                 "movl %edi, (%eax,%esi,4) \n\t"
                 "shr %cl, %ebx  \n\t"
                 "inc %esi \n\t"
                 "cmp %edx, %esi \n\t"
                 "jnz loop_10 \n\t"
                 "popl %esi \n\t"    // recover saved register 
                 "popl %edi \n\t"
                 "popl %edx \n\t"
                 "popl %ecx \n\t"
                 "popl %ebx \n\t"
                 "popl %eax \n\t"
                );
        }
    #endif
#else  // normal C function
    void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)
    {
        int i,ww, wd1;
        wd1=wd;
        for(i=0; i<ilast; i++){
            ww=(int)(wd1 & pat);
            //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
            K5pos[i]=ww;
            wd1>>= adbit ; 
        }
    }
#endif

void parseADdata_3N(K5VSSP_file_t *K5)
{
/*
    decode K5/VSSP (IP-VLBI) data into integer data

      [in]
         K5->ubuf    ---  sampler (K5/VSSP (IP-VLBI) board) out data 
         K5->wordread --  # of data in K5->ubuf (32bit word unit)
         K5->adbit -- sampler A/D resolution bits (1,2,4,8)
      [return]
         K5->idata -- data parsed into each sampled data
         K5->npos  -- valid number of data in xdata array

     Relation between # of ch (numch) and each ch time siries

        sdata(m,k)=xdata(k*numch+m)

             where  numch :  # of channels  (1, 4)
                      m   :  CH#
                      k   :  k-th sampling
        
*/
     int numb8, pat, n, ilast, ipos;
     int pattbl[9]={1,1,3,3,15,15,15,15,255};
     u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
     u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
     u_int32_t wd1,wd2;
     int *K5pos;
     unsigned char bpat;

     if(K5->vflag != 1) goto exitend;
     //numb8=numb;
     numb8=K5->wordread*4;

     ipos=0;
     ilast=32/K5->adbit;
     pat=pattbl[K5->adbit];
     bpat=(unsigned char)pat;
     wd1=K5->ubuf[0];

     //printf("Debug IN PARSE adbit, pat %d %d\n",K5->adbit,pat);
     K5pos=K5->idata;
     for( n=1; n < K5->wordread; n++){
        wd2=K5->ubuf[n];
        /* check sync pattern */
        if ( (wd1 == 0xFFFFFFFFUL) && (((wd2 >> 24) & 0xFF) == (K5->spatn & 0xFFUL)) ) { // sync detected for K5/VSSP
            printf("Sync detected \n"); 
            continue;  // pass decoding
        }
//        for(i=0; i<ilast; i++){
//             ww=(int)(wd1 & pat);
//             //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
//             K5->idata[ipos]=ww;
//             wd1=wd1 >> K5->adbit ; 
//             ipos++;
//        }
        /* use of inline assembler */
        K5pos=&K5->idata[ipos];
        decode_data(K5pos, wd1, ilast, pat, K5->adbit);   // use of inline assembler
        //decode_data2(K5pos, wd1, ilast, pat, K5->adbit); // normal C function
                              // no big difference found between decode_data and decode data2
                              //   ex. decode_data  34.279 sec for 8MHz 1bit 4CH 20 sec data
                              //   ex. decode_data2 34.289 sec for 8MHz 1bit 4CH 20 sec data
                              //  direct use like below is also almost same processing speed
                              //     i.e. 34.289 sec
/*
        __asm{
            mov edx, ilast
            mov eax, K5pos
            mov ebx, wd1
            mov cl, bpat
            xor esi,esi  
    loop_1:
            mov edi, ebx
            and edi, pat
            mov [eax+esi*4], edi
            shr ebx, cl
            inc esi
            cmp esi, edx
            jnz loop_1
        }
*/
        ipos+=ilast;

        wd1=wd2;
        //K5pos+=ilast;

     }
/*     
     for(i=0; i<ilast; i++){
         ww=(int)(wd1 & pat);
         //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
         K5->idata[ipos]=ww;
         wd1=wd1 >> K5->adbit ; 
         ipos++;
     }
*/     

     //K5pos=K5->idata+ipos;
     K5pos=&K5->idata[ipos];
     decode_data(K5pos, wd1, ilast, pat, K5->adbit);  
     //decode_data2(K5pos, wd1, ilast, pat, K5->adbit);
     /* use of inline assembler */
/*
     __asm{
        mov edx, ilast
        mov eax, K5pos
        mov ebx, ww
        mov cl, bpat
        xor esi,esi  
loop_2:
        mov edi, ebx
        and edi, pat
        mov [eax+esi*4], edi
        shr ebx, cl
        inc esi
        cmp esi, edx
        jnz loop_2
     }
*/
     ipos+=ilast;

     K5->npos=ipos;

exitend:;
     //printf("numb8,ipos %d %d\n",numb8,ipos);
}




int CheckK5header_with_Read(K5VSSP_file_t *S)
{
    /* 
       Read header check to get sampling parameters 
    
       if first 64bit data is not header block, read file 
       untill finding header block.

       After call, stream pointer is set to jsut next to the header
       block.

       Return
               +ve   # of words read till header finding
               -1    no header found till EOF
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       S->headsizew=2; // for K5/VSSP
       //S->headsizew=8; // for K5/VSSP32

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       if ((i=CheckK5header(S)) == 0) {  // sync detection
           goto exit_loop;
       } else if(i==-99){
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}

int CheckK5header_with_Read2(K5VSSP_file_t *S, int maxword)
{
    /* 
       Read 64bit Data header check to get sampling parameters 
    
       if first 64bit data is not header block, read file at maximum 
       maxword 32bit-words untill finding header block.

       Return
               +ve   # of words  read till header finding
               -1    no header found after maxword read
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
       //                 &S->seconds,&S->hh,&S->mm,&S->ss);
       if ((i=CheckK5header(S)) == 0) {  // sync detection
           goto exit_loop;
       } else if(i==-99){
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
           if(i32+2 > maxword) return -1;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}


int fwdNsecK5(int nsec, K5VSSP_file_t *K5)
{
    /* 
       forward fast N sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind 1sec data
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one

    */
    int i,k;
    int maxword;
    int nbytes, nback;
    int64_t tword;
	int64_t pos;

    if(nsec < 1) return 0; // nothing to do

    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    nbytes=K5->bytesec*nsec;
    K5->pos+=nbytes;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,nbytes,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=nbytes/4;
    K5->twordread+=(int64_t)K5->wordread;


    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if((i=CheckK5header(&(*K5))) < 0){  // this is not the case of header
       if(i==-99) return -1;
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               if(nsec==1){
                   nback=(int)(K5->pos-pos);  // rewind to initial position
               } else {
                   nback=K5->bytesec;   // rewind just 1 sec
               }

               fseek(K5->stream,-nback,SEEK_CUR); // set position at initial position
               K5->twordread-=nback/4;            // word counter rewind
               K5->pos-=nback;  // initial position rewind
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

int fwd1secK5(K5VSSP_file_t *K5)
{
    /* 
       forward fast 1 sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind previous header position
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one
    */
    int i,k;
    int maxword;
    int64_t tword;
	int64_t pos;
    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    K5->pos+=K5->bytesec;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,K5->bytesec,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=K5->bytesec/4;
    K5->twordread+=(int64_t)K5->wordread;

    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if((i=CheckK5header(&(*K5))) < 0){  // this is not the case of header
       if(i==-99) return -1;
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
               K5->twordread=tword;            // word counter retrieve
               K5->pos=pos;  // retrieve initial position
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

#endif
