/**************************************************************/
/*                                                            */
/*   File name :  libvssp32b.h                                */
/*                                                            */
/*      Version:   2006-01-31                                 */
/*                                                            */
/*   Copyright (c) 2006 T.Kondo/NICT All Right Reserved       */
/*                                                            */
/**************************************************************/
/*

2006-01-31
  libvssp32b.h includes type definitions and functions 
  for utilities under src directory those handle K5/VSSP and VSSP32 
  format data and convert in AD resolution, number of channels etc.

*/
#ifndef _LIBVSSP32B_
#define _LIBVSSP32B_

typedef struct ipboard {
    int adbit;         // A/Drbg (1,2,4,8)
    int numch;         // ͂@(1,4)
    int sfreq;         // TvOg(kHz)(40,100,200,500,1000,2000,4000,8000,16000)
    long int seconds;    // Ǒoߕb
    //unsigned char hd[8]; // ho{[hwb_[ 
    unsigned char hd[32]; // ho{[hwb_[ 
    long int nbsec;       // Pbf[^̃oCgiwb_[܂ށj 
    long int bpos;       // ݊JnoCgʒuiOX^[gj
} IPBOARD_T;

void ip_hd_make_ini(IPBOARD_T *ipb, int psync2);
void ip_hd_make(IPBOARD_T *ipb);
int checkheader(ip_vlbi_file_t *S);
int fwdNsecHere(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat);
int conv4byte2int(unsigned char *b4);
int conv32bit2int(int *idat32);

/***********************************/


void ip_hd_make_ini(IPBOARD_T *ipb, int psync2)
{
    /* IP-VLBI{[ho̓f[^wb_[ׂč쐬 */
    int n,fid,aid;
    long int w1;
    unsigned char b,c;
    double fsampl;
    /* sync pattern 1 set */
    for (n=0; n<4; n++){
       ipb->hd[n]=0xFF;
    }
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=(unsigned char)((w1 & 0x10000) >> 16);
    /* ch flag set */
    if( ipb->numch == 4){
        b=b | 0x2;
    }
    /* sampling freq index set */
    fid=0;
    //if( ipb->sfreq ==    40) fid=0;
    if( ipb->sfreq ==   100) fid=1;
    if( ipb->sfreq ==   200) fid=2;
    if( ipb->sfreq ==   500) fid=3;
    if( ipb->sfreq ==  1000) fid=4;
    if( ipb->sfreq ==  2000) fid=5;
    if( ipb->sfreq ==  4000) fid=6;
    if( ipb->sfreq ==  8000) fid=7;
    if( ipb->sfreq == 16000) fid=8;
    if( ipb->sfreq == 32000) fid=9;
    if( ipb->sfreq == 64000) fid=10; //2006-01-31
    c=(unsigned char)(fid << 2);
    b=b | c;
    /* A/D bit length set */
    aid=0;
    //if( ipb->adbit == 1 ) aid=0;
    if( ipb->adbit == 2 ) aid=1;
    if( ipb->adbit == 4 ) aid=2;
    if( ipb->adbit == 8 ) aid=3;
    c=(unsigned char)(aid << 6);
    b=b | c;
    ipb->hd[6]=b;
    /* sync pattern 2 set */
    //ipb->hd[7]=0x8B;
    ipb->hd[7]= (unsigned char)(psync2 & 0xFF);
    /* internal data initialize */
    fsampl=1000.0*(double)ipb->sfreq; // sampling frequency (Hz)

    ipb->nbsec=(((long)fsampl)*ipb->adbit*ipb->numch)/8;  // # of bytes in 1 sec data except header
    ipb->nbsec+=8;
    ipb->bpos=0;
}

void ip_hd_make(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[̂ݍXV */
    long int w1;
    unsigned char b,c;
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=ipb->hd[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    ipb->hd[6]=b;
}


int checkheader(ip_vlbi_file_t *S) // original is in libcor.c
{
/*
     2006-01-03

    Read 1st header to get sampling prameters and 
    to check sampler type either K5/VSSP or K5/VSSP32
    
       if first block is not header block, found header block automatically
       return value > 0 means above condition occurred
                    =-1 means EOF detection (file closed)

*/
      
       int iok, i, n, i32;
       int cdat[2]={1,4};
       int sdat[4]={1,2,4,8};
       int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                     32000,64000,128000,256000,512000,1024000,2048000}; 
       union HEAD_VSSP hd;
       //fpos_t save;
       i32=0;
       iok=-1;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decision made
       S->vflag=0;

       //S->hd.format=0; // must be initialize before 1st time call!!

       hd.bbuf=S->hd.bhead;

       memset(S->hd.bhead, 0, 32);  // clear S->hd.head32 array with 0
       /* read 32bit data 1st*/
       S->bytesread = fread( S->hd.bhead, sizeof(char), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 ){
           if(S->bytesread==0){
               printf("checkheader: File EOF! (%s)\n",S->fname);
           } else {
               printf("checkheader: File Read Error! (%s)\n",S->fname);
           }
	       i=fclose( S->stream ) ;
           S->open_flag=0;
		   i32=-1;
           goto exit_end;
	   }

loop1:
       /* read 32 bit data and check sync patter by step-by-step */
       S->bytesread = fread( S->hd.bhead+4, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 ){
           if(S->bytesread==0){
               printf("checkheader: File EOF! (%s)\n",S->fname);
           } else {
               printf("checkheader: File Read Error! (%s)\n",S->fname);
           }
	       i=fclose( S->stream ) ;
           S->open_flag=0;
		   i32=-1;
           goto exit_end;
	   }
       /*  debug */
       /*
       printf("%02X%02X%02X%02X %02X\n",
           S->hd.bhead[0],S->hd.bhead[1],S->hd.bhead[2],S->hd.bhead[3],S->hd.bhead[7]);
       printf("%02X%02X%02X%02X %02X\n",
           hd.bbuf[0],hd.bbuf[1],hd.bbuf[2],hd.bbuf[3],hd.bbuf[7]);
       printf("%08X %02X\n",hd.hd->syncptn1,hd.hd->syncptn2);
       printf("totalsec0 %d %d\n",hd.hd->totalsec0, S->hd.bhead[5]*256+S->hd.bhead[4]);
       printf("AD SAMPLING %d %d %02X\n",hd.hd->bitwidth,hd.hd->fsample, S->hd.bhead[6]);
       exit(0);
       */
       /* check sync pattern */
       if(S->hd.format==0){
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == 0x8B) S->hd.format=1;// K5/VSSP
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == 0x8C) S->hd.format=2;// K5/VSSP32
          if(S->hd.format !=0){
             iok=0;  // sync detected
             S->hd.psync2=(unsigned char)hd.hd->syncptn2; // save second sync pattern
          }
       } else {
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == S->hd.psync2) iok=0;
       }
       if(iok==0){
           S->hd.sfreq=fdat[hd.hd->fsample];
           S->hd.adbit=sdat[hd.hd->bitwidth];
           S->hd.numch=cdat[hd.hd->chmode];
           S->hd.seconds=hd.hd->totalsec1*65536+hd.hd->totalsec0;
           sec2hms(S->hd.seconds,&S->hd.hh,&S->hd.mm,&S->hd.ss);
           if(S->hd.format == 2){ // more header read
                /* 1st read 4 byte to get AUX field size info */
                S->bytesread = fread( S->hd.bhead+8, sizeof( char ), 4, S->stream );
                if( ferror( S->stream ) || S->bytesread==0 ){
                    if(S->bytesread==0){
                        printf("checkheader: File EOF! (%s)\n",S->fname);
                    } else {
                        printf("checkheader: File Read Error! (%s)\n",S->fname);
                    }
	                i=fclose( S->stream ) ;
                    S->open_flag=0;
		            i32=-1;
                    goto exit_end;
                }
                S->hd.aux_size=hd.hd->auxsize;
                S->hd.year=hd.hd->year+2000;
                S->hd.totalday=hd.hd->totalday;
                S->hd.v_major=hd.hd->v_major;
                S->hd.v_minor=hd.hd->v_minor;
                //printf("debug1 %d %d %d\n",S->hd.aux_size,S->hd.year,S->hd.totalday);
                //printf("debug2 %d %d %d\n",S->hd.v_major,S->hd.v_minor,S->bytesread);

                /* read AUX field */
                S->bytesread = fread( S->hd.bhead+12, sizeof( char ), S->hd.aux_size, S->stream );
                if( ferror( S->stream ) || S->bytesread==0 ){
                    if(S->bytesread==0){
                        printf("checkheader: File EOF! (%s)\n",S->fname);
                    } else {
                        printf("checkheader: File Read Error! (%s)\n",S->fname);
                    }
	                i=fclose( S->stream ) ;
                    S->open_flag=0;
		            i32=-1;
                    goto exit_end;
                }
                /* parse AUX field data */
                S->hd.aux_fmt=hd.hd->fmt;
                S->hd.filter =hd.hd->filter;
                if(S->hd.aux_fmt==1 || S->hd.aux_fmt==2){
                    for(n=0; n<8; n++) S->hd.aux_hostname[n]=hd.hd->hostname[n];
                    S->hd.aux_hostname[8]='\0';
                    if(S->hd.aux_fmt==1){
                       for(n=0; n<8; n++) S->hd.aux_station[n]=hd.hd->station[n];
                       S->hd.aux_station[8]='\0';
                       S->hd.aux_sid2[0]=hd.hd->sid2[0]; S->hd.aux_sid2[1]=hd.hd->sid2[1];
                       S->hd.aux_sid2[2]='\0';
                    } else {
                       S->hd.aux_station[0]='\0';
                       S->hd.aux_sid2[0]='\0';
                    }
                } else {
                    S->hd.aux_hostname[0]='\0';
                    S->hd.aux_station[0]='\0';
                    S->hd.aux_sid2[0]='\0';
                }
                //printf("debug3 fmt,filter %d %d\n",S->hd.aux_fmt,S->hd.filter);
                //printf("debug4 host station sid2 (%s) (%s) (%s)\n",S->hd.aux_hostname,S->hd.aux_station,S->hd.aux_sid2);
               
           }
           goto exit_loop;
       } 
       if(i32 ==0) {
           printf("checkheader: First 64bit is not header block!\n");
           printf("checkheader:   Data file name (%s)\n",S->fname);
           printf("checkheader:   searching header.....");
       }
       /* read next 32 bit data */
       for (n=0; n<4; n++) S->hd.bhead[n]=S->hd.bhead[n+4];
       i32++;
       goto loop1;

exit_loop:

       if(i32 <0) return i32;
       if (iok == 0){   // sync detection
          if(i32 !=0) printf("  found after %d bits read!\n",i32*32);
          if(S->hd.format==1)
             printf("checkheader: Header (K5/VSSP) Sync Detected!!\n");
          if(S->hd.format==2)
             printf("checkheader: Header (K5/VSSP32) Sync Detected!!\n");
		  printf("checkheader:   File : %s\n",S->fname);
          printf("checkheader:   A/D(bits) %d  CHs %d  SFreq(kHz) %d",
                    S->hd.adbit,S->hd.numch, S->hd.sfreq);
          printf("  Time %02d:%02d:%02d  sec %d\n",
                    S->hd.hh,S->hd.mm,S->hd.ss,S->hd.seconds);
          // following 4 lines are just for keep backword compatibility
          // should be obsoleted 2006-01-03
          S->adbit=S->hd.adbit;
          S->numch=S->hd.numch;
          S->sfreq=S->hd.sfreq;
          S->hh=S->hd.hh; S->mm=S->hd.mm; S->ss=S->hd.ss; S->seconds=S->hd.seconds;
	   }
exit_end:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->hd.seconds;
       //printf("checkheader:   vflag     dtime_vflag     %d %d\n",S->vflag,S->dtime_vflag);
       //printf("checkheader:   vflag_old dtime_vflag_old %d %d\n",S->vflag_old,S->dtime_vflag_old);
       return i32;
}


int fwdNsecHere(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
    //printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
                return -99;
            }
        }
        
        if(checkheader(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}


int conv4byte2int(unsigned char *b4)
{
    int i, k, iwork, iw2, ipick, flag, pflag;
    unsigned char iw;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    pflag=0;  // flag to change parity position   not used yet
    for(i=0; i<4; i++){
       iw=b4[i];
       iw2=(int)iw;
       for(k=0; k<8; k++){
           ipick=iw2 & 0x1;

           if(flag==0){
               iwork=iwork >> 1;
               ipick =ipick << 31;
               iwork= ipick | (iwork & 0x7FFFFFFF);
           } else {
               iwork=iwork << 1;
               ipick =ipick & 0x1;
               iwork= ipick | (iwork & 0xFFFFFFFE);
           }
           iw2=iw2 >> 1;
       }
    }
    return iwork;
}

int conv32bit2int(int *idat32)
{
    //int idat32[32];   // 32 bit data
    //int i4;   return 
    int i, iwork, ipick, flag;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    for(i=0; i<32; i++){
       ipick=idat32[i];
       if(flag==0){
           iwork=iwork >> 1;
           ipick =ipick << 31;
           iwork= ipick | (iwork & 0x7FFFFFFF);
       } else {
           iwork=iwork << 1;
           ipick =ipick & 0x1;
           iwork= ipick | (iwork & 0xFFFFFFFE);
       }
    }
    return iwork;
}


#endif
