/*******************************************************************/
/*                                                                 */
/*   Program Name :  one2four                                      */
/*                                                                 */
/*      Version:  1.00  2006-01-31                                 */
/*                                                                 */
/*      Copyright (c) 2004-2006 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*
 ho|ukah{[hP[hŎ擾S̃f[^t@C
 ̂S[h̃f[^t@C쐬

          T.Kondo 2004.11.15
               Ver 0.00 2004.11.15
               Ver 0.01 2005.06.12  update due to update in common header name
               Ver 1.00 2006.01.30  support K5/VSSP32 format data

    点 one2four file1 file2 file3 file4 [outfile]
         file1 -- jTf[^t@C (CH1)
               file2 -- jTf[^t@C (CH2) fBNgȂ
                                                   Ƃfile1ƓfBNg
               file3 -- jTf[^t@C (CH3)      V
               file4 -- jTf[^t@C (CH4)      V
               outfile -- f[^o̓t@C
                           ftHgfile1+'.K5m'  (m refers to merge)
                                 fBNgȂ
                                 Ƃfile1ƓfBNg
               secoffset -- X^[g̃ItZbgb
                                 ftHg͂O
               span  ----  f[^ϊ̋ԁibjiΉj
                             ftHgׂ͂


*/
#define ONE2FOUR_VERSION "Ver. 2006-01-31"

#include "libk5.h"      // common header for IP-VLBI utilities
#include "libvssp32b.h"      // functions and type to handle K5/VSSP32 (#2)

/*    Offset from dB to dBm conversion from A/D multibit sampling data */ 
#define OFFSET1  16.0     //1 bit sampling full power convert to 0 dB (theoretical)
      /* following constants are provisional  (measured on 2002/5/18) */
#define OFFSET2   9.0     //2 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET4  -1.0     //4 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET8 -26.0     //8 bit sampling data offset measured at 200kHz for PC(vlbi11)


void parse_dir_and_name(char *fpath, char *dir, char *fname) 
{
    int len,i,k;
    char cbuf[128];
    len=strlen(fpath);
    // converting '\' to '/' if so
    for(i=0; i<len; i++){
       if(fpath[i]=='\\'){
            cbuf[i]='/';
       } else {
            cbuf[i]=fpath[i];
       }
    }
    cbuf[len]='\0';
    k=-1;
    for (i=len-1; i>=0; i--){
        if(cbuf[i] == '/') {
            k=i;
            break;
        }
    }
    //printf("debug %d %s\n",k,cbuf);
    if(k<0){
        dir[0]='\0';
        strcpy(fname,cbuf);
    } else if (k == len-1){
        strcpy(dir,cbuf);
        fname[0]='\0';
    } else {
        strncpy(dir,cbuf,k+1);
        dir[k+1]='\0';
        strcpy(fname,cbuf+k+1);
    }
    //printf("debug %d %s\n",k,dir);
}


int main(int argc, char *argv[])
{
       int sfreq, adbit, numch, kpos;
       int startoffset[4]; // start time offset in sec
       long int counts;        //sample counter
       long int counts2;        //extract sample counter
       //long int sofst_pos;    // start sample position from sec tic
       int nloop;
       int i,imax, m, k,n;
       int smode,usampl;
       int obuf[1024];
       int nwrite=1024;
       int iwrite;
       int idat32[32];
       int idat,ipick;
       int ibcount;
       //int opos, nrepeat;
       int opos;
       //unsigned char *ibuf;
       int nbyte=5000;
       //fpos_t pos;
       int bytesread;
       int bits, bytes, numb;
       //int nch;
       int k5num;
       int span,secc;
       int iflag,secmax,secmin;
       int hbytes; // header bytes
       int nerr;
       double fsampl,tsampl,fact;
       //int *idata;
       //double soffset, fsec;
       double soffset;

       char oname[80],cbuf[80];
       char lab2[60], ltime1st[60];
       char labfsampl[10];
       char dflt_oname[80];
       char dir0[80],dir[80],fname[80];
       ip_vlbi_file_t X[4];
       FILE *fo;
       IPBOARD_T ipb;

       if(argc == 1){
          printf("one2four  %s\n",ONE2FOUR_VERSION);
          printf("Usage: one2four file1 file2 file3 file4 [outfile]]\n");
          printf("\n");
          printf("         where  file1 ---- K5 file name for CH 1\n");
          printf("                file2 ---- K5 file name for CH 2\n");
          printf("                file3 ---- K5 file name for CH 3\n");
          printf("                file4 ---- K5 file name for CH 4\n");
          printf("                outfile ---- Merge data outfile name\n");
          printf("                             default is filename+.k5m\n");         
          printf("                secofst ---- Merge start time offset (sec)\n");
          printf("                             default is 0\n");         
          printf("                span    ---- Merge span (sec)\n");
          printf("                             default is all\n");         
          exit(0);
       }

       /* data file name get */

       for(n=0; n<4; n++) 

       k5num=0;
       n=2;

       while(argc >= n){
          strcpy(cbuf,*(argv+ n-1) );
          if(k5num >= 4) {  // OUT file name
              strcpy(dflt_oname,cbuf);
              break;
          } else {
              X[k5num].fname = (char *)malloc(80);  
              strcpy(X[k5num].fname,cbuf);
              if(k5num==0){ 
                 /* default oname create */
                 sprintf(dflt_oname,"%s.k5m\0",cbuf);
              }
              k5num++;
          }
          n++;
       }
       /* Out file name get */
       strcpy(oname,dflt_oname);

       n++;
       /* start offset get */
       if (argc>=n)
       {
          soffset = atof(*(argv + n-1 ));
       } else {
          soffset =0.0;   
       }
       n++;

       /* span get */
       if (argc>=n)
       {
          span = atoi(*(argv + n-1 ));
       } else {
          span =0;   
       }

       // file2 , file3, file4 outfile if dir is omitted, dir of file1
       //  is supposed


       parse_dir_and_name(X[0].fname, dir0, fname);
       if(dir0[0]!='\0'){
           for(n=1; n<k5num; n++){
              parse_dir_and_name(X[n].fname, dir, fname);
              if(dir[0]=='\0'){
                  strcpy(X[n].fname,dir0);
                  strcat(X[n].fname,fname);
              }
           }
       }
       parse_dir_and_name(oname, dir, fname);
       if(dir[0]=='\0'){
            strcpy(oname,dir0);
            strcat(oname,fname);
       }

       /* Run condition monitor for debug*/
       for(n=0; n<k5num; n++){
           printf("K5 FILE %d is %s\n",n+1,X[n].fname);
       }
       printf("Out File is %s\n",oname);
       printf("soffst  %f\n",soffset);
       printf("span    %d\n",span);

       /* Program Name Display */
       printf("*******************************************************\n");
       printf("* K5 1CH DATA FILE MERGE PROGRAM (one2four)           *\n");
       printf("*         %s    by  T.KONDO/NICT      *\n",ONE2FOUR_VERSION);
       printf("*******************************************************\n");
       printf("\n");

       /* Data file open */

       for(n=0; n<k5num; n++){ 
           if( (X[n].stream  = fopen( X[n].fname, "rb" )) == NULL ){
              printf("Data File %s   Open Error! \n",X[n].fname);
              goto exitend2;
           }
           /* Data header check to get sampling parameters */
           X[n].hd.format=0;  // for 1st call  2006-01-03
           if(checkheader(&X[n]) < 0){
                goto exitend3;
           }
       }


       /* check whether all K5 data has same sampling parameters and same format*/
       iflag=0;
       for(n=0; n<k5num-1; n++){
            for(k=n+1; k<k5num; k++){
                if(X[n].hd.sfreq != X[k].hd.sfreq) iflag=1;
                if(X[n].hd.adbit != X[k].hd.adbit) iflag=1;
                if(X[n].hd.numch != X[k].hd.numch) iflag=1;
                if(X[n].hd.format!= X[k].hd.format) iflag=1;
            }
       }
       if(iflag == 1){
           printf("Sampling parameters or data format are not same among K5 files!\n");
           for(n=0; n<k5num-1; n++){
               printf("FILE %d ",n+1);
               if(X[n].hd.format == 1){
                  printf(" (K5/VSSP format)");
               } else if (X[n].hd.format == 2){
                  printf(" (K5/VSSP32 format)");
               } else {
                  printf("()");
               }
               printf(" sfreq= %d  adbit= %d  numch= %d\n",
                    X[n].sfreq,X[n].adbit,X[n].numch);
               i=fclose( X[n].stream ) ;
           }
           goto exitend2;
       }

       if(X[0].numch != 1){  // this is not 1ch mode file
           printf("K5 file is not 1 ch mode!\n");
           for(n=0; n<k5num-1; n++){
               i=fclose( X[n].stream ) ;
           }
           goto exitend2;
       }

       /* output file create */
       if( (fo  = fopen( oname, "wb" )) == NULL )
       {
          printf("File  %s  Open Error! \n",oname);
          goto exitend3;
       } 
      
        sfreq=X[0].hd.sfreq;
        adbit=X[0].hd.adbit;
        numch=X[0].hd.numch;
        fsampl=1000.0*(float)sfreq;   // sampling frequency (Hz)
        tsampl=1.0/fsampl;            // sampling period (sec)
        bits=((long)fsampl)*adbit*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X[0].fsample=(float)fsampl;

        //sofst_pos=0;
        //nrepeat=32/adbit;


        /* start time check */
        secmax=X[0].seconds;
        secmin=X[0].seconds;
        for(n=1; n<k5num; n++){
            if(X[n].seconds > secmax) secmax=X[n].seconds;
            if(X[n].seconds < secmin) secmin=X[n].seconds;
        }

        /* offset setup */
        for(n=0; n<k5num; n++) startoffset[n]=0;
        secmin=X[0].seconds;
        if(secmax != secmin) { // start offset set
            for(n=0; n<k5num; n++){
                startoffset[n]=secmax-X[n].seconds;
            }
        }

        for(n=0; n<k5num; n++){
          startoffset[n]+=(int)soffset;  // another offset plus
        }

        /* header parameters setup for output */
        ipb.adbit=adbit;
        ipb.numch=4;   // compulsory set to 4ch mode
        ipb.sfreq=sfreq;
        ipb.seconds=secmax;
        ip_hd_make_ini(&ipb,X[0].hd.psync2);

        /* File read buffer size set */
        smode=1;                         // low resolution mode

        if(smode ==1){ usampl=10000; }
        else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples

        //printf("imax numb  %d %d\n",imax,numb);    //debug


        for(n=0; n<k5num; n++){
           X[n].ibuf = (unsigned char *)malloc(numb);
           X[n].idata = (int *)malloc(4*usampl*sizeof(int));
        }

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq < 1000){
            sprintf(labfsampl,"%dkHz\0",sfreq);
        } else {
            sprintf(labfsampl,"%dMHz\0",sfreq/1000);
        }

       /* Data extract start point set */ 


       //fsec=soffset-(double)startoffset;
       //sofst_pos=(long int)(fsec/tsampl + 0.01);

       /* Data extraction period set */
       
    //   smpl_count=(long int)(period/tsampl + 0.01);

       /* Run condition display */

       //printf("startoffset, sofst_pos, smpl_count %d %d %d\n",startoffset,sofst_pos,smpl_count);
       printf("\n");
       for(n=0; n<k5num; n++){
           printf("  Data File %d : %s",n+1,X[n].fname);
           if(X[n].hd.format == 1){
              printf(" (K5/VSSP format)\n");
           } else if (X[n].hd.format == 2){
              printf(" (K5/VSSP32 format)\n");
           } else {
              printf("\n");
           }
       }
       printf("  Sampling Freq : %s\n",labfsampl);
       printf("  A/D bits      : %d\n",adbit);
       printf("  Number of CHs : %d\n",numch);

       /* foward startoffset seconds */
       for(n=0; n<k5num; n++){
           if((nerr=fwdNsecHere(X[n].ibuf, startoffset[n], numb, imax, &X[n])) <0){
               if(nerr == -99) goto exitend;
           }
       }
       printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
           X[0].hh,X[0].mm,X[0].ss,X[0].seconds);
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",X[0].hh,X[0].mm,X[0].ss);
       strcpy(lab2,ltime1st);
        
       /* continue run info display */
       printf("  Start Time of Merge : %02d:%02d:%02d\n",
           X[0].hh,X[0].mm,X[0].ss);
     //  printf("  Period (sec)             : %f\n",period);
       printf("  Out File : %s\n",oname);

       /* header data update */

       hbytes=8;
       if(X[0].hd.format == 2){  // for K5/VSSP32
            for(n=0; n<4; n++) ipb.hd[n+8]=X[0].hd.bhead[n+8];
            for(n=0; n<X[0].hd.aux_size; n++) ipb.hd[n+12]=X[0].hd.bhead[n+12];
            hbytes += (4+X[0].hd.aux_size);
       }

       ipb.seconds=X[0].seconds;
       ip_hd_make(&ipb);


       /* Data info out to out_file */
//       if(omode == 0) {
//          fprintf(fo,"**  Data File : %s\n",fname);
//          fprintf(fo,"**  Sampling Freq : %s\n",labfsampl);
//          fprintf(fo,"**  A/D bits      : %d\n",adbit);
//          fprintf(fo,"**  Number of CHs : %d\n",numch);
//          fprintf(fo,"**  Extracted CH# : %d\n",nch);
//          fprintf(fo,"**  Start Time of Data : %02d:%02d:%02d.%08d\n",X.hh,X.mm,X.ss,(long int)(fsec*1.0e8));
//         fprintf(fo,"**  Period (sec)       : %f\n",period);
//      }
       
       /* Data extraction start */
       counts=0;
       counts2=0;
       secc=0;
       //m=nch-1;
       nloop=0;
       ibcount=0;

       /* convert header byte data to integer data */
       opos=0;
       for(n=0; n < hbytes/4; n++){
          obuf[opos]=conv4byte2int(ipb.hd+n*4);
          opos++;
       }

       /* Data read loop start here */

loop1:
        if(nloop >0) {
           for(n=0; n<k5num; n++){
               if(checkheader(&X[n]) < 0){ goto exitend; }
             //   printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
           }
           if(span != 0 && secc > span) goto exitend;
           secc++;
           /* header data update */
           ipb.seconds=X[0].seconds;
           ip_hd_make(&ipb);
           /* put K5 header data into obuf */
           for(n=0; n < hbytes/4; n++){
              obuf[opos]=conv4byte2int(ipb.hd+n*4);
              opos++;
              if(opos == nwrite){
                 //printf("Debug %d\n",nloop);
                 iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                 opos=0;
              }
           }
        }

        for(i=0; i< imax; i++)
        {

             for(n=0; n<k5num; n++){
                 bytesread = fread( X[n].ibuf, sizeof( char ), numb, X[n].stream );
                 //printf("bytes read 2 %d\n",bytesread); //debug
                 if( ferror( X[n].stream ) || bytesread==0 ) goto exithere; 
                 //parseADdata_2(X[n].ibuf,bytesread,adbit,X[n].idata, &kpos);
                 parseADdata_3(X[n].ibuf,bytesread,adbit,X[n].idata, &kpos);
             }
             //printf("adbit= %d\n",adbit);
             for(k=0; k<kpos/numch; k++)  // only numch = 1 is supported now 
             {
                counts++;

                for(n=0; n<k5num; n++){
                   //idat=X[n].idata[k*numch+nch-1];
                   idat=X[n].idata[k*numch];

                   for(m=0; m< adbit; m++){
                      ipick=idat & 0x1;
                      //printf(" %d",ipick);
                      idat32[ibcount]=ipick;
                      ibcount++;
                      idat=idat >> 1;
                   } 
                   if(ibcount==32){
                       obuf[opos]=conv32bit2int(idat32);
                       opos++;
                       if(opos == nwrite){
                          //printf("Debug %d\n",nloop);
                          iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                          opos=0;
                       }
                       ibcount=0;
                   }
                }
             }
       }
       nloop++;
       goto loop1;
exithere:
       printf("\n");
       printf("Four 1-ch data to one 4-ch data conversion finished.\n");

       i=fclose( fo ) ;

exitend: 
       
       for(n=0; n<k5num-1; n++){
          free(X[n].ibuf);
          free(X[n].idata);
          free(X[n].fname);
       }

exitend3: 
       for(n=0; n<k5num-1; n++){
          i=fclose( X[n].stream ) ;
       }

 exitend2: 

       return 0;
}
