/************************************************************************/
/*                                                                      */
/*   Program Name :  pctimeset                                          */
/*                                                                      */
/*      Version:  1.00  2003-06-18                                      */
/*                1.01  2003-07-20  add env variable monitor            */
/*                1.02  2005-06-12  minor change                        */
/*                2.00  2005-12-29  support VSSP32                      */
/*                                                                      */
/*      Copyright (c) 2003-2005 T.Kondo/NICT All Right Reserved         */
/*                                                                      */
/************************************************************************/
/*
/*
     PC Time Set Program  
     
     Set PC time using IP-VLBI (K5/VSSP or K5/VSSP32) board time with log out.

      How to use
         pctimeset


*/
#define PCTIMESET_VERSION "Ver. 2005-12-29"
#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi.h"  // header file for IP-VLBI utilitie functions
#include "libipvlbi_board.h"  // header file for IP-VLBI sampler board functions

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int y2k(int yyyy, int mm, int dd);

int y2k(int yyyy, int mm, int dd)
{
  int i, sum = 0;
  int month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30 , 31, 30, 31};

  if(((yyyy % 4 == 0) && (yyyy % 100 != 0)) || (yyyy % 400 == 0))
    month[1] = 29;
  for (i=0; i<mm-1; i++) {
    sum += month[i];
  }
  sum += dd;

  return sum;
}

int main(int argc, char **argv)
{
    //char dev[] = "/dev/tds0";  // IP-VLBI BOARD control variable
    int fd_in,rv;
    unsigned int time,day,sec; 
    unsigned int secold, stat;
    int year,ddd,hh,mm,ss, mday, month ;
    int  h, m, s;
    int counter;
    int  y3; // 7 bits year  (0-127)
    int yyyy;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[80];
    float tofset, tofset1, fsec;
    double t1;
    struct timeval tv;
    struct timezone tz;

    printf("pctimeset %s\n",PCTIMESET_VERSION);
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"pctimeset(%s)",PCTIMESET_VERSION);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(DEV_FILE, O_RDONLY);  // device open
    if (fd_in == -1) {
        perror("device open() error ");
        sprintf(cbuf,"device open() error");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    
    /* board status check */
    if((rv=check_vssp(fd_in, &stat, 1)) == -1) {
        perror("ioctl() status get failed ");
        goto end;
    }
    
    /* get time from board */
    secold=100000;
    counter=0;

    do{
#ifdef _VSSP32
          rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
          yyyy=y3+2000;
#endif
          rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
          if (rv == -1){
              perror("ioctl() time get failed ");
              return -1;
          }
          day=TDS_GET_DAYS(time);
          sec=TDS_GET_SEC(time);
          if (sec!=secold){      // to catch exact second tic
              sec2hms(sec,&h,&m,&s);
              //printf("%03d %02d:%02d:%02d\n",day,h,m,s);
              if (counter >=1 ) goto exith1 ;
              counter++; 
          }
          secold=sec;
    } while(counter <=10);
    exith1:
        
    tofset=0.0;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    tofset=(float)sec-((float)(hh*3600+mm*60+ss)+fsec); 
    //printf("System Time : %04d/%02d/%02d %02d:%02d:%02d\n",year,month,mday,hh,mm,ss);  //debug
    //printf("Offset %f\n",tofset);  //debug
    //printf("%d  %d\n",(long)tofset,(long)(fmod(tofset,1.0)*1.0e6));  //debug
    tofset1=tofset;

    /* set PC time */
    gettimeofday(&tv,&tz);
    //printf("tz.tz_minuteswest=%d tz.tz_dsttime=%d\n",tz.tz_minuteswest,tz.tz_dsttime); // debug
    //printf("tv.tv_sec=%d tv.tv_usec=%d\n",tv.tv_sec,tv.tv_usec);   //debug
    t1=(double)tv.tv_sec+(double)tv.tv_usec/1.0e6;
    //printf("%.6f\n",t1);
    t1+=(double)tofset;   
    tv.tv_sec=(long)t1;
    tv.tv_usec=(long)(fmod(t1,1.0)*1.0e6);
    //printf("%d %d\n",tv.tv_sec,tv.tv_usec);
    if(settimeofday(&tv,&tz)==-1){
        printf("time set error! %d\n",errno);
        if(errno==EPERM) printf("Only super-user can issue this command!!\n");
        // log out
        printf("Current Offset (IP-Board time - PC time) is %f sec\n",tofset);
        sprintf(cbuf,"pctimeset failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode
        sprintf(cbuf,"Current Offset (IP-Board time - PC time) is %f sec",tofset);
        if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode
        close(fd_in);
        goto end;
    }

    secold=100000;
    counter=0;

    do{
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1){
              perror("ioctl() time get failed ");
              return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        if (sec!=secold){        // to catch exact second tic
            sec2hms(sec,&h,&m,&s);
            //printf("%03d %02d:%02d:%02d\n",day,h,m,s);
            if (counter >=1 ) { goto exith2 ;}
            counter++; 
        }
        secold=sec;
    } while(counter <=10);
    exith2:
    tofset=0.0;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    tofset=(float)sec-((float)(hh*3600+mm*60+ss)+fsec); 
    //iofset=sec-(hh*3600+mm*60+ss);
    printf("System Time : %04d/%02d/%02d %02d:%02d:%02d\n",year,month,mday,hh,mm,ss);
    printf("Offset (IP-Board time - PC time)  %f sec was adjusted\n",tofset1);
    printf("Current Offset (IP-Board time - PC time) is %f sec\n",tofset);
    sprintf(cbuf,"System Time : %04d/%02d/%02d %02d:%02d:%02d",year,month,mday,hh,mm,ss);
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode
    sprintf(cbuf,"Offset (IP-Board time - PC time)  %f sec was adjusted!",tofset1);
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode
    sprintf(cbuf,"Current Offset (IP-Board time - PC time) is %f sec",tofset);
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode
        
    close(fd_in);
	
    
    // log out
    sprintf(cbuf,"pctimeset completed");
    //if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");  //PC time mode

    printf("\n");
end:
    return 0;
}
