/*********************************************************************/
/*                                                                   */
/*   Program Name :  signalcheck                                     */
/*                                                                   */
/*      Version:  0.00  2002-10-04                                   */
/*                0.10  2002-10-04                                   */
/*                1.13  2005-09-28   support linux2                  */
/*                2.00  2005-12-20  support VSSP32                   */
/*                                                                   */
/*      Copyright (c) 2002-2005  T.Kondo/NICT All Right Reserved     */
/*                                                                   */
/*********************************************************************/
/*
/*
     Signal check program 
        gp@  signalcheck

               IP-VLBI{[hɓK؂ȃx10MHz,1PPS
               ڑĂ邩ǂ̃`FbNs

*/
#include "chead.h"      // common header for IP-VLBI utilities
#ifdef linux
  #include <stdint.h>  // necessary for handling uint32_t etc.
  #include <sys/ioctl.h>
  #ifdef _VSSP32
    #include <sys/tdsio.h>
    #include <sys/tdssdh.h>
  #else
    #include <linux/tdsio.h>
  #endif
#else 
  #include <sys/tdsio.h>  // IP-VLBI board header
#endif

/* device driver file set  (added on 2005-12-20) */
#ifdef _VSSP32
  #define DEV_FILE "/dev/utds0"     // device file for K5/VSSP32
#else
  #define DEV_FILE "/dev/tds0"      // device file for K5/VSSP
#endif


unsigned int board_ref_check(int fd_in)
{
    /* board reference input signal check */
/*
    [IN}
       fd_in  :  device file directive (must be opened in advance)
    [return]
       reference signal status
*/
    int rv;
    unsigned int stat;
    unsigned int ref, pps, rhz, pll;
    stat=0;
    /* board status check */
    if((rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat))==-1){
        perror("   ioctl() status get failed ");
        goto end;
    }
    /* satatus get again just in case */
    if((rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat))==-1){
        perror("   ioctl() status get failed ");
        goto end;
    }
    //printf("ioctl() status get succeed\n");
    //printf(" status = %X\n",stat);   // debug
#ifdef _VSSP32
    // separate command
    //stat|=0x8;  // means this is VSSP32
    //ioctl(fd_in, TDSIO_GET_REFINPUT, &ref);
    //if(ref==1) stat|=1;
    //ioctl(fd_in, TDSIO_GET_10MHZINPUT, &rhz);
    //if(rhz==1) stat|=0x4;
    //ioctl(fd_in, TDSIO_GET_1PPSINPUT, &pps);
    //if(pps==1) stat|=0x2;
    
    /* set stat again to keep compatibility with K5/VSSP */
    ref=0; rhz=0; pps=0; pll=0;
    if((stat & 4) == 4) pps=1;        // 1PPS supplied
    if((stat & 8) == 8) ref=1;        // reference signal supplied
    if((stat & 0x20) == 0x20) rhz=1;  // reference signal is 5MHz
    if((stat & 0x10) == 0x10) pll=1;  // PLL locked
    stat=0;
    stat|=0x10;  // means this is VSSP32
    if(ref==1) stat|=1;
    if(pps==1) stat|=0x2;
    if(rhz==1) stat|=0x4;
    if(pll==1) stat|=0x8;
#else
    stat&=3;  // 2005-12-20
#endif
end:
    return stat;
}


int main(int argc, char **argv)
{
    //char dev[] = "/dev/tds0";
    char dev[] = DEV_FILE;
    int fd_in,rv;
    unsigned int stat;
    unsigned int ref, pps, rhz;
    
    
#ifdef _VSSP32
    printf("\n  K5/VSSP32 (IP-VLBI) board Reference and 1PPS signal checking..\n\n");
#else
    printf("\n  K5/VSSP (IP-VLBI) board 10MHz and 1PPS signal checking..\n\n");
#endif
    fd_in = open(dev, O_RDONLY);  //foCX̃I[v
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    //printf("         Device open successd No.[%d]\n",fd_in);

    /* board reference signal check */
    stat=board_ref_check(fd_in);
    if((stat & 0x10) == 0x10){ // this is VSSP32
        if((stat & 1) != 1){
            printf("   Warning! 10MHz or 5MHz signal is not supplied. \n");
        } else {
            if((stat & 4) == 4){
                printf("   Reference (5MHz) signal is OK!");
            } else {
                printf("   Reference (10MHz) signal is OK!");
            }
            if((stat & 8) == 8){
                printf("   PLL locked!\n");
            } else {
                printf("   Warning! PLL unlocked!\n");
            }
        }
    } else {   // this is VSSP
        if((stat & 1) != 1){
            printf("   Warning! 10MHz signal is not supplied. \n");
        } else {
            printf("   10MHz signal is OK!\n");
        }
    }
    if((stat & 2) != 2){
        printf("   Warning! 1PPS signal is not supplied. \n");
    } else {
        printf("   1PPS  signal is OK!\n");
    }
    if((stat & 3) == 0 || (stat & 3) == 2){
       printf("   Check signal connection and try again!\n");
       goto end;
    }
end:
    printf("\n");
    close(fd_in);

}
